#include "objectdomain.h"
#include "../../application/implementation/glclient.h"

#include <string>
#include <trace.h>


namespace DOMAIN 
{
	CObjectDomain::~CObjectDomain()
	{
		serverClientInfo->ReleaseRef();
		delete actionObj;
	}

	void CObjectDomain::Init()
	{
//		fe = ((APPLICATION::CGFXApp*)APPLICATION::CApplication::GetApplication())->GetFrontEnd();

		CChatDomain::Init();
		network = APPLICATION::CApplication::GetApplication()->GetNetwork();

		//if (!objMan) objMan = new CSimpleObjectManager(this); //(CSimpleObjectManager*)APPLICATION::CApplication::GetApplication()->GetObjectMan();

	/*	CAnimatedCharacterObject *co;
		co = new CAnimatedCharacterObject(
						25, 1002, 3, 
						GENERAL::CVector3f(0.0f,32.0f,-150.0f),
						GENERAL::CVector3f(0.0f,0.0f,0.0f),
						GENERAL::CVector3f(1.0f,1.0f,1.0f),
						1.0f, 1.0f, 1.0f, 10.0f, 0, 0,
						GENERAL::CVector3f(0.0f,0.0f,0.0f)
						);

		objMan->Add(co);
		objMan->SetVisible((CGraphObject*)co);*/
		

	
	}

	void CObjectDomain::updateObject(GENERAL::CBuffer &buffer, CGameObject *go, NETWORK::PClientInfoList from)
	{
		CGameObject::TSerializedData *sd=(CGameObject::TSerializedData*)buffer.Peek(sizeof(CGameObject::TSerializedData));
		if (sd->who!=GetControllingObj()->GetID())	
			go->DeSerialize(buffer);
	}

	void CObjectDomain::HandleDomainCommand(sptr<DOMAIN::CDomainCommand> cmd)
	{
		switch (cmd->GetCommand())
		{
		case 1: // Use object as Controlling Object
			{
				long iod=*((long*)cmd->GetParam());
				cameraObj = controllingObj = objMan->GetRef(iod);
				controllingObj->SetPDU();
				controllingObj->SetControl();
				((APPLICATION::CGFXApp*)APPLICATION::CApplication::GetApplication())->GetFrontEnd()->SetCamera(cameraObj);
			}
			break;
		case 2: // ping request
			{
				struct TPingReplyData {
					GENERAL::TGameTick servertick;
					GENERAL::TGameTick owntick;
				} *prd = (TPingReplyData*)malloc(sizeof(TPingReplyData));

				prd->servertick = *(GENERAL::TGameTick*)cmd->GetParam();
					prd->owntick = APPLICATION::CApplication::GetApplication()->GetTicker().getTick();
				cmd->SetCommand(3);
				free(cmd->GetParam());
				cmd->SetParamLen(sizeof(TPingReplyData));
				cmd->SetParam(prd);

				APPLICATION::CApplication::GetApplication()->GetNetwork()->PushObject(cmd,cmd->GetSendToClientList());	
			}
			break;
		case 4: // Timesynch
			{
				struct TSynchTimeData {
					GENERAL::TGameTick servertick;
					GENERAL::TGameTick avgpingtime;
				} *std = (TSynchTimeData*)cmd->GetParam();

				GENERAL::TGameTick change = APPLICATION::CApplication::GetApplication()->GetTicker().setTick(std->servertick+(std->avgpingtime>>1));
				dprintf("**************** Time synch ****************\n");
				dprintf("      delta       %ld\n",change);
				dprintf("      tickTime    %ld\n",std->servertick+(std->avgpingtime>>1));
				dprintf("      servertime  %ld\n",std->servertick);
				dprintf("      acgpingtime %ld\n",std->avgpingtime);
				dprintf("      currenttime %ld\n",APPLICATION::CApplication::GetApplication()->GetTicker().getTick());

// perform automatic login (user/pass "c1"/"c1")
				static bool isFirstSynch=true;

				if (isFirstSynch)
				{
					CLoginData *loginData = new CLoginData("c1","c1");
					loginData->SetSendToClientList(cmd->GetSendToClientList());
					APPLICATION::CApplication::GetApplication()->GetNetwork()->PushObject(loginData,cmd->GetSendToClientList());
//1					loginData->ReleaseRef();

					isFirstSynch=false;
				}
			}
			break;

		case DOMAINCMD_PDU:
			{
				sptr<CGameObject> go = objMan->GetRef(cmd->GetID());
				
				if (!go->IsControled())
				{
					if (go->GetPDUSize()==cmd->GetParamLen())
					{
						go->CopyPDUfrom((char*)cmd->GetParam());
					}
				}
			}
			break;
		case DOMAINCMD_OBJECTACTION_REQUEST:
			{
				TDomainObjectAction *act = (TDomainObjectAction*)cmd->GetParam();
				sptr<CGameObject> go = objMan->GetRef(act->sourceObj);
				if (go!=NULL)
				{
					go->HandleDomainObjectActionRequest(cmd);
				} 
				else
				{
					/// @todo handle error
				}
			}
			break;
		case DOMAINCMD_OBJECTACTION_REPLY:
			{
				TDomainObjectAction *act = (TDomainObjectAction*)cmd->GetParam();
				sptr<CGameObject> go = objMan->GetRef(act->sourceObj);
				if (go!=NULL)
				{
					go->HandleDomainObjectActionReply(cmd);
				} 
				else
				{
					/// @todo handle error
				}
			}
			break;
		case DOMAINCMD_REQUEST_OBJECT_REPLY:
			{
				sptr<CDomainCommand> request = objMan->GetPtr(cmd->GetID());
				objMan->Release(request);
				sptr<CDomainCommand> peddingCmd = objMan->GetPtr(((CDomainCommand*)request->GetUserdata())->GetID());
				HandleDomainCommand(peddingCmd);
			}
			break;
		}
	}


	void CObjectDomain::handleNetworkObject(sptr<CSerializeableObject> &so)
	{
		// override auth from CChatDomain
		NETWORK::TClientInfoList::iterator it;
		if (so->GetClassKey()==GET_KEY(NETWORK::CNetworkEvent))
		{
			sptr<NETWORK::CNetworkEvent> ne=so;

			switch (ne->GetTyp())
			{
			case NETWORK::CNetworkEvent::NEWCONNECTION:
				{
					serverClientInfo->push_back(ne->GetClientInfo());

					std::string newline="application:Connected to ";
					newline.append(ne->GetClientInfo()->GetClientInfoString());

					consoleData.lines.push_front(newline);
					consoleData.updated=true;
//1
				}
				break;
			default:
				CChatDomain::handleNetworkObject(so);
				break;
			}
		} else if (ISOFTYPE(so,DOMAIN::CDomainCommand))
		{
			HandleDomainCommand(so);
//1			so->ReleaseRef();
		} else if (ISOFTYPE(so,CGameObject))
		{

			sptr<CGameObject> go = so;

			sptr<CGameObject> goOM = objMan->GetRef(go->GetID());
			if (goOM==NULL) 
			{
				if (go->GetID()==50)
				{
					int blu=27;
				}
				go->SetPDU();
				objMan->Add(go);
				

//				consoleData.lines.push_front(std::string("application:CGameObject Added"));

				if (ISOFTYPE(so,CGraphObject))
				{

					long parentid = ((CGraphObject*)&go)->GetParentId();
					if (parentid>0)
					{
						sptr<CGraphObject> god = objMan->GetRef(parentid);
						if (god!=NULL)
						{
							god->AddChild(go);
						}
						else
						{
							DOMAINTHROW("Received Object and Parent doesnt exist!!");
						}
					} else {
						if (ISOFTYPE(so,CLevel))
						{
							GetRootObject()->AddChild(go);
						}
					}
//					else GetRootObject()->AddChild(go);

					if (!((CGraphObject*)&so)->Loaded())
					{
						APPLICATION::CApplication::GetApplication()->GetLoadingthread().LoadGameObject((CGraphObject*)&so);
					} else 
					{
						((CGraphObject*)&go)->Load();
					}

					if (ISOFTYPE(so,CLevelObject))
					{
						long levelid=((CLevelObject*)&so)->GetLvlId();
						((CLevelObject*)&so)->SetLvl((CLevel*)&objMan->GetRef(levelid));
					}
				}
			} else {
				int bla=1;
//				goOM->Assign(go);

				//consoleData.lines.push_front(std::string("application:CGameObject Assigned"));
			}

		/*	if (ISOFTYPE(so,CTestObject))
			{
				char outstr[1024];
				sprintf(outstr,"x:%f, y:%f, r:%f, angle:%f, speed:%f\n",
					((CTestObject*)so)->x,
					((CTestObject*)so)->y,
					((CTestObject*)so)->r,
					((CTestObject*)so)->angle,
                    ((CTestObject*)so)->speed);
				OutputDebugString(outstr);
			}*/
//1			go->ReleaseRef();

		} else 
			CChatDomain::handleNetworkObject(so);
	}

	CDomain::PObjectList CObjectDomain::GetVisibleObjects()
	{
		if (APPLICATION::CApplication::GetApplication()->GetLoadingthread().IsLoading())
		{
			return NULL;
		} else {
			return NULL;
//3			return &GetObjMan()->GetVisibleList();
		}
	}

	void CObjectDomain::help(char *cmd)
	{
		if (cmd!=NULL)
		{
			if ( (strcmp(cmd,"help")==0) ||
				(strcmp(cmd,"?")==0) )
			{
				consoleData.lines.push_front("    /help  /?   get this text");
				consoleData.lines.push_front("    /login /l  login to server");
			} else
			if (strcmp(cmd,"login")==0)
			{
				consoleData.lines.push_front("application: /login <username> <password>");
				consoleData.lines.push_front("	           logs in to the server with the given username and password");
			} else
			if (strcmp(cmd,"l")==0)
			{
				consoleData.lines.push_front("application: /l <username> <password>");
				consoleData.lines.push_front("	           logs in to the server with the given username and password");
			}
		}
	}


	void CObjectDomain::execute()
	{
		CSimpleObjectManager::TObjectMap &objectList = GetObjMan()->GetAllList();
		CSimpleObjectManager::TObjectMap::iterator it;

		CChatDomain::execute();

		if (APPLICATION::CApplication::GetApplication()->GetLoadingthread().IsLoading())
			return;

		TGameTick currentTick;
		TGameTick deltaTick;

		currentTick = APPLICATION::CApplication::GetApplication()->GetTicker().getTick();
		deltaTick = APPLICATION::CApplication::GetApplication()->GetTicker().getDeltaTick();


		for (it=objectList.begin(); it!=objectList.end(); it++)
		{
			sptr<CGameObject> go=it->second;
	//		go->TimeStep(currentTick-it->second->GetTimeStamp());

//			if ( (go->id==6001) && (go->timestamp[0]==go->timestamp[1]) );
			go->TimeStep(deltaTick);
			if (go->IsControled() && (go->GetID()>0))
			{
				if (go->GetPDUTreshHold()>1.0f)
				{
//					dprintf("PDU sent!\n");
					CDomainCommand *doc=new CDomainCommand(this);
					doc->SetCommand(DOMAINCMD_PDU);
					doc->SetID(go->GetID());

					int pdusize = go->GetPDUSize();
					doc->SetParamLen(pdusize);
					char *data = (char*)malloc(pdusize);
					go->SetPDU();
					go->CopyPDUto(data);
					doc->SetParam(data);
	
					network->PushObject(doc,serverClientInfo);
//1					doc->ReleaseRef();
				}
			}

		}

/*		CObjectDomainActions::TDirtyObjects::iterator doit;
		for (doit=actionObj->GetDirtyObjects().begin();doit!=actionObj->GetDirtyObjects().end();doit++)
		{
			if ((currentTick-doit->second->GetLastPush())>50)
			{
				doit->second->SetWho(controllingObj->GetID());
				network->PushObject(doit->second,doit->second->GetSendToClientList());
			}
		}*/
		actionObj->GetDirtyObjects().clear();
		GetObjMan()->CleanUp();
	}


}










