#include "CTL2ClientWindow.h"
#include "../implementation/glFE.h"

namespace FRONTEND
{
	CCTL2ClientWindow::CCTL2ClientArea::CCTL2ClientArea(CFrontEnd *_fe, DOMAIN::CGraphObject *_domainObj, HResGUIStyle &_guiStyle, void *_userdata, FCTL2CallBack _callback)
		: CCTL2Container(_fe, _domainObj, _guiStyle, _userdata, _callback)
	{}
	
	void CCTL2ClientWindow::CCTL2ClientArea::Resize(int w, int h)
	{
//		CCTL2Container::_Resize(w,h);
			
		if ((GetPos().w!=w)||(GetPos().h!=h))
			((CCTL2ClientWindow*)parent)->ClientResized(w,h);
	}


	CCTL2ClientWindow::CCTL2ClientWindow(CFrontEnd *_fe, DOMAIN::CGraphObject *_domainObj, HResGUIStyle &_guiStyle, void *_userdata, FCTL2CallBack _callback)
		: CCTL2Container(_fe, _domainObj, _guiStyle, _userdata, _callback),
		  clientArea(new CCTL2ClientArea(_fe, _domainObj, _guiStyle, _userdata, _callback)),
		  borderSize()
	{
		float vertices[4*3] = 
		{
			-1.0f,-1.0f,0.0f,
			-1.0f, 1.0f,0.0f,
			 1.0f, 1.0f,0.0f,
			 1.0f,-1.0f,0.0f,
		};		
		float normals[4*3] = 
		{
			0.0f,0.0f,1.0f,
			0.0f,0.0f,1.0f,
			0.0f,0.0f,1.0f,
			0.0f,0.0f,1.0f,
		};

		int i;

		for (i=0;i<8;i++)
		{
			borderSurfaces[i].numVertices = 4;
			borderSurfaces[i].primitiveType = GL_QUADS;
			borderSurfaces[i].shader = styleElement->borderShader;
			borderSurfaces[i].lightMap = NULL;
			borderSurfaces[i].vertices = (GENERAL::CVector3f*)malloc(4*sizeof(GENERAL::CVector3f));
			borderSurfaces[i].normals = (GENERAL::CVector3f*)malloc(4*sizeof(GENERAL::CVector3f));
			borderSurfaces[i].lightMapTexCoords = NULL;
			borderSurfaces[i].texCoords = NULL;
			borderSurfaces[i].modelView = NULL;
			borderSurfaces[i].colors=NULL;
			borderSurfaces[i].name=0;
			borderSurfaces[i].flags=0;
			borderSurfaces[i].type = TSurface::SURFACETYPE_POLYGON;

			memcpy(borderSurfaces[i].normals, normals, 4*sizeof(GENERAL::CVector3f));
			memcpy(borderSurfaces[i].vertices, vertices, 4*sizeof(GENERAL::CVector3f));
		}

		HideBackground();
		clientArea->SetAnchorLeft();
		clientArea->SetAnchorTop();
		clientArea->SetAnchorRight();
		clientArea->SetAnchorBottom();
		CCTL2Container::InsertControl((sptr<CCTL2Window>)clientArea);
		SetClientSize();
	}

	CCTL2ClientWindow::~CCTL2ClientWindow()
	{
		int i;
		for (i=0;i<8;i++)
		{
			SAFE_FREE(borderSurfaces[i].vertices);
			SAFE_FREE(borderSurfaces[i].normals);
		}
	}

	void CCTL2ClientWindow::Init(sptr<CRenderAble> &_self)
	{
		CCTL2Container::Init(_self);
		clientArea->Init((sptr<CRenderAble>)clientArea);
		clientArea->SetNotMoveable();
		clientArea->SetNotResizeable();
	}

	void CCTL2ClientWindow::SetStyleElement(CResGUIStyle::TElement *_styleElement)
	{
		styleElement = _styleElement;
		clientArea->SetStyleElement(_styleElement);
		SetBorderSize(styleElement->borderLeft,styleElement->borderTop,styleElement->borderRight,styleElement->borderBottom);
		for (int i=0; i<8; i++) 
			borderSurfaces[i].shader=styleElement->borderShader;
	}

	void CCTL2ClientWindow::SetSubStyle(int _num)
	{
		clientArea->SetSubStyle(_num);
		SetBorderSize(styleElement[_num].borderLeft,styleElement[_num].borderTop,styleElement[_num].borderRight,styleElement[_num].borderBottom);
		for (int i=0; i<8; i++) 
			borderSurfaces[i].shader=styleElement[_num].borderShader;
	}

	void CCTL2ClientWindow::InsertControl(sptr<CCTL2Window> &_ctl)
	{
		clientArea->InsertControl(_ctl);
	}

	void CCTL2ClientWindow::RemoveControl(CCTL2Window *_ctl)
	{
		clientArea->RemoveControl(_ctl);
	}

	void CCTL2ClientWindow::BringToFront(CCTL2Window *_ctl)
	{
		clientArea->BringToFront(_ctl);
	}

	void CCTL2ClientWindow::SendToBack(CCTL2Window *_ctl)
	{
		clientArea->SendToBack(_ctl);
	}

	void CCTL2ClientWindow::CalculateClientSize(TMorfRecti &rect)
	{
		rect.x += borderSize.l;
		rect.y += borderSize.b;
		rect.w += GetPos().w-(borderSize.l+borderSize.r);
		rect.h += GetPos().h-(borderSize.t+borderSize.b);
	}

	void CCTL2ClientWindow::SetClientSize()
	{
		TMorfRecti rect(0,0,0,0);
		CalculateClientSize(rect);
		clientArea->_SetPos(rect.x,rect.y,rect.w,rect.h);
	}

	void CCTL2ClientWindow::_Resize(int w, int h)
	{
		CCTL2Container::_Resize(w,h);
		SetClientSize();
	}

	void CCTL2ClientWindow::CalculateWindowSize(int &_cx, int &_cy)
	{
		if (HasBorder())
		{
			_cx+=borderSize.l+borderSize.r;
			_cy+=borderSize.b+borderSize.t;
		}
	}

	void CCTL2ClientWindow::ClientResized(int w, int h)
	{
		CalculateWindowSize(w,h);
		_Resize(w,h);
//		_Resize(w+borderSize.l+borderSize.r,h+borderSize.b+borderSize.t);
	}

	void CCTL2ClientWindow::RenderScissor(TMorfRecti _scissor, int _dt)
	{
		
		if ( (HasBorder()) && ((borderSize.t) || (borderSize.b) || (borderSize.r) || (borderSize.l)))
		{
			for (int i=0; i<8; i++)
				((CGLFrontEnd*)fe)->GetRenderer().PushGUISurface(borderSurfaces+i,_scissor,i+BORDER_ELEMENT_B_TL);
		}
		CCTL2Container::RenderScissor(_scissor, _dt);
	}

	void CCTL2ClientWindow::CalculateSurfaces()
	{
		TMorfPointi screenPos(pos.pos);
		if (GetParent()!=NULL)
			screenPos += GetParent()->GetScreenPos();

/*		if (parent)
		{
			screenPos = parent->GetScreenPos();
			screenPos+=pos.pos;
			//screenPos.x+=parent->GetBorderSize().l;
			//screenPos.y+=parent->GetBorderSize().b;
			screenPos.x-=(int)parent->GetRenderOfs().x;
			screenPos.y-=(int)parent->GetRenderOfs().y;
		}*/

		GENERAL::TMorfPointi nbs(pos.w-(borderSize.l+borderSize.r),pos.h-(borderSize.t+borderSize.b));

/*		surface.vertices[0].x = (float)(borderSize.l+screenPos.x);
		surface.vertices[0].y = (float)(borderSize.b+screenPos.y);
		surface.vertices[1].x = (float)(borderSize.l+screenPos.x);
		surface.vertices[1].y = (float)(borderSize.b+nbs.y+screenPos.y);				
		surface.vertices[2].x = (float)(borderSize.l+nbs.x+screenPos.x);
		surface.vertices[2].y = (float)(borderSize.b+nbs.y+screenPos.y);
		surface.vertices[3].x = (float)(borderSize.l+nbs.x+screenPos.x);
		surface.vertices[3].y = (float)(borderSize.b+screenPos.y);*/

		GENERAL::TMorfRecti borderRects[8];
		
		// Top-Left corner
		borderRects[0] = GENERAL::TMorfRecti(0,borderSize.b+nbs.y,borderSize.l,borderSize.t);
		// Top border
		borderRects[1] = GENERAL::TMorfRecti(borderSize.l,borderSize.b+nbs.y,nbs.x,borderSize.t);
		// Top-Right corner
		borderRects[2] = GENERAL::TMorfRecti(borderSize.l+nbs.x,borderSize.b+nbs.y,borderSize.r,borderSize.t);
		// Left border
		borderRects[3] = GENERAL::TMorfRecti(0,borderSize.b,borderSize.l,nbs.y);
		// Right border
		borderRects[4] = GENERAL::TMorfRecti(borderSize.l+nbs.x,borderSize.b,borderSize.r,nbs.y);
		// Lower-Left corner	
		borderRects[5] = GENERAL::TMorfRecti(0,0,borderSize.l,borderSize.b);
		// Bottom border
		borderRects[6] = GENERAL::TMorfRecti(borderSize.l,0,nbs.x,borderSize.b);
		// Lower-Right corner
		borderRects[7] = GENERAL::TMorfRecti(borderSize.l+nbs.x,0,borderSize.r,borderSize.b);

		for (int i=0; i<8; i++)
		{
			borderSurfaces[i].vertices[0].x=(float)(screenPos.x+borderRects[i].x);
			borderSurfaces[i].vertices[0].y=(float)(screenPos.y+borderRects[i].y);
			borderSurfaces[i].vertices[1].x=(float)(screenPos.x+borderRects[i].x);
			borderSurfaces[i].vertices[1].y=(float)(screenPos.y+borderRects[i].y+borderRects[i].h);
			borderSurfaces[i].vertices[2].x=(float)(screenPos.x+borderRects[i].x+borderRects[i].w);
			borderSurfaces[i].vertices[2].y=(float)(screenPos.y+borderRects[i].y+borderRects[i].h);
			borderSurfaces[i].vertices[3].x=(float)(screenPos.x+borderRects[i].x+borderRects[i].w);
			borderSurfaces[i].vertices[3].y=(float)(screenPos.y+borderRects[i].y);
		}

		CCTL2Container::CalculateSurfaces();
	}

	ECTL2ResizeArea CCTL2ClientWindow::GetResizeArea(int x,int y)
	{
		bool isLeft,isRight,isTop,isBottom;

		isLeft=x<borderSize.l;
		isRight=isLeft?false:x>=(pos.w-borderSize.r);
		isBottom=y<borderSize.b;
		isTop=isBottom?false:y>=(pos.h-borderSize.t);

		if (isTop)
		{
			if (isLeft) return CTL2RESIZEAREA_TOPLEFT;
			if (isRight) return CTL2RESIZEAREA_TOPRIGHT;
			return CTL2RESIZEAREA_TOP;
		} else if (isBottom)
		{
			if (isLeft) return CTL2RESIZEAREA_BOTTOMLEFT;
			if (isRight) return CTL2RESIZEAREA_BOTTOMRIGHT;
			return CTL2RESIZEAREA_BOTTOM;
		} else if (isLeft)
		{
			return CTL2RESIZEAREA_LEFT;
		} else if (isRight)
		{
			return CTL2RESIZEAREA_RIGHT;
		} else return CTL2RESIZEAREA_NONE;
	}

	void CCTL2ClientWindow::SetUserdata(void *_userdata)
	{
		CCTL2Window::SetUserdata(_userdata);
		clientArea->SetUserdata(_userdata);
	}


}