#include "CTL2Edit.h"
#include "CTL2Container.h"

#define EXTRACHARS 16

namespace FRONTEND
{
	CCTL2Edit::CCTL2Edit(CFrontEnd *_fe, DOMAIN::CGraphObject *_domainObj, HResGUIStyle &_guiStyle, void *_userdata, FCTL2CallBack _callback)
		: CCTL2Label(_fe, _domainObj, _guiStyle, _userdata, _callback),
		  markStart(0),
		  markEnd(0)
	{
	}

	CCTL2Edit::~CCTL2Edit()
	{
	}

	void CCTL2Edit::Init(sptr<CRenderAble> &_self)
	{
		CCTL2Label::Init(_self);
		SetNotMoveable();
	}

	void CCTL2Edit::SetText(char *_text)
	{
		CCTL2Label::SetText(_text);
		markStart = markEnd = (int)strlen(text);
	}

	int CCTL2Edit::GetCharAt(int x, int y)
	{
		int charofs;

		int l=(int)lineInfos.size();

		if (l==0) return 0;

		// find Line
		TLineInfo *li = 0;
		while (l--)
		{
			li=&lineInfos[l];

			if (li->pos.y+GetShader()->fnt->GetHeight()>y)
				break;
		}

		// find Char
		if (l<0) 
		{
			charofs=0;
		} else {
			charofs=li->cofs;
			x-=(int)li->pos.x;
			while ((text[charofs]!='\0') && (x>GetShader()->fnt->GetCharWidth(text[charofs])))
			{
				x-=GetShader()->fnt->GetCharWidth(text[charofs]);
				charofs++;				
				if (charofs==li->cofs+li->ccnt)
					break;
			}
		}

		return charofs;
	}

	bool CCTL2Edit::DeleteChar(int index)
	{
		if (text!=0)
		{
			if ((index>=0) && (index<(int)strlen(text)))
			{
				char *newtext=(char*)malloc(EXTRACHARS+sizeof(char)*strlen(text));
				memcpy(newtext,text,index);
				memcpy(newtext+index,text+index+1,strlen(text)-index);
				free(text);
				text=newtext;
				CalculateTextMetrics();
				return true;
			}
		}
		return false;
	}

	bool CCTL2Edit::DeleteMark()
	{
		int ms,me;
		if (markStart<markEnd)
		{
			ms = markStart;
			me = markEnd;
		} else {
			me = markStart;
			ms = markEnd;
		}

		if (ms<me)
		{
			char *newtext=(char*)malloc(EXTRACHARS+(sizeof(char)*strlen(text)-(me-ms))+1);
			memcpy(newtext,text,ms);
			memcpy(newtext+ms,text+me,(strlen(text)-(me-ms))+1);
			free(text);
			text=newtext;
			markEnd=markStart=ms;
			CalculateTextMetrics();
			return true;
		}
		return false;
	}

	bool CCTL2Edit::KeyDown(int key,TKeyStates &keyState)
	{
		int ms,me;
		if (markStart<markEnd)
		{
			ms = markStart;
			me = markEnd;
		} else {
			me = markStart;
			ms = markEnd;
		}

		switch (key)
		{
		default:
			return true; // prevent all keydown from beeing passed to parent controls
//			return CCTLLabel::KeyDown(key,keyState);
		case VK_LEFT:
			if (keyState[VK_SHIFT])  // Shift is Down
			{
				if (markEnd>0)
				{
					markEnd--;
				}
			} else {
				if (ms==me)
				{
					if (ms>0)
					{
						markStart=markEnd=ms-1;
					}
				} else markStart=markEnd=ms;
			}
			CalculateSurfaces();
			break;
		case VK_RIGHT:
			if (keyState[VK_SHIFT])  // Shift is Down
			{
				if (markEnd<(int)strlen(text))
				{
					markEnd++;
				}
			} else {
				if (ms==me)
				{
					if (ms<(int)strlen(text))
					{
						markStart=markEnd=ms+1;
					}
				} else markEnd=markStart=me;
			}
			CalculateSurfaces();
			break;
		case VK_BACK:
			if (me==ms)
			{
				if (DeleteChar(ms-1))
					markStart=markEnd=ms-1;
			} else {
				DeleteMark();
			}
			break;
		case VK_DELETE:
			if (me==ms)
			{
				DeleteChar(ms);
			} else {
				DeleteMark();
			}
			break;
		}
		return true;	
	}

	bool CCTL2Edit::KeyPress(int key,TKeyStates &keyState)
	{
		char *newtext;

		int ms,me;
		if (markStart<markEnd)
		{
			ms = markStart;
			me = markEnd;
		} else {
			me = markStart;
			ms = markEnd;
		}

		if (isprint(key) || ((key=='\r') && IsMultiLined()))
		{
			if (key=='\r')
				key='\n';
			if (text!=NULL)
			{
				newtext=(char*)malloc(EXTRACHARS+sizeof(char)*(2+strlen(text)-(me-ms)));
				strncpy(newtext,text,ms);
				newtext[ms]=(char)key;
				strncpy(newtext+ms+1,text+me,1+strlen(text)-me);
				SAFE_FREE(text);
				text=newtext;
			}
			else
			{
				text=(char*)malloc(2);
				text[0]=(char)key;
				text[1]='\0';
			}
			markStart=markEnd=ms+1;
			CalculateTextMetrics();
			return true;
		}
		if (key=='\r')
		{
			if (ms!=me)
			{
				DeleteMark();
				return true;
			}
		}
		return CCTL2Label::KeyPress(key,keyState);
	}

	bool CCTL2Edit::MouseDown(EMouseButton button, INT x,INT y,INT z,TMouseButtons &buttons,TKeyStates &keys)
	{
		if (button==MOUSEBUTTON_LEFT)
		{
			markEnd=markStart=GetCharAt(x,y);
			state=STATE_MARKING;
			SetMouseCapture();
		}
		return CCTL2Label::MouseDown(button,x,y,z,buttons,keys);
	}

	bool CCTL2Edit::MouseMove(INT x, INT y, INT z, TMouseButtons &buttons, TKeyStates &keys)
	{
		switch (state)
		{
		default:
		case STATE_NONE:
			return CCTL2Label::MouseMove(x,y,z,buttons,keys);

		case STATE_MARKING:
			markEnd=GetCharAt(x,y);
			CalculateSurfaces();
			return true;
		}
	}

	bool CCTL2Edit::MouseUp(EMouseButton button, INT x,INT y,INT z,TMouseButtons &buttons,TKeyStates &keys)
	{
		switch (state)
		{
		default:
		case STATE_NONE:
			return CCTL2Label::MouseUp(button,x,y,z,buttons,keys);

		case STATE_MARKING:
			if (button==MOUSEBUTTON_LEFT)
			{ 
				ReleaseMouseCapture();
				state=STATE_NONE;
			}
			return true;
		}
	}

	void CCTL2Edit::CalculateSurfaces()
	{
		CCTL2Window::CalculateSurfaces();

		TMorfPointi screenPos(pos.pos);
		if (GetParent()!=NULL)
			screenPos += GetParent()->GetScreenPos();

		surface.texts.clear();
		TLineInfoList::iterator it;

		surface.flags&=~TSurface::SF_TEXTONLY;
		bool marked=false;
		for (it=lineInfos.begin();it!=lineInfos.end();it++)
		{
			TLineInfo &li=(*it);
			CVector3f lp = li.pos;
			lp.x+=(SCALAR)screenPos.x;
			lp.y+=(SCALAR)screenPos.y;

			int lMarkStart=-1;
			int lMarkEnd=-1;
			int tp1,tp2;
			int ms,me;

			if (markStart<markEnd)
			{
				ms = markStart;
				me = markEnd;
			}
			else
			{
				me = markStart;
				ms = markEnd;
			}

			if (li.cofs+li.ccnt>=ms)
			{
				lMarkStart = ms-li.cofs;
			}
			if (li.cofs+li.ccnt>=me)
			{
				lMarkEnd = me-li.cofs;
			}

			tp1=lMarkStart;
			tp2=lMarkEnd;
			if (tp1<0) 
			{
				tp1=tp2;
				tp2=-1;
			}

			if (tp1>=0)
			{
				surface.AddText(&surface.shader->fnt,lp,TColor4ub(0,0,0,255),text+li.cofs,tp1,marked,true);
				marked=!marked;

				if (tp2>=0)
				{
					surface.AddText(&surface.shader->fnt,lp,TColor4ub(0,0,0,255),text+li.cofs+tp1,tp2-tp1,marked,false);
					marked=!marked;
					surface.AddText(&surface.shader->fnt,lp,TColor4ub(0,0,0,255),text+li.cofs+tp2,li.ccnt-tp2,marked,false);
				} 
				else 
				{
					surface.AddText(&surface.shader->fnt,lp,TColor4ub(0,0,0,255),text+li.cofs+tp1,li.ccnt-tp1,marked,false);
				}
			} 
			else 
			{
				surface.AddText(&surface.shader->fnt,lp,TColor4ub(0,0,0,255),text+li.cofs,li.ccnt,marked,true);
			}
		}
	}
}
