#ifndef CTL2LABEL_H
#define CTL2LABEL_H

#include "CTL2Window.h"

namespace FRONTEND
{
	enum ETextFlags	{
		TEXTFLAG_HLEFT		= 0x0000,
		TEXTFLAG_HCENTER	= 0x0001,
		TEXTFLAG_HRIGHT		= 0x0002,
		TEXTFLAG_HALIGN		= 0x0003,

		TEXTFLAG_VTOP		= 0x0000,
		TEXTFLAG_VCENTER	= 0x0004,
		TEXTFLAG_VBOTTOM	= 0x0008,
		TEXTFLAG_VALIGN		= 0x000C,

		TEXTFLAG_MULTILINE	= 0x0010,
		TEXTFLAG_WORDWRAP	= 0x0020,
	};

	class CCTL2Label : public CCTL2Window
	{
	protected:
		char *text;
		unsigned short textFlags;
		unsigned char baseLine;
		TLineInfoList lineInfos;

		virtual void _Resize(int w, int h);
		virtual void CalculateTextMetrics();
	public:
		CCTL2Label(CFrontEnd *_fe, DOMAIN::CGraphObject *_domainObj, HResGUIStyle &_guiStyle, void *_userdata, FCTL2CallBack _callback = DummyCallBack);		
		virtual ~CCTL2Label();
//		virtual void Init(sptr<CRenderAble> &_self);
		
		virtual void CalculateSurfaces();
		virtual void SetText(char *_text);
		inline char *GetText() { return text; }
		virtual void SetStyleElement(CResGUIStyle::TElement *_styleElement);

		inline SetAlignHLeft() { textFlags = (textFlags & (~TEXTFLAG_HALIGN)) | TEXTFLAG_HLEFT; }
		inline SetAlignHCenter() { textFlags = (textFlags & (~TEXTFLAG_HALIGN)) | TEXTFLAG_HCENTER; }
		inline SetAlignHRight() { textFlags = (textFlags & (~TEXTFLAG_HALIGN)) | TEXTFLAG_HRIGHT; }
		inline unsigned short GetHAlign() {return textFlags & TEXTFLAG_HALIGN;}
		inline bool IsHLeft() { return GetHAlign()==TEXTFLAG_HLEFT; };
		inline bool IsHCenter() {  return GetHAlign()==TEXTFLAG_HCENTER; };
		inline bool IsHRight() {  return GetHAlign()==TEXTFLAG_HRIGHT; };

		inline SetAlignVTop() { textFlags = (textFlags & (~TEXTFLAG_VALIGN)) | TEXTFLAG_VTOP; }
		inline SetAlignVCenter() { textFlags = (textFlags & (~TEXTFLAG_VALIGN)) | TEXTFLAG_VCENTER; }
		inline SetAlignVBottom() { textFlags = (textFlags & (~TEXTFLAG_VALIGN)) | TEXTFLAG_VBOTTOM; }
		inline unsigned short GetVAlign() {return textFlags & TEXTFLAG_VALIGN;}
		inline bool IsVTop() {  return GetVAlign()==TEXTFLAG_VTOP; };
		inline bool IsVCenter() {  return GetVAlign()==TEXTFLAG_VCENTER; };
		inline bool IsVBottom() {  return GetVAlign()==TEXTFLAG_VBOTTOM; };

		CCTL2_ACCESSFLAG(SetWordWrap,ClearWordWrap,ToggleWordWrap,DoWordWrap,textFlags,TEXTFLAG_WORDWRAP);
		CCTL2_ACCESSFLAG(SetMultiLine,ClearMultiLine,ToggleMultiLine,IsMultiLined,textFlags,TEXTFLAG_MULTILINE);
	};
}

#endif
