#include "CTL2ScrollBar.h"

namespace FRONTEND
{
	CCTL2ScrollBar::CCTL2ScrollBar(CFrontEnd *_fe, DOMAIN::CGraphObject *_domainObj, HResGUIStyle &_guiStyle, EScrollBarType _scrollBarType, void *_userdata, FCTL2CallBack _callback)
		: CCTL2ClientWindow(_fe,_domainObj, _guiStyle, _userdata, _callback),
		  btnLow(new CCTL2Button(_fe,_domainObj, _guiStyle, NULL, StaticCallBack)),
		  btnHigh(new CCTL2Button(_fe,_domainObj, _guiStyle, NULL, StaticCallBack)),
		  slider(new CCTL2ClientWindow(_fe,_domainObj, _guiStyle, NULL, StaticCallBack)),
		  scrollBarType(_scrollBarType),
		  maxVal(100),
		  minVal(0),
		  pageSize(10),
		  scrollPos(minVal)
	{
		if (scrollBarType==SB_VERTICAL)
		{
			ClearAnchorLeft();
			SetAnchorRight();
			SetAnchorTop();
			SetAnchorBottom();

			btnLow->ClearAnchorBottom();
			btnLow->SetAnchorTop();
			btnLow->ClearAnchorLeft();
			btnLow->SetAnchorRight();

			btnHigh->SetAnchorBottom();
			btnHigh->ClearAnchorTop();
			btnHigh->ClearAnchorLeft();
			btnHigh->SetAnchorRight();

			slider->ClearAnchorBottom();
			slider->SetAnchorTop();
			slider->SetAnchorLeft();
			slider->SetAnchorRight();
		} 
		else
		{
			ClearAnchorTop();
			SetAnchorRight();
			SetAnchorLeft();
			SetAnchorBottom();

			btnLow->SetAnchorBottom();
			btnLow->ClearAnchorTop();
			btnLow->SetAnchorLeft();
			btnLow->ClearAnchorRight();

			btnHigh->SetAnchorBottom();
			btnHigh->ClearAnchorTop();
			btnHigh->ClearAnchorLeft();
			btnHigh->SetAnchorRight();

			slider->SetAnchorBottom();
			slider->SetAnchorTop();
			slider->ClearAnchorLeft();
			slider->SetAnchorRight();		
		}

		slider->SetNotResizeable();
		slider->SetMoveable();
	}

	void CCTL2ScrollBar::Init(sptr<CRenderAble> &_self)
	{
		btnHigh->SetUserdata(this);
		btnLow->SetUserdata(this);
		slider->SetUserdata(this);
		slider->GetClientArea()->ClearReceiveKeyEvents();
		slider->GetClientArea()->ClearReceiveMouseMoveEvents();
		slider->GetClientArea()->ClearReceiveMouseButtonEvents();
		slider->SetMoveable();

		CCTL2ClientWindow::Init(_self);
		HideBorder();
		SetBorderSize(0,0,0,0);
		if (scrollBarType==SB_VERTICAL)
		{
			btnHigh->SetStyleElement(guiStyle->btnup);
			btnHigh->Init((sptr<CRenderAble>)btnHigh);
			btnLow->SetStyleElement(guiStyle->btndown);
			btnLow->Init((sptr<CRenderAble>)btnLow);
			_Resize(btnLow->GetPos().w,GetPos().h);
			InsertControl((sptr<CCTL2Window>)btnHigh);
			InsertControl((sptr<CCTL2Window>)btnLow);

			slider->SetStyleElement(guiStyle->btndefault);
			slider->Init((sptr<CRenderAble>)slider);
			InsertControl((sptr<CCTL2Window>)slider);
	
			btnHigh->Move(0,0);
			btnLow->Move(0,GetPos().h-btnLow->GetPos().h);
		}
		else
		{
			btnHigh->SetStyleElement(guiStyle->btnright);
			btnHigh->Init((sptr<CRenderAble>)btnHigh);
			btnLow->SetStyleElement(guiStyle->btnleft);
			btnLow->Init((sptr<CRenderAble>)btnLow);
			_Resize(GetPos().w,btnLow->GetPos().h);
			InsertControl((sptr<CCTL2Window>)btnHigh);
			InsertControl((sptr<CCTL2Window>)btnLow);

			slider->SetStyleElement(guiStyle->btndefault);
			slider->Init((sptr<CRenderAble>)slider);
			InsertControl((sptr<CCTL2Window>)slider);

			btnLow->Move(0,0);
			btnHigh->Move(GetPos().w-btnHigh->GetPos().w,0);		
		}

		SetSliderSize();
		SetSliderPos();
		CalculateSurfaces();
	}

	void CCTL2ScrollBar::SetSliderSize()
	{
		int slideLen = (maxVal-minVal);
		if (scrollBarType==SB_VERTICAL)
		{
			int screenLen=GetPos().h-(btnHigh->GetPos().h+btnLow->GetPos().h);
			int sliderSize = slideLen==0?0:(pageSize*screenLen)/slideLen;
			sliderSize = min(sliderSize,screenLen);
			if (sliderSize<8) sliderSize=8;
			
			slider->Resize(btnHigh->GetPos().w,sliderSize);
		}
		else
		{
			int screenLen=GetPos().w-(btnHigh->GetPos().w+btnLow->GetPos().w);
			int sliderSize = slideLen==0?0:(pageSize*screenLen)/slideLen;
			sliderSize = min(sliderSize,screenLen);
			if (sliderSize<8) sliderSize=8;
			
			slider->Resize(sliderSize,btnHigh->GetPos().h);
		}
	}

	void CCTL2ScrollBar::SetSliderPos()
	{
		int slideLen = (maxVal-minVal);
		int slidePos = (scrollPos-minVal);

		if (scrollBarType==SB_VERTICAL)
		{
			int screenLen=GetPos().h-(btnHigh->GetPos().h+btnLow->GetPos().h);
			int sliderPos = slideLen==0?0:(screenLen*slidePos)/slideLen;
			slider->Move(0,screenLen-sliderPos+btnHigh->GetPos().h-slider->GetPos().h);
		}
		else
		{
			int screenLen=GetPos().w-(btnHigh->GetPos().w+btnLow->GetPos().w);
 			int sliderPos = slideLen==0?0:(screenLen*slidePos)/slideLen;
			slider->Move(sliderPos+btnLow->GetPos().w,0);
		}
	}

	int CCTL2ScrollBar::CalculatePosFromSlider()
	{
		int slideLen = (maxVal-minVal)-pageSize;
		int screenLen;
		int sliderPos;

		if (scrollBarType==SB_VERTICAL)
		{
			screenLen = GetPos().h-(btnHigh->GetPos().h+btnLow->GetPos().h+slider->GetPos().h);
			sliderPos = screenLen-(slider->GetPos().y-btnHigh->GetPos().h);
		}
		else
		{
			screenLen = GetPos().w-(btnHigh->GetPos().w+btnLow->GetPos().w+slider->GetPos().w);
			sliderPos = slider->GetPos().x-btnLow->GetPos().w;
		}
		return screenLen==0?0:(slideLen*sliderPos)/screenLen;
	};

	void CCTL2ScrollBar::_Resize(int w, int h)
	{
		CCTL2ClientWindow::_Resize(w,h);
		SetSliderSize();
		SetSliderPos();
	}

	bool CCTL2ScrollBar::StaticCallBack(CCTL2Window* ctl,ECTL2CallBackAction action,unsigned long lparam,void *pparam, void *userdata)
	{
		if (userdata!=NULL)
			return ((CCTL2ScrollBar*)userdata)->CallBack(ctl,action,lparam,(TCTL2PParam*)pparam);
		else
			return DummyCallBack(ctl,action,lparam,pparam,userdata);
	}

	bool CCTL2ScrollBar::CallBack(CCTL2Window* ctl,ECTL2CallBackAction action,unsigned long lparam,TCTL2PParam *pparam)
	{
		int oldSP = scrollPos;
		static bool calcScrollPos = true;
		if (ctl==&slider)
		{
			switch (action)
			{
			case CB2ACTION_MOVING:
				if (scrollBarType==SB_VERTICAL)
				{
					pparam->move.x = 0;
					if (pparam->move.y<btnHigh->GetPos().h) pparam->move.y=btnHigh->GetPos().h;
					if (pparam->move.y>btnLow->GetPos().y-slider->GetPos().h) pparam->move.y=btnLow->GetPos().y-slider->GetPos().h;
				}
				else
				{
					pparam->move.y = 0;
					if (pparam->move.x<btnLow->GetPos().w) pparam->move.x=btnLow->GetPos().w;
					if (pparam->move.x>btnHigh->GetPos().x-slider->GetPos().w) pparam->move.x=btnHigh->GetPos().x-slider->GetPos().w;
				}

				// fall through
			case CB2ACTION_MOVED:
				if (calcScrollPos)
					scrollPos = CalculatePosFromSlider();
				if (oldSP!=scrollPos)
					callBack(this, CB2ACTION_SCROLLBARCHANGE, scrollPos, NULL, userdata);
				break;
			}
		} 
		else if (ctl==&btnHigh)
		{
			switch (action)
			{
			case CB2ACTION_MOUSECLICK:
				calcScrollPos = false;
				scrollPos+=pageSize/2;
				if (scrollPos>maxVal-pageSize) scrollPos=maxVal-pageSize;
				SetSliderPos();
				if (oldSP!=scrollPos)
					callBack(this, CB2ACTION_SCROLLBARCHANGE, scrollPos, NULL, userdata);
				calcScrollPos = true;
				break;
			}
		}
		else if (ctl==&btnLow)
		{
			switch (action)
			{
			case CB2ACTION_MOUSECLICK:
				calcScrollPos = false;
				scrollPos-=pageSize/2;
				if (scrollPos<minVal) scrollPos=minVal;
				SetSliderPos();
				if (oldSP!=scrollPos)
					callBack(this, CB2ACTION_SCROLLBARCHANGE, scrollPos, NULL, userdata);
				calcScrollPos = true;
				break;
			}
		}
		return true;
	}

}
