#include "CTL2Window.h"
#include "../implementation/glFE.h"
#include "CTL2Container.h"

namespace FRONTEND
{
	sptr<CCTL2Window> CCTL2Window::focusedControl;
	sptr<CCTL2Window> CCTL2Window::mouseOverControl;
	sptr<CCTL2Window> CCTL2Window::mouseCaptureControl;
	sptr<CCTL2Window> CCTL2Window::mouseButtonDownControl[CCTL2_NUM_MOUSEBUTTONS];

	CCTL2Window::TDragInfo		CCTL2Window::dragInfo;
	CCTL2Window::TResizeInfo	CCTL2Window::resizeInfo;

	bool CCTL2Window::DummyCallBack(CCTL2Window*,ECTL2CallBackAction action,unsigned long,void *pparam, void *userdata)
	{
		switch (action)
		{
		case CB2ACTION_KEYDOWN:
		case CB2ACTION_KEYPRESS:
		case CB2ACTION_KEYUP:
			return false;
		default: 
			return true;
		}
	}

	CCTL2Window::CCTL2Window(CFrontEnd *_fe, DOMAIN::CGraphObject *_domainObj, HResGUIStyle &_guiStyle, void *_userdata, FCTL2CallBack _callback)
		  : CRenderAble(_fe, _domainObj),
			ctlID(0),
			callBack(_callback),
			pos(0,0,100,100),
			parent(0),
			focusControl(NULL),
			userdata(_userdata),
			flags(
				CTL2WINFLAG_VISIBLE |
				CTL2WINFLAG_HASBORDER |
				CTL2WINFLAG_HASBACKGROUND |
				CTL2WINFLAG_MOVEABLE |
				CTL2WINFLAG_RESIZEABLE |
				CTL2WINFLAG_ANCHOR_TOP |
				CTL2WINFLAG_ANCHOR_LEFT),
			reveiveEvents(-1),
			guiStyle(_guiStyle),
			styleElement(guiStyle->window),
			state(STATE_NONE)
	{
		float vertices[4*3] = 
		{
			-1.0f,-1.0f,0.0f,
			-1.0f, 1.0f,0.0f,
			 1.0f, 1.0f,0.0f,
			 1.0f,-1.0f,0.0f,
		};		
		float normals[4*3] = 
		{
			0.0f,0.0f,1.0f,
			0.0f,0.0f,1.0f,
			0.0f,0.0f,1.0f,
			0.0f,0.0f,1.0f,
		};

		surface.numVertices = 4;
		surface.primitiveType = GL_QUADS;
		surface.shader = styleElement->clientShader;
		surface.lightMap = NULL;
		surface.vertices = (GENERAL::CVector3f*)malloc(4*sizeof(GENERAL::CVector3f));
		surface.normals = (GENERAL::CVector3f*)malloc(4*sizeof(GENERAL::CVector3f));
		surface.lightMapTexCoords = NULL;
		surface.texCoords = NULL;
		surface.modelView = NULL;
		surface.colors=NULL;
		surface.name=0;
		surface.flags=0;
		surface.type = TSurface::SURFACETYPE_POLYGON;

		memcpy(surface.normals, normals, 4*sizeof(GENERAL::CVector3f));
		memcpy(surface.vertices, vertices, 4*sizeof(GENERAL::CVector3f));
	}
	
	CCTL2Window::~CCTL2Window()
	{
		SAFE_FREE(surface.vertices);
		SAFE_FREE(surface.normals);
	}

	void CCTL2Window::Init(sptr<CRenderAble> &_self)
	{
		CRenderAble::Init(_self);
		
		self = _self;

		SetStyleElement(styleElement);
		SetSubStyle(0);
		CalculateSurfaces();
	}
	
	void CCTL2Window::SetStyleElement(CResGUIStyle::TElement *_styleElement)
	{
		styleElement=_styleElement;
		surface.shader = styleElement->clientShader;
		Resize(styleElement->sizex,styleElement->sizey);
	}

	void CCTL2Window::SetSubStyle(int _num)
	{
		surface.shader = styleElement[_num].clientShader;
	}

	void CCTL2Window::SetPos(int x, int y, int w, int h)
	{
		_SetPos(x,y,w,h);
	}

	void CCTL2Window::Move(int x, int y)
	{
		_Move(x,y);
	}

	void CCTL2Window::Resize(int w, int h)
	{
		_Resize(w,h);
	}

	void CCTL2Window::_SetPos(int x, int y, int w, int h)
	{
		_Move(x,y);
		_Resize(w,h);
	}

	void CCTL2Window::_Move(int x, int y)
	{
		TCTL2PParam pparam;
		pparam.move.ox = pos.x;
		pparam.move.oy = pos.y;
		pparam.move.x = x;
		pparam.move.y = y;

		if (callBack(this,CB2ACTION_MOVING,0,&pparam,userdata))
		{
			pos.pos.x = pparam.move.x;
			pos.pos.y = pparam.move.y;

			CalculateSurfaces();
			callBack(this,CB2ACTION_MOVED,0,&pparam,userdata);
		}
	}

	void CCTL2Window::_Resize(int w, int h)
	{
		pos.size.x = w;
		pos.size.y = h;
		CalculateSurfaces();
	}

	void CCTL2Window::SetFocus()
	{
		if (focusControl!=NULL) 
		{
			focusControl->SetFocus();
		} 
		else 
		{
			bool setFocus = true;
			if (focusedControl!=NULL)
			{
				setFocus = focusedControl->LostFocus();
			}

			if (setFocus)
			{
				focusedControl = self;
				GotFocus();
			}
		}
	}

	bool CCTL2Window::LostFocus()
	{
		return true;
	}

	void CCTL2Window::GotFocus()
	{
	}

	void CCTL2Window::SetMouseCapture()
	{
		((CGLFrontEnd*)fe)->SetMouseCapture();
		mouseCaptureControl = self;
	}

	void CCTL2Window::ReleaseMouseCapture()
	{
		mouseCaptureControl = 0;
		((CGLFrontEnd*)fe)->ReleaseMouseCapture();
	}

	void CCTL2Window::ClientToScreen(TMorfPointi &_windowPos)
	{
		ClientToParent(NULL,_windowPos);
	}

	void CCTL2Window::ScreenToClient(TMorfPointi &_screenPos)
	{
		_screenPos.x -= pos.x;
		_screenPos.y -= pos.y;
		if (parent!=NULL)
			parent->ScreenToClient(_screenPos);
	}

	void CCTL2Window::ClientToParent(CCTL2Container *_parent, TMorfPointi &_windowPos)
	{
		if (_parent==this) 
			return;

		_windowPos.x += pos.x;
		_windowPos.y += pos.y;
		if (parent!=NULL)
			parent->ClientToParent(_parent, _windowPos);
	}

	void CCTL2Window::ParentToClient(CCTL2Container *_parent, TMorfPointi &_parentPos)
	{
		_parentPos.x-=pos.x;
		_parentPos.y-=pos.y;

		if ((parent!=NULL) && (parent!=_parent))
			parent->ParentToClient(_parent,_parentPos);
	}

	bool CCTL2Window::HasParent(CCTL2Container *_parent)
	{
		CCTL2Container *p=GetParent();

		while (p!=NULL)
		{
			if (p==_parent) return true;
			p=p->GetParent();
		}
		return false;
	}

	bool CCTL2Window::IsOver(int x, int y)
	{
		return (IsVisible() &&  (x>=0) && (y>=0) && (x<pos.w) && (y<pos.h));
	}

	bool CCTL2Window::KeyDown(int key,TKeyStates &keystate)
	{
		if (!callBack(this,CB2ACTION_KEYDOWN,key,&keystate,userdata))
			if (parent!=NULL) return parent->KeyDown(key,keystate);
		return true;
	}

	bool CCTL2Window::KeyUp(int key,TKeyStates &keystate)
	{
		if (!callBack(this,CB2ACTION_KEYUP,key,&keystate,userdata))
			if (parent!=NULL) return parent->KeyUp(key,keystate);
		return true;
	}

	bool CCTL2Window::KeyPress(int key,TKeyStates &keystate)
	{
		if (!callBack(this,CB2ACTION_KEYPRESS,key,&keystate,userdata))
			if (parent!=NULL) return parent->KeyPress(key,keystate);
		return true;
	}

	bool CCTL2Window::MouseEnter()
	{
//		dprintf("Enter\n");
		callBack(this,CB2ACTION_MOUSEENTER,0,NULL,userdata);		
		return true;
	}

	bool CCTL2Window::MouseLeave()
	{
//		dprintf("Leave\n");
		callBack(this,CB2ACTION_MOUSELEAVE,0,NULL,userdata);		
		return true;
	}

	bool CCTL2Window::MouseMove(INT x,INT y,INT z,TMouseButtons &buttons,TKeyStates &keys)
	{
		if ((mouseCaptureControl==NULL) && (mouseOverControl != self))
		{
			/*
			sptr<CCTL2Window> oldMouseOverControl = mouseOverControl;
			CCTL2Window *win=NULL;
			if (oldMouseOverControl!=NULL)
			{
				win = &oldMouseOverControl;
				while (win!=NULL)
				{
					if (!win->HasParent((CCTL2Container*)this))
						break;

					win->MouseLeave();
					win = win->GetParent(); 
				}
			}

			CCTL2Window *win2 = this;
			while ( (win2!=win) && (win2!=NULL) )
			{
				win2->MouseEnter();
				win2=win2->GetParent();
			}*/

			if (mouseOverControl!=NULL)
			{
				mouseOverControl->MouseLeave();
			}
			mouseOverControl = self;
			mouseOverControl->MouseEnter();
		}
		if (state==STATE_MOVEING)
		{
			DragMove(x,y);
		}
		if (state==STATE_RESIZING)
		{
			ResizeMove(x,y);
		}
		TCTL2PParam param;
		param.mouse.button=MOUSEBUTTON_NONE;
		param.mouse.buttons=&buttons;
		param.mouse.keys=&keys;
		param.mouse.x=x;
		param.mouse.y=y;
		param.mouse.z=z;
		callBack(this,CB2ACTION_MOUSEMOVE,0,&param,userdata);		
		return true;
	}

	bool CCTL2Window::MouseDown(EMouseButton button, INT x,INT y,INT z,TMouseButtons &buttons,TKeyStates &keys)
	{
		if (IsOver(x,y))
		{
			SetFocus();

			if (parent!=NULL)
			{
				parent->BringToFront(this);
			}

			mouseButtonDownControl[button]=self;

			if (button==MOUSEBUTTON_LEFT)
			{
				ECTL2ResizeArea resizeArea=GetResizeArea(x,y);
				if ((IsResizeable()) && (resizeArea!=CTL2RESIZEAREA_NONE))
				{
					ResizeStart(resizeArea,x,y);
				} 
				else if (IsMoveable())
				{
					DragStart(x,y);
				}
			}

			TCTL2PParam param;
			param.mouse.button=button;
			param.mouse.buttons=&buttons;
			param.mouse.keys=&keys;
			param.mouse.x=x;
			param.mouse.y=y;
			param.mouse.z=z;
			callBack(this,CB2ACTION_MOUSEDOWN,0,&param,userdata);		
		}
		return true;
	}

	bool CCTL2Window::MouseUp(EMouseButton button, INT x,INT y,INT z,TMouseButtons &buttons,TKeyStates &keys)
	{
		if ((state==STATE_MOVEING) && (button==MOUSEBUTTON_LEFT))
		{
			DragEnd();
		}
		if ((state==STATE_RESIZING) && (button==MOUSEBUTTON_LEFT))
		{
			ResizeEnd();
		}

		if ((mouseButtonDownControl[button]==self) && IsOver(x,y))
		{
			MouseClick(button,x,y,z,buttons,keys);
		}
		mouseButtonDownControl[button]=0;

		TCTL2PParam param;
		param.mouse.button=button;
		param.mouse.buttons=&buttons;
		param.mouse.keys=&keys;
		param.mouse.x=x;
		param.mouse.y=y;
		param.mouse.z=z;
		callBack(this,CB2ACTION_MOUSEUP,0,&param,userdata);		
		return true;
	}

	bool CCTL2Window::MouseClick(EMouseButton button, INT x,INT y,INT z,TMouseButtons &buttons,TKeyStates &keys)
	{
		TCTL2PParam param;
		param.mouse.button=button;
		param.mouse.buttons=&buttons;
		param.mouse.keys=&keys;
		param.mouse.x=x;
		param.mouse.y=y;
		param.mouse.z=z;

		callBack(this,CB2ACTION_MOUSECLICK,0,&param,userdata);
	//	ToggleBackgroundVisiblitiy();
		return true;
	}

	void CCTL2Window::Render(int _dt)
	{
		RenderScissor(GENERAL::TMorfRecti(0,0,((CGLFrontEnd*)fe)->GetWidth(),((CGLFrontEnd*)fe)->GetHeight()),_dt);
	}

	void CCTL2Window::RenderScissor(TMorfRecti _scissor, int _dt)
	{
		if (HasBackground())
			((CGLFrontEnd*)fe)->GetRenderer().PushGUISurface(&surface,_scissor,0);
	}

	void CCTL2Window::CalculateSurfaces()
	{
		TMorfPointi screenPos(pos.pos);
		if (GetParent()!=NULL)
			screenPos += GetParent()->GetScreenPos();
/*		if (parent)
		{
			screenPos = parent->GetScreenPos();
			screenPos+=pos.pos;
			screenPos.x+=parent->GetBorderSize().l;
			screenPos.y+=parent->GetBorderSize().b;
			screenPos.x-=(int)parent->GetRenderOfs().x;
			screenPos.y-=(int)parent->GetRenderOfs().y;
		}*/

		surface.vertices[0].x = (float)(0+screenPos.x);
		surface.vertices[0].y = (float)(0+screenPos.y);
		surface.vertices[1].x = (float)(0+screenPos.x);
		surface.vertices[1].y = (float)(0+pos.h+screenPos.y);				
		surface.vertices[2].x = (float)(0+pos.w+screenPos.x);
		surface.vertices[2].y = (float)(0+pos.h+screenPos.y);
		surface.vertices[3].x = (float)(0+pos.w+screenPos.x);
		surface.vertices[3].y = (float)(0+screenPos.y);
	}

	void CCTL2Window::DragStart(int x, int y)
	{
		TMorfPointi p(x,y);
		ClientToScreen(p);
		dragInfo.x=p.x;
		dragInfo.y=p.y;
		dragInfo.sx=GetPos().pos.x;
		dragInfo.sy=GetPos().pos.y;
		state = STATE_MOVEING;
		SetMouseCapture();
	}
	
	void CCTL2Window::DragEnd()
	{
		state = STATE_NONE;
		ReleaseMouseCapture();
	}
	
	void CCTL2Window::DragMove(int x, int y)
	{
		TMorfPointi p(x,y);
		ClientToScreen(p);
		_Move(dragInfo.sx+(p.x-dragInfo.x), dragInfo.sy+(p.y-dragInfo.y));
	}

	ECTL2ResizeArea CCTL2Window::GetResizeArea(int x,int y)
	{
		return CTL2RESIZEAREA_NONE;
	}

	void CCTL2Window::ResizeStart(ECTL2ResizeArea _ra, int x, int y)
	{
		TMorfPointi p(x,y);
		ClientToScreen(p);
		resizeInfo.sizeTop= ((_ra==CTL2RESIZEAREA_TOPLEFT)||(_ra==CTL2RESIZEAREA_TOP)||(_ra==CTL2RESIZEAREA_TOPRIGHT));
		resizeInfo.sizeBottom= ((_ra==CTL2RESIZEAREA_BOTTOMLEFT)||(_ra==CTL2RESIZEAREA_BOTTOM)||(_ra==CTL2RESIZEAREA_BOTTOMRIGHT));
		resizeInfo.sizeLeft= ((_ra==CTL2RESIZEAREA_TOPLEFT)||(_ra==CTL2RESIZEAREA_LEFT)||(_ra==CTL2RESIZEAREA_BOTTOMLEFT));
		resizeInfo.sizeRight= ((_ra==CTL2RESIZEAREA_TOPRIGHT)||(_ra==CTL2RESIZEAREA_RIGHT)||(_ra==CTL2RESIZEAREA_BOTTOMRIGHT));
		resizeInfo.x=p.x;
		resizeInfo.y=p.y;
		resizeInfo.px=GetPos().x;
		resizeInfo.py=GetPos().y;
		resizeInfo.sx=GetPos().w;
		resizeInfo.sy=GetPos().h;
		state = STATE_RESIZING;
		SetMouseCapture();
	}

	void CCTL2Window::ResizeMove(int x, int y)
	{
		TMorfPointi p(x,y);
		ClientToScreen(p);

		TMorfRecti newPos(resizeInfo.px,resizeInfo.py,resizeInfo.sx,resizeInfo.sy);

		if (resizeInfo.sizeRight)
		{
			newPos.w+=p.x-resizeInfo.x;
		}
		if (resizeInfo.sizeTop)
		{
			newPos.h+=p.y-resizeInfo.y;
		}
		if (resizeInfo.sizeLeft)
		{
			newPos.w-=p.x-resizeInfo.x;
			newPos.x+=p.x-resizeInfo.x;
		}
		if (resizeInfo.sizeBottom)
		{
			newPos.h-=p.y-resizeInfo.y;
			newPos.y+=p.y-resizeInfo.y;
		}

		_SetPos(newPos.x, newPos.y, newPos.w, newPos.h);
	}

	void CCTL2Window::ResizeEnd()
	{
		state = STATE_NONE;
		ReleaseMouseCapture();
	}

	void CCTL2Window::SetUserdata(void *_userdata) 
	{
		userdata=_userdata;
	}

}
