#include "camera.h"
#include <math.h>

namespace FRONTEND
{
	CFrustum::~CFrustum()
	{
	}

	void CFrustum::CalculateFrustum(GENERAL::CMatrix4f &proj,GENERAL::CMatrix4f &model)
	{
		// see http://www.racer.nl/reference/vfc.htm
		GENERAL::CMatrix4f frustum(proj*model);
		GENERAL::CPlanef *p;
		GENERAL::SCALAR *m = frustum.m;

		p=planes+PLANE_RIGHT;
	    p->n.x=m[3]-m[0];
		p->n.y=m[7]-m[4];
		p->n.z=m[11]-m[8];
		p->D=m[15]-m[12];
	   
		p=planes+PLANE_LEFT;
		p->n.x=m[3]+m[0];
		p->n.y=m[7]+m[4];
		p->n.z=m[11]+m[8];
		p->D=m[15]+m[12];
	   
		p=planes+PLANE_BOTTOM;
		p->n.x=m[3]+m[1];
		p->n.y=m[7]+m[5];
		p->n.z=m[11]+m[9];
		p->D=m[15]+m[13];
	   
		p=planes+PLANE_TOP;
		p->n.x=m[3]-m[1];
		p->n.y=m[7]-m[5];
		p->n.z=m[11]-m[9];
		p->D=m[15]-m[13];
	   
		p=planes+PLANE_FAR;
		p->n.x=m[3]-m[2];
		p->n.y=m[7]-m[6];
		p->n.z=m[11]-m[10];
		p->D=m[15]-m[14];
	   
		p=planes+PLANE_NEAR;
		p->n.x=m[3]+m[2];
		p->n.y=m[7]+m[6];
		p->n.z=m[11]+m[10];
		p->D=m[15]+m[14];
	   
		// Normalize all plane normals
		for(int i=0;i<6;i++)
		{
			planes[i].Normalize();		
		}
	}

	bool CFrustum::IsPointInside(GENERAL::CVector3f &point)
	{
		if (!planes[0].Visible(point)) return false;
		if (!planes[1].Visible(point)) return false;
		if (!planes[2].Visible(point)) return false;
		if (!planes[3].Visible(point)) return false;
		if (!planes[4].Visible(point)) return false;
		if (!planes[5].Visible(point)) return false;
		return true;
	}

	bool CFrustum::IsBoxInside(GENERAL::CVector3f &min,GENERAL::CVector3f &max)
	{
		GENERAL::CVector3f p[8] = 
			{ GENERAL::CVector3f(min.x,min.y,min.z),
			  GENERAL::CVector3f(min.x,min.y,max.z),
			  GENERAL::CVector3f(min.x,max.y,min.z),
			  GENERAL::CVector3f(min.x,max.y,max.z),
			  GENERAL::CVector3f(max.x,min.y,min.z),
			  GENERAL::CVector3f(max.x,min.y,max.z),
			  GENERAL::CVector3f(max.x,max.y,min.z),
			  GENERAL::CVector3f(max.x,max.y,max.z) };

		int i;

		for (i=0; i<6; i++)
		{
			if (planes[i].Visible(p[0])) continue;
			if (planes[i].Visible(p[1])) continue;
			if (planes[i].Visible(p[2])) continue;
			if (planes[i].Visible(p[3])) continue;
			if (planes[i].Visible(p[4])) continue;
			if (planes[i].Visible(p[5])) continue;
			if (planes[i].Visible(p[6])) continue;
			if (planes[i].Visible(p[7])) continue;
			return false;
		}
		return true;
	}

/// @todo remove defaultCamerObj or set domain
	CCamera::CCamera()
	{
		attached = sptr<DOMAIN::CWorldObject>(new DOMAIN::CWorldObject(NULL));
		attached->SetPos(GENERAL::CVector3f(0.0f,0.0f,0.0f));
		attached->SetRotation(GENERAL::CVector3f(0.0f,0.0f,-1.0f));

		levelobj = new DOMAIN::CPhysicalLevelObject(NULL);
		levelobj->SetCollisionMode(DOMAIN::COLLISIONMODE_SPHERE);

		levelobj->SetBoundingRadius(1.5);


		lvlda = false;

	}

	CCamera::~CCamera()
	{
	}

	void CCamera::Look()
	{
/*		GENERAL::CVector3f &pos = attached->GetPos();
		GENERAL::CVector3f vec(pos+(attached->GetRotation()));
		
		gluLookAt(pos.x,pos.y,pos.z,vec.x,vec.y,vec.z,0.0f,1.0f,0.0f);*/

		GENERAL::CVector3f pos = attached->GetPos();

		pos.y += 25;

		GENERAL::CVector3f dir = attached->GetRotation();
		
		
		if (lvlda) 
		{
			cpos = levelobj->GetLvl()->Trace(&levelobj,pos,pos-(dir*150));
		} else
		{
			cpos = pos-(dir*150);
		}



		gluLookAt(cpos.x,cpos.y,cpos.z,pos.x,pos.y,pos.z,0.0f,1.0f,0.0f);
		//gluLookAt(cpos.x,cpos.y,cpos.z,pos.x,pos.y,pos.z,0.0f,1.0f,0.0f);

		//gluLookAt(cpos.x,cpos.y+50,cpos.z,pos.x,pos.y+25,pos.z,0.0f,1.0f,0.0f);

//		cpos = pos;
//		gluLookAt(cpos.x,cpos.y,cpos.z,pos.x+dir.x,pos.y+dir.y,pos.z+dir.z,0.0f,1.0f,0.0f);

		//cpos=GENERAL::CVector3f(650.0f,50.0f,-650.0f);
		GENERAL::CMatrix4f proj;
		GENERAL::CMatrix4f model;

		glGetFloatv(GL_PROJECTION_MATRIX,proj.m);
		glGetFloatv(GL_MODELVIEW_MATRIX,model.m);

		frustum.CalculateFrustum(proj,model);
	}
}
