#ifndef GLFE_H
#define GLFE_H

#include "fmod.h"
#include "fmod_errors.h"

#pragma comment (lib,"fmodvc.lib")
#pragma comment (lib,"opengl32.lib")
#pragma comment (lib,"glu32.lib") 
#pragma comment (lib,"glaux.lib") 
#pragma comment (lib,"odbc32.lib") 
#pragma comment (lib,"odbccp32.lib")


#include <windows.h>		// Header File For Windows
#include <gl\gl.h>			// Header File For The OpenGL32 Library
#include <gl\glu.h>			// Header File For The GLu32 Library
#include <gl\glaux.h>

#include <morfbase.h>
#include "resman.h"
#include "camera.h"
#include "renderer.h"
#include "../gl/glextensions.h"
#include "../control2/CTL2.h"
#include "../../domain/implementation/objectdomain.h"
#include "../dialogs2/DLG2Console.h"
#include "../dialogs2/DLG2Status.h"
#include "../dialogs2/DLG2ActionPopup.h"
#include "../dialogs2/DLG2IconContainer.h"
#include "../dialogs2/DLG2ChatWindow.h"
#include "../dialogs2/DLG2FSMInfo.h"
#include "../dialogs2/DLG2PlayerSelection.h"



namespace FRONTEND
{
	extern int numSpiders;
	class CGLFrontEnd;
	class CGLLoadingScreenGL
	{
	protected:
		CGLFrontEnd *fe;

		TSurface splash;
		HResShader loadingSplash;
		HResShader loading;

		HResTTFontGL fnt;

		char *statusString;


	public:
		CGLLoadingScreenGL(CGLFrontEnd *_fe);

		virtual void Render(int _dt);
		inline void SetStatusString(char *_st) { statusString = _st; }; 
	};

	class CGLFrontEnd : public CFrontEnd
	{
	public:
		enum ERenderMode {	RENDERMODE_PERSPECTIVE=0,
							RENDERMODE_RASTER2D=1};	
						
	private:

		LPCSTR szClassName;
		HDC			hDC;		
		HGLRC		hRC;		
		HWND		hWnd;		
		HINSTANCE	hInstance;	

		int height,width;
		float fps;

		bool fullscreen;

		ERenderMode renderMode;

		APPLICATION::CWinApplication *app;
		
		DOMAIN::CObjectDomain *domain;
		DOMAIN::CObjectDomainActions &domainActions;

		CResManagerGL *res;

		HResGUIStyle guiStyle;
		sptr<CCTL2Container>		rootui2;
//R		sptr<CCTLContainer>			rootui;
		sptr<CDLG2ActionPopUp>		popup;
		sptr<CDLG2PlayerSelection>	pselection;
		sptr<CDLG2Spiderquest>		pspider;
//I		sptr<CSGL2IconContainer>	ic;
		sptr<CDLG2Console>			console;
		sptr<CDLG2Status>			statusDialog;
		
		CCamera camera;
		CRendererGL *renderer;
		CGLLoadingScreenGL *loadingSrceen;


		DOMAIN::CDomain::TObjectList *objects;
		
		bool InitGL();

	protected:
		virtual void CreateIconContainer(DOMAIN::CGraphObject *go);

	public:

		virtual void SpiderDied();

		CGLFrontEnd(APPLICATION::CWinApplication *_app, HINSTANCE _hinst, DOMAIN::CObjectDomain *_domain);
		virtual ~CGLFrontEnd();

		virtual void SetMouseCapture();
		virtual void ReleaseMouseCapture();
		virtual void MouseMove(INT x,INT y,INT z,TMouseButtons &buttons,TKeyStates &keys);
		virtual void MouseDown(EMouseButton button, INT x,INT y,INT z,TMouseButtons &buttons,TKeyStates &keys);
		virtual void MouseUp(EMouseButton button, INT x,INT y,INT z,TMouseButtons &buttons,TKeyStates &keys);
		virtual void MouseClick(EMouseButton button, INT x,INT y,INT z,TMouseButtons &buttons,TKeyStates &keys);
		virtual void KeyDown(int key,TKeyStates &keystate);
		virtual void KeyUp(int key,TKeyStates &keystate);
		virtual void KeyPress(int key,TKeyStates &keystate);
		virtual void Resize(INT x, INT y);

		virtual void ShowMessageBox(char *_title, char *_message);


		virtual bool Init();
		virtual void ReSizeWindow(int width, int height);
		virtual void KillWindow();
		virtual bool CreateFeWindow(
			LPCSTR title,
			int width, int height, int bits, 
			bool fullscreenflag
			
			);

		virtual void Render(DOMAIN::CDomain::TObjectList *_obj);
		virtual CRenderAble *RenderSelect(int x, int y);
		virtual int RenderSelectDetail(int x, int y, CRenderAble *_obj);

		virtual void SetRENDERMODE_RASTER2D();
		virtual void SetRENDERMODE_PERSPECTIVE();


		inline CResManagerGL *GetResMan() { return res; };
		inline int GetHeight() { return height; };
		inline int GetWidth() { return width; };
		inline CCamera &GetCamera() { return camera; };
		inline CRendererGL &GetRenderer() { return *renderer; };

//		virtual CRenderAble *CreateObject(DOMAIN::CGraphObject *_from);
		virtual sptr<CRenderAble> CreateObject(DOMAIN::CGraphObject *_from, int _typ, int iParam=0, void *pParam=NULL);

		virtual void SetCamera(sptr<DOMAIN::CWorldObject> _obj);


		//helpers

		//static void DrawStyledQuad(UIRECT &pos, char *_styleElement);
		

		virtual void DumpGLState();

		virtual void CreateUIObject(DOMAIN::CGraphObject *_from, int _uitype, int lParam = 0, void *pParam = 0);
		virtual void PopUp(CRenderAble *ro, DOMAIN::CGameObject::tActionList &_actionMask);
		virtual sptr<CDLG2ActionPopUp> &GetPopUp() {return popup;};
		virtual void ExecuteCommand(char *cmd);


		static bool __cdecl staticRootUIHandler(CCTL2Window *control, ECTL2CallBackAction action, unsigned long int lparam, void *pparam, void *userdata);
		bool __cdecl RootUIHandler(CCTL2Window *control, ECTL2CallBackAction action, unsigned long int lparam, TCTL2PParam *pparam);

		void PreloadResources();

	public:
		FSOUND_STREAM *backgroundStream;

		virtual bool InitSoundEngine();
		FSOUND_STREAM *CreateSound(char *filename, bool loop);
		int PlaySound(FSOUND_STREAM *sound, int channel = FSOUND_FREE);
		void FreeSound(FSOUND_STREAM *sound);
		virtual void CloseSoundEngine();

		inline DOMAIN::CObjectDomain *GetDomain() { return domain; };

		virtual void SelectionChanged(DOMAIN::CGameObject *_old, DOMAIN::CGameObject *_new);

	};





}


#endif