#include "font.h"

#include <morfbase.h>
#include <stdio.h>
#include <glheaders.h>
#include "../implementation/resman.h"
#include "../gl/glextensions.h"

#include <trace.h>

namespace FRONTEND
{
	CResFont::~CResFont()
	{
	}

	int CResFont::CalculateMultilinedText(
			GENERAL::TMorfPointf &size,
			char *string,
			EHAlign halign,
			EVAlign valign,
			bool wordwrap,
			bool multiline,
			TLineInfoList &lineinfo)
	{
		int linecnt=1;
		char *c;
		char *lastWord=string;
		char *lastWordEnd=string;
		bool lastCharWasSpace=true;
		int x=0;
		int wordsInLine=0;
		int wordWidth=0;
		int charWidth=GetCharWidth('i');
		int spaceWidth=0;

		lineinfo.clear();
		lineinfo.push_back(TLineInfo(0,-1,-1,GENERAL::CVector3f(-1,-1,-1)));

		for (c=string; (*c)!='\0'; c++)
		{
			charWidth = GetCharWidth(*c);
			if (!isspace(*c))
			{
				if (lastCharWasSpace)
				{
					lastWord=c;
					wordsInLine++;
					wordWidth=0;
				}
			} else {
				if ( (!lastCharWasSpace) && (isspace(*c)) )
				{
					lastWordEnd=c;
					lastCharWasSpace=true;
					spaceWidth=0;
				}
				spaceWidth+=charWidth;
			}

			wordWidth+=charWidth;

			if ((multiline) && (*c=='\n'))
			{
				lineinfo[linecnt-1].ccnt=(int)(c-string)-lineinfo[linecnt-1].cofs;
				lineinfo[linecnt-1].width=x;
				lineinfo.push_back(TLineInfo((int)(c-string)+1,-1,-1,GENERAL::CVector3f(-1,-1,-1)));

				linecnt++;
				wordsInLine=0;
				x=0;
				lastCharWasSpace=true;
				continue;
			}

			x+=charWidth;
			if ((multiline) && (x>size.x))
			{
				if ((!isspace(*c)) && (wordwrap) && (wordsInLine>1))
				{
					lineinfo[linecnt-1].ccnt=(int)(lastWordEnd-string)-lineinfo[linecnt-1].cofs;
					lineinfo[linecnt-1].width=x-wordWidth-spaceWidth;
					lineinfo.push_back(TLineInfo((int)(lastWord-string),-1,-1,GENERAL::CVector3f(-1,-1,-1)));

					x=wordWidth;
					wordsInLine=1;				
				} else {
					if (wordwrap)
					{
						while (isspace(*c)&&(*(c+1)!='\0')) c++;
					}
					lineinfo[linecnt-1].ccnt=(int)(c-string)-lineinfo[linecnt-1].cofs;
					lineinfo[linecnt-1].width=x-charWidth;
					lineinfo.push_back(TLineInfo((int)(c-string),-1,-1,GENERAL::CVector3f(-1,-1,-1)));

					x=charWidth;
					wordsInLine = isspace(*c)?0:1;
				}

				linecnt++;
			}

			lastCharWasSpace=(isspace(*c)!=0);
		}
		lineinfo[linecnt-1].ccnt=(int)(c-string)-lineinfo[linecnt-1].cofs;
		lineinfo[linecnt-1].width=x;

		GENERAL::SCALAR height=(GENERAL::SCALAR)(GetHeight()*linecnt);
		GENERAL::SCALAR starty;
		switch(valign)
		{
		case VALIGN_TOP:	starty=size.y-GetHeight(); 
							break;
		case VALIGN_CENTER: starty=(size.y+height)/2.0f-(GENERAL::SCALAR)GetHeight();
							break;
		case VALIGN_BOTTOM:	starty=height-(GENERAL::SCALAR)GetHeight();
							break;
		}
		starty+=+GetDescent();

		TLineInfoList::iterator it;

		for (it=lineinfo.begin(); it!=lineinfo.end(); it++)
		{
			it->pos.y=starty;
			it->pos.z=0;
			starty-=GetHeight();

			switch (halign)
			{
			case HALIGN_LEFT:	it->pos.x = 0;
								break;
			case HALIGN_CENTER: it->pos.x = (size.x-(GENERAL::SCALAR)it->width)/2.0f;
								break;
			case HALIGN_RIGHT:	it->pos.x = size.x-(GENERAL::SCALAR)it->width;
								break;
			}
		}

		return linecnt;
	}

	CResTTFontGL::CResTTFontGL(CResManagerGL *_resMan,char *_fontname, int _size, int _targetstatus) 
		: CResFont(_resMan, _fontname, _targetstatus),glyphMetrics(NULL)
	{
		size = _size;
		if (_targetstatus!=RESSTATUS_NOTLOADED) LoadFromFile(_fontname, _targetstatus);
	}

	CResTTFontGL::~CResTTFontGL()
	{
		if (font!=0) DeleteObject(font);
		if (glyphMetrics) free(glyphMetrics);
	}

	void CResTTFontGL::SetupRenderStatesPrint(bool pushAttrib)
	{
		if (pushAttrib) 
			glPushAttrib(GL_ENABLE_BIT);
		glDisable(GL_TEXTURE_2D);
		glEnable(GL_BLEND);
		glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
	}

	void CResTTFontGL::SetupRenderStatesMark(bool pushAttrib)
	{
		if (pushAttrib) 
			glPushAttrib(GL_ENABLE_BIT);
		glLogicOp(GL_INVERT);
		glEnable(GL_COLOR_LOGIC_OP);
	}


	void CResTTFontGL::Print(char *string)
	{	
		SetupRenderStatesPrint(true);
		glRasterPos2i(0,descent);
		glListBase(base - 32);								
		glCallLists((GLsizei)strlen(string), GL_UNSIGNED_BYTE, string);
		glPopAttrib();
	}


/*	void CResTTFontGL::Print(GENERAL::TMorfPointf *pos, char *string, int chars, bool mark)
	{
		static unsigned char bitmapmark[128]=
		{
			0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
			0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
			0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
			0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
			0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
			0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
			0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
			0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff	};

/ *		glPushAttrib(GL_LIST_BIT | GL_ENABLE_BIT);							
		glDisable(GL_TEXTURE_2D);
		glEnable(GL_BLEND);* /
		SetupRenderStatesPrint(true);

		if (pos!=NULL) 
			glRasterPos2i((GLint)pos->x,(GLint)pos->y+descent);

		if (mark)
			glPushAttrib(GL_CURRENT_BIT);

		glListBase(base - 32);								
		glCallLists(chars, GL_UNSIGNED_BYTE, string);

		if (mark)
		{
			glPopAttrib();

			SetupRenderStatesMark(false);

			glPixelStorei(GL_UNPACK_ALIGNMENT,1);
			for (int i=0; i<chars; i++)
			{
				glColor4ub(255,255,255,255);
				glBitmap(
					(GLsizei)GetCharWidth(string[i]),(GLsizei)GetHeight(),
					0.0f,(GLfloat)descent,
					(GLfloat)GetCharWidth(string[i]),0.0f,
					bitmapmark);
			}
		}		
		glPopAttrib();
	}*/

	void CResTTFontGL::Print(GENERAL::CVector3f *pos, char *string, int chars, bool mark)
	{
		static unsigned char bitmapmark[128]=
		{
			0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
			0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
			0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
			0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
			0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
			0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
			0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
			0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff	};

		SetupRenderStatesPrint(true);

		if (pos!=NULL) 
		{
			glRasterPos3f(pos->x,pos->y,pos->z);
/*			glBitmap(
				0,0,
				0.0f,0.0f,
				0.0f,(float)descent,
				NULL);*/
//			glRasterPos3f(pos->x,pos->y+(float)descent,pos->z);
		}

		if (mark)
			glPushAttrib(GL_CURRENT_BIT);

		glListBase(base - 32);								
		glCallLists(chars, GL_UNSIGNED_BYTE, string);

		if (mark)
		{
			glPopAttrib();

			SetupRenderStatesMark(false);

			glPixelStorei(GL_UNPACK_ALIGNMENT,1);
			for (int i=0; i<chars; i++)
			{
				glColor4ub(255,255,255,255);
				glBitmap(
					(GLsizei)GetCharWidth(string[i]),(GLsizei)GetHeight(),
					-1,(GLfloat)descent,
					(GLfloat)GetCharWidth(string[i]),0.0f,
					bitmapmark);
			}
		}		
		glPopAttrib();
	}

	void CResTTFontGL::PrintCursor()
	{
		SetupRenderStatesPrint(true);
		glCallList(listCursor);
		glPopAttrib();
	}

	void CResTTFontGL::Printf(char *string,...)
	{
		char		text[512];										
		va_list		ap;												

		if (string == NULL)											
			return;													

		va_start(ap, string);										
			vsprintf(text, string, ap);								
		va_end(ap);	
		Print(text);
	}

	int CResTTFontGL::GetWidth(char *string)
	{
		int result = 0;
		for (int i=0; string[i]!='\0'; i++)
		{
			result+=glyphMetrics[string[i]-32].gmCellIncX;
		};
		return result;
	}

	int CResTTFontGL::GetDescent()
	{
		return descent;
	}
	int CResTTFontGL::GetHeight()
	{
		return height;
	}
	
	int CResTTFontGL::GetCharWidth(char c)
	{
		return ((c>=32)&&(c<96+32))?glyphMetrics[c-32].gmCellIncX:0;
	}

	bool CResTTFontGL::LoadFromFile(char *_file, int _targetstatus)
	{
		HFONT	oldfont;

		font = CreateFont( 	-size,							
							0,								
							0,								
							0,								
							FW_NORMAL,						
							FALSE,							
							FALSE,							
							FALSE,							
							ANSI_CHARSET,					
							OUT_TT_PRECIS,					
							CLIP_DEFAULT_PRECIS,			
							ANTIALIASED_QUALITY,			
							FF_DONTCARE|DEFAULT_PITCH,
							_file);


		if (font==0) return FALSE;

		TEXTMETRIC metrics;
		oldfont = (HFONT)SelectObject(((CResManagerGL*)resMan)->GetHDC(), font);

//		GetCharWidth32(((CResManagerGL*)resMan)->GetHDC(),0,95, charWidth);
		GetTextMetrics(((CResManagerGL*)resMan)->GetHDC(),&metrics);

		ascent=metrics.tmAscent;
		descent=metrics.tmDescent;
		height=metrics.tmHeight;

	
		MAT2 m;
		m.eM11.fract=0;	m.eM11.value=1;
		m.eM22.fract=0;	m.eM22.value=1;
		m.eM12.fract=0;	m.eM12.value=0;
		m.eM21.fract=0;	m.eM21.value=0;

		glyphMetrics = (GLYPHMETRICS*)malloc(sizeof(GLYPHMETRICS)*96);

		for (int i=0; i<96;i++)
		{
			GetGlyphOutline(
				((CResManagerGL*)resMan)->GetHDC(),
				32+i,
				GGO_METRICS,
				glyphMetrics+i,
				0,
				NULL,
				&m);
		}

		SelectObject(((CResManagerGL*)resMan)->GetHDC(), oldfont);	

		status = RESSTATUS_INMEMORY;

		if ((_targetstatus==RESSTATUS_INMEMORYANDGA) ||
			(_targetstatus==RESSTATUS_ONLYINGA))
			return LoadIntoGraphicAdapter(_targetstatus);
			else return TRUE;
	}
	
	bool CResTTFontGL::LoadIntoGraphicAdapter(int _targetsatus)
	{
		base = glGenLists(96);

		HFONT oldfont;

		oldfont = (HFONT)SelectObject(((CResManagerGL*)resMan)->GetHDC(), font);
		wglUseFontBitmaps(((CResManagerGL*)resMan)->GetHDC(), 32, 96, base);
		SelectObject(((CResManagerGL*)resMan)->GetHDC(), oldfont);	

		status = RESSTATUS_INMEMORYANDGA;

		GLubyte *cursorBitmap = (GLubyte*)malloc(sizeof(GLubyte)*GetHeight());
		memset(cursorBitmap,0xff,sizeof(GLubyte)*GetHeight());
		listCursor = glGenLists(1);
		glPixelStorei(GL_UNPACK_ALIGNMENT,1);
		glNewList(listCursor,GL_COMPILE);
			glBitmap(1,GetHeight(),0,0,0,0,cursorBitmap);
		glEndList();
		free(cursorBitmap);

		if (_targetsatus==RESSTATUS_ONLYINGA)
		{
			DeleteObject(font);
			font =  0;
			status = RESSTATUS_ONLYINGA;
		}



		return TRUE;
	}
	
	bool CResTTFontGL::UnloadFromGraphicAdapter()
	{
		glDeleteLists(base,96);
		if (status==RESSTATUS_ONLYINGA) status=RESSTATUS_NOTLOADED;
		else status=RESSTATUS_INMEMORY;

		return TRUE;
	}

	void CResTTFontGL::PrintBox(
			GENERAL::TMorfPointf &size,
			char *string,
			EHAlign halign,
			EVAlign valign,
			bool multiline,
			bool wordwrap)
	{
		TLineInfoList li;
		li.clear();
		int linecnt = CalculateMultilinedText(
			size,
			string,
			halign,
			valign,
			wordwrap,
			multiline,li);

		TLineInfoList::iterator it;


		SetupRenderStatesPrint(true);
		glListBase(base - 32);								
		for (it=li.begin(); it!=li.end(); it++)
		{
			TLineInfo &info=*it;
			glRasterPos2f(info.pos.x,info.pos.y+descent);
			glCallLists(info.ccnt, GL_UNSIGNED_BYTE, string+info.cofs);
		}
		glPopAttrib();
	}

	CResBMPFontGL::CResBMPFontGL(CResManagerGL *_resMan,char *_fontname, int _targetstatus)
		: CResTTFontGL(_resMan,NULL,0,RESSTATUS_NOTLOADED)
	{
		if (_targetstatus!=RESSTATUS_NOTLOADED) LoadFromFile(_fontname, _targetstatus);
	}

	bool CResBMPFontGL::LoadFromFile(char *_file, int _targetstatus)
	{
		FILE *f=GENERAL::morfOpenFile(_file,"rb");

		if (f)
		{
			fread(&header,sizeof(header),1,f);
			if ( (header.id[0]=='F') && 
				 (header.id[1]=='N') && 
				 (header.id[2]=='T') )
			{
				fseek(f,header.charofs,SEEK_SET);

				data = (char**)malloc(sizeof(char*)*header.charcount);
				glyphMetrics = (GLYPHMETRICS*)malloc(sizeof(GLYPHMETRICS)*header.charcount);

				height=header.height;
				ascent=header.ascent;
				descent=header.descent;

				for (int i=0; i<header.charcount; i++)
				{
					long datasize;
					fread(glyphMetrics+i,sizeof(GLYPHMETRICS),1,f);
					fread(&datasize,sizeof(datasize),1,f);
					if (datasize>0)
					{
						data[i] = (char*)malloc(datasize);
						fread(data[i],1,datasize,f);
					} else data[i]=NULL;
					glyphMetrics[i].gmptGlyphOrigin.x-=1;
					glyphMetrics[i].gmCellIncX+=1;
				}

				fclose(f);
				status = RESSTATUS_INMEMORY;

				if ((_targetstatus==RESSTATUS_INMEMORYANDGA) ||
					(_targetstatus==RESSTATUS_ONLYINGA))
					return LoadIntoGraphicAdapter(_targetstatus);
					else return true;
			}

			fclose(f);
			return false;
		}
		return false;
	}

	bool CResBMPFontGL::LoadIntoGraphicAdapter(int _targetsatus)
	{
	/*	static GLubyte bitmap[128]=
		{ 0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
		  0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
		  0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
		  0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
		  0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
		  0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
		  0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
		  0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff };
*/
		glPixelStorei(GL_UNPACK_ALIGNMENT,4);
		base = glGenLists(header.charcount);
		unsigned long int dummydata=0;
		for (int i=0; i<header.charcount; i++)
		{
			glNewList(base+i,GL_COMPILE);
				if ((header.format==0) || (data[i]==NULL))
				{
					glBitmap(
						glyphMetrics[i].gmBlackBoxX,
						glyphMetrics[i].gmBlackBoxY,
						(GLfloat)glyphMetrics[i].gmptGlyphOrigin.x,
						(GLfloat)glyphMetrics[i].gmptGlyphOrigin.y,
						(GLfloat)glyphMetrics[i].gmCellIncX,
						(GLfloat)glyphMetrics[i].gmCellIncY,
						(data[i]==0?(GLubyte*)&dummydata:(GLubyte*)data[i]));
				} else if (header.format==3)
				{
					glBitmap(
						0,0,0.0f,0.0f,
						(GLfloat)-glyphMetrics[i].gmptGlyphOrigin.x,
						(GLfloat)-glyphMetrics[i].gmptGlyphOrigin.y,
						NULL);
					glDrawPixels(
						(GLuint)glyphMetrics[i].gmBlackBoxX,
						(GLuint)glyphMetrics[i].gmBlackBoxY,
						GL_ALPHA,GL_UNSIGNED_BYTE,data[i]);
					glBitmap(
						0,0,0.0f,0.0f,
						(GLfloat)(glyphMetrics[i].gmCellIncX+glyphMetrics[i].gmptGlyphOrigin.x),
						(GLfloat)(glyphMetrics[i].gmCellIncY+glyphMetrics[i].gmptGlyphOrigin.y),
						NULL);
				}
			glEndList();
		}

		GLubyte *cursorBitmap = (GLubyte*)malloc(sizeof(GLubyte)*GetHeight());
		memset(cursorBitmap,0xff,sizeof(GLubyte)*GetHeight());
		listCursor = glGenLists(1);
		glPixelStorei(GL_UNPACK_ALIGNMENT,1);
		glNewList(listCursor,GL_COMPILE);
			glBitmap(1,GetHeight(),0.0f,(GLfloat)descent,0.0f,0.0f,cursorBitmap);
		glEndList();
		free(cursorBitmap);

		status = RESSTATUS_INMEMORYANDGA;

		if (_targetsatus==RESSTATUS_ONLYINGA)
		{
			for (i=0; i<header.charcount; i++)
			{
				if (data[i]) free(data[i]);
			}
			free(data);
			status = RESSTATUS_ONLYINGA;
		}

		return true;
	}

	void CResBMPFontGL::SetupRenderStatesPrint(bool pushAttrib)
	{
		if (header.format==0)
		{
			CResTTFontGL::SetupRenderStatesPrint(pushAttrib);
		} else {
			if (pushAttrib) 
				glPushAttrib(GL_ENABLE_BIT | GL_PIXEL_MODE_BIT);

			glDisable(GL_TEXTURE_2D);
			glEnable(GL_BLEND);
			glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);

			
			glEnable(GL_ALPHA_TEST);
			glAlphaFunc(GL_NOTEQUAL,0.0f);
			static GLfloat curColor[4]={1.0,1.0,1.0,1.0};
			glGetFloatv(GL_CURRENT_COLOR,curColor);
			glPixelTransferf(GL_RED_BIAS,curColor[0]);
			glPixelTransferf(GL_GREEN_BIAS,curColor[1]);
			glPixelTransferf(GL_BLUE_BIAS,curColor[2]);
			glPixelTransferf(GL_ALPHA_SCALE,curColor[3]);
		}
	}


/*	CResBMPFontGL::CResBMPFontGL(CResManagerGL *_resMan,char *_fontname, int _targetstatus) 
		: CResFont(_resMan, _fontname, _targetstatus)
	{
		if (_targetstatus!=RESSTATUS_NOTLOADED) LoadFromFile(_fontname, _targetstatus);
	}

	CResBMPFontGL::~CResBMPFontGL()
	{
	}

	void CResBMPFontGL::Print(char *string)
	{
		glEnable(GL_TEXTURE_2D);
		glBindTexture(GL_TEXTURE_2D, texture->GetTextureName());				
		
		glPushMatrix();												

		glDepthFunc(GL_ALWAYS);	
		glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
		glEnable(GL_BLEND);
		
		glListBase(base-32);										
		glCallLists((GLsizei)strlen(string), GL_UNSIGNED_BYTE, string);		

		glPopMatrix();												
	}

	void CResBMPFontGL::Print(GENERAL::TMorfPointf *pos, char *string, int chars, bool mark)
	{
		glPushMatrix();
		glTranslatef((GLfloat)pos->x,(GLfloat)pos->y,0.0f);
		Print("<CResTTFontGL::Print not implement yet>");
		glPopMatrix();
	}

	void CResBMPFontGL::Printf(char *string,...)
	{
		char		text[512];										
		va_list		ap;												

		if (string == NULL)											
			return;													

		va_start(ap, string);										
			vsprintf(text, string, ap);								
		va_end(ap);	
		Print(text);
	}

	int CResBMPFontGL::GetWidth(char *string)
	{
		int result = 0;
		for (int i=0; string[i]!='\0'; i++)
		{
			result+=charWidth[string[i]-32];
		};
		return result;
	}
	
	int CResBMPFontGL::GetHeight()
	{
		//return size;
		return 0;
	}
	
	int CResBMPFontGL::GetCharWidth(char c)
	{
		return charWidth[c-32];
	}

	bool CResBMPFontGL::LoadFromFile(char *_file, int _targetstatus)
	{
		char datname[255];

		datname[0] = '\0';

		strncat(datname,  _file, strlen(_file)-4);
		strcat(datname, ".dat");

		FILE *ini = fopen(datname, "rb");

		memset(charWidth, 32, 256);
		if (ini != 0) 
		{
			fread(charWidth, sizeof(unsigned char), 256, ini);
			fclose(ini);
		};

		texture = (CResTextureGL*)resMan->GetPtr(_file);

		if ((_targetstatus==RESSTATUS_INMEMORYANDGA) ||
			(_targetstatus==RESSTATUS_ONLYINGA))
		return LoadIntoGraphicAdapter(_targetstatus);
			else return TRUE;
	}
	
	bool CResBMPFontGL::LoadIntoGraphicAdapter(int _targetsatus)
	{
		static GLfloat offsetx=-1.0f;
		static GLfloat offsety=4.0f;
		
		base=glGenLists(95);										
		
		int i = texture->GetTextureName();

		glBindTexture(GL_TEXTURE_2D, texture->GetTextureName());
		
		for (int loop=0; loop<95; loop++)							
		{

			glCullFace(GL_FRONT);

			float cx=((float)(loop%16))/16.0f;				
			float cy=((float)((loop)/16))/16.0f;	
			float xofs=16.0f-((float)charWidth[loop])/2.0f; 

			glNewList(base+loop, GL_COMPILE);			
				glBegin(GL_QUADS);	
					glTexCoord2f(cx+xofs/512.0f,		1-cy-0.0625f);		
					glVertex2f(1+offsetx,32+offsety);
					glTexCoord2f(cx+0.0625f-xofs/512.0f,1-cy-0.0625f);
					glVertex2f(33+offsetx-xofs*2,32+offsety);
					glTexCoord2f(cx+0.0625f-xofs/512.0f,1-cy);		
					glVertex2f(33+offsetx-xofs*2,0+offsety);					
					glTexCoord2f(cx+xofs/512.0f,		1-cy);				
					glVertex2f(1+offsetx,0+offsety);					
				glEnd();								
				glTranslatef(charWidth[loop],0,0);				
			glEndList();								

			glCullFace(GL_BACK);
		}


		status = RESSTATUS_INMEMORYANDGA;

		if (_targetsatus==RESSTATUS_ONLYINGA)
		{
			status = RESSTATUS_ONLYINGA;
		}

		return TRUE;
	}
	
	bool CResBMPFontGL::UnloadFromGraphicAdapter()
	{
		glDeleteLists(base,95);
		if (status==RESSTATUS_ONLYINGA) status=RESSTATUS_NOTLOADED;
		else status=RESSTATUS_INMEMORY;

		return TRUE;
	}*/



}