#ifndef RES_FONT_H
#define RES_FONT_H

#include <morfbase.h>
#include <glheaders.h>
#include <vector>

#include "texture.h"

namespace FRONTEND
{
	typedef enum {HALIGN_CENTER,HALIGN_LEFT,HALIGN_RIGHT} EHAlign;
	typedef enum {VALIGN_CENTER,VALIGN_TOP,VALIGN_BOTTOM} EVAlign;


	class CResManagerGL;

	struct TLineInfo
	{
	public:
		inline TLineInfo(int _cofs, int _ccnt, int _width, GENERAL::CVector3f _pos)
			: cofs(_cofs), ccnt(_ccnt), pos(_pos), width(_width) {};
		inline TLineInfo(const TLineInfo &li)
			: cofs(li.cofs), ccnt(li.ccnt), pos(li.pos), width(li.width) {}

		inline TLineInfo &operator = (const TLineInfo &li)
		{
			cofs = li.cofs;
			ccnt = li.ccnt;
			pos = li.pos;
			width = li.width;

			return *this;
		}

		int cofs;
		int ccnt;
		int width;
		GENERAL::CVector3f pos;
	};

	typedef std::vector<TLineInfo> TLineInfoList;

	class CResFont : public CFEResource
	{
	public:

		inline CResFont(CResourceManager *_resMan,char *_file, int _targetstatus) : CFEResource(_resMan, _file, _targetstatus) {};

		virtual ~CResFont();
	
		virtual void Print(char *string) = 0;

		virtual void Print(GENERAL::CVector3f *pos, char *string, int chars, bool mark) = 0;
		virtual void Printf(char *string,...) = 0;
		virtual void PrintCursor() = 0;

		virtual int GetWidth(char *string) = 0;
		
		virtual int GetDescent() = 0;
		virtual int GetHeight() = 0;
		virtual int GetCharWidth(char c) = 0;

		virtual int CalculateMultilinedText(
			GENERAL::TMorfPointf &size,
			char *string,
			EHAlign hAlign,
			EVAlign vAlign,
			bool wordwrap,
			bool multiline,
			TLineInfoList &lineinfo);
	};

	class CResTTFontGL : public CResFont
	{
	private:
		int size;		
	protected:

	public:
		GLuint base;
		GLuint listCursor;
		GLYPHMETRICS *glyphMetrics;

		HFONT	font;	

		int descent;
		int ascent;
		int height;

	public:

		CResTTFontGL(CResManagerGL *_resMan,char *_fontname, int _size, int _targetstatus);
			

		virtual ~CResTTFontGL();
	
		virtual void SetupRenderStatesPrint(bool pushAttrib);
		virtual void SetupRenderStatesMark(bool pushAttrib);

		virtual void Print(char *string);
		virtual void Print(GENERAL::CVector3f *pos, char *string, int chars, bool mark);
		virtual void Printf(char *string,...);
		// Prints a text in a box from (0,0,size.x,size.y)
		virtual void PrintBox(
			GENERAL::TMorfPointf &size,
			char *string,
			EHAlign halign,
			EVAlign valign,
			bool wordwrap,
			bool multiline);
		virtual void PrintCursor();


		virtual int GetWidth(char *string);
		
		virtual int GetHeight();
		virtual int GetCharWidth(char c);
		virtual int GetDescent();

		virtual bool LoadFromFile(char *_file, int _targetstatus);
		virtual bool LoadIntoGraphicAdapter(int _targetsatus);
		virtual bool UnloadFromGraphicAdapter();


	};

	class CResBMPFontGL : public CResTTFontGL
	{
	protected:
		struct THeader
		{
			char id[3];
			long firstchar;
			long charcount;
			long format;  
					// 0 ... Bitmap
					// 1 ... 2 Bit Grayscale
					// 2 ... 8 Bit Alpha
			long alignment;
			long height;
			long ascent;
			long descent;
			long charofs;
		};

		struct TGlyphData
		{
			GLYPHMETRICS metrics;
			int datasize;
			char *data;
		};

	public:
		THeader header;
		char **data;
	public:
		CResBMPFontGL(CResManagerGL *_resMan,char *_fontname, int _targetstatus);
		virtual bool LoadFromFile(char *_file, int _targetstatus);
		virtual bool LoadIntoGraphicAdapter(int _targetsatus);
		virtual void SetupRenderStatesPrint(bool pushAttrib);

	};

/*
	class CResBMPFontGL : public CResFont
	{
	protected:

		GLuint base;
		unsigned char charWidth[256];
		int height;
		int size;

		CResTextureGL *texture;

	public:

		CResBMPFontGL(CResManagerGL *_resMan,char *_fontname, int _targetstatus);
			

		virtual ~CResBMPFontGL();
	
		virtual void Print(char *string);
		virtual void Print(GENERAL::TMorfPointf *pos, char *string, int chars, bool mark);
		virtual void Printf(char *string,...);

		virtual int GetWidth(char *string);
		
		virtual int GetHeight();
		virtual int GetCharWidth(char c);

		virtual bool LoadFromFile(char *_file, int _targetstatus);
		virtual bool LoadIntoGraphicAdapter(int _targetsatus);
		virtual bool UnloadFromGraphicAdapter();


	};*/



	typedef sptr<CResFont> HResFont;
	typedef sptr<CResTTFontGL> HResTTFontGL;
	typedef sptr<CResBMPFontGL> HResBMPFontGL;




}
#endif