#ifndef WORLD_H
#define WORLD_H

#include <list>
#include <map>

//#include "../../general/base/morfwindows.h"
#include "../../general/base/morfforward.h"
#include "../../general/base/morfmodules.h"
#include "../implementation/domaincommand.h"
#include "../objects/objects.h"

#define DOMAINCMD_CONTROLLOBJECT		1
#define DOMAINCMD_PINGREQUEST			2
#define DOMAINCMD_PINGREPLAY			3
#define DOMAINCMD_SYNCHTIME				4
#define DOMAINCMD_GOTCONTROLLOBJECT     5
#define DOMAINCMD_PDU					6

namespace DOMAIN 
{

	class CDomainException : public GENERAL::CException
	{
	public:
		#ifdef _DEBUG
			inline CDomainException(char *what, char *file, int line) : CException(what,file,line) {}
		#else
			inline CDomainException(char *what) : CException(what) {}
		#endif
	};

	#define DOMAINTHROW(message) MORFTHROW(message,DOMAIN::CDomainException)

	class CAction : public GENERAL::CSerializeableObject
	{
	public:
		DECLARE_SERIALIZEABLE(CAction);
		virtual ~CAction();
		virtual void execute();
	};
		
	class CDomain 
	{
	public:
		// Typedefs
		typedef struct 
		{
			bool updated;
			bool visible;

			std::list<std::string>	lines;
			std::string				currentLine;
		} TConsoleData, *PConsoleData;

		typedef std::map<long, sptr<DOMAIN::CGraphObject> > TObjectList, *PObjectList;
	protected:

		CGraphObject *rootObject;
		CObjectmanager *objMan;

		CGameObject *selectedObject;
		CGameObject *selectedPlayerObject;

	public:

		inline CGameObject *GetSelectedObject() { return selectedObject; };
		inline void SetSelectedObject(CGameObject *_obj) { CGameObject *dummy = selectedObject; selectedObject=_obj; OnSelectedObjectChange(dummy,_obj);  };

		inline CGameObject *GetSelectedPlayerObject() { return selectedPlayerObject; };
		inline void SetSelectedPlayerObject(CGameObject *_obj) { CGameObject *dummy = selectedPlayerObject; selectedPlayerObject=_obj; OnSelectedPlayerObjectChange(dummy,_obj); };

		virtual void OnSelectedObjectChange(CGameObject *_old, CGameObject *_new);
		virtual void OnSelectedPlayerObjectChange(CGameObject *_old, CGameObject *_new);



        
		// interface nach CNetwork
		virtual void pushIncomingNetwork(GENERAL::CBuffer *o,NETWORK::PClientInfoList _from) = 0;
		virtual void pushNetworkEvent(sptr<GENERAL::CSerializeableObject> o) = 0;

		// interfaces zur Applikation
		virtual void execute() = 0;
		virtual bool ShouldTerminate() = 0;
		virtual void Terminate() = 0;

		// interfaces zum Frontend
		virtual CDomain::PObjectList GetVisibleObjects() = 0;
		inline CGraphObject *GetRootObject() { return rootObject; };

		inline CDomain() : objMan(0), rootObject(0) {};
		virtual void RequestObjects(int num, long objectIDs[], sptr<CDomainCommand> &delaydCmd); 
		virtual void Init();
		virtual ~CDomain();

		inline CObjectmanager *GetObjMan() { return objMan; };
	};

}

#endif