#ifndef DOMAIN_LEVEL_OBJECT_H
#define DOMAIN_LEVEL_OBJECT_H

#include "objects.h"

namespace DOMAIN
{

	typedef enum {	MOVEMENTMODE_FLOATING  =0,
					MOVEMENTMODE_GRAVITY   =1 } EMovementModes; 

	typedef enum {	COLLISIONMODE_NONE = 0,
					COLLISIONMODE_SPHERE,
					COLLISIONMODE_BOX,
					COLLISIONMODE_POINT } ECollisionModes; 

	typedef enum {	COLLISIONACTION_STOP = 0,
					COLLISIONACTION_SLIDE = 1 } ECollisionAction;

	class CLevelObject;
	class CPhysicalLevelObject;

	class CLevel : public CWorldObject
	{
	public:
		DECLARE_SERIALIZEABLE(CLevel);
		inline CLevel(CDomain *_domain) : CWorldObject(_domain) {};
		inline CLevel(CDomain *_domain, int _odid, long _id) : CWorldObject(_domain,_odid,_id) {};
		virtual ~CLevel();

		virtual GENERAL::CVector3f Trace(CPhysicalLevelObject *_obj,GENERAL::CVector3f _start, GENERAL::CVector3f _end);
		virtual void LevelObjectMoved(CLevelObject *_obj);
		virtual void FitTo(GENERAL::CVector3f _direction);
	};

	class CLevelObject : public CWorldObject
	{
	protected:

		long levelInfo;
		CLevel *lvl;
		long lvlId;

	public:
		DECLARE_SERIALIZEABLE(CLevelObject);

		inline CLevelObject(CDomain *_domain) : CWorldObject(_domain) { SetClassKey(GET_KEY(CLevelObject)); };
		inline CLevelObject(CDomain *_domain, int _odid,long _id, CLevel* _lvl) : CWorldObject(_domain,_odid,_id) , lvl(_lvl) { SetClassKey(GET_KEY(CLevelObject)); lvlId = _lvl->GetID(); };
		inline CLevelObject(CDomain *_domain, int _odid,long _id, int _lvlId) : CWorldObject(_domain,_odid,_id) , lvlId(_lvlId) { SetClassKey(GET_KEY(CLevelObject)); lvl = 0; };
		virtual ~CLevelObject();

		
		void SetLvl(CLevel *_lvl);
		inline CLevel *GetLvl() { return lvl; };

		inline void SetLvlId(long _lvl) { lvlId = _lvl; };
		inline long GetLvlId() { return lvlId; };
		inline long &GetLevelInfo() {return levelInfo;}


	//	inline CLevel *GetLvl() { return lvl; };

		virtual long GetSerializedSize();
		virtual long Serialize(GENERAL::CBuffer &buffer);
		virtual long DeSerialize(GENERAL::CBuffer &buffer);

//		virtual void Assign(CGameObject *o);

	};

	class CPhysicalLevelObject : public CLevelObject
	{
	public:
		struct TPDU : public CWorldObject::TPDU {
			GENERAL::CVector3f velocity;
		};
	private:
		bool isInterpolating;
		GENERAL::CQuadricBezierPatchVec3f pPatch;

	protected:
        
		GENERAL::CVector3f velocity[2];	// Geschwindigkeit des Objekts
		GENERAL::CVector3f anglevelocity;
		

		int movementMode;
		int collisionMode;

		float mass;						// Masse des Objekts
		float boundingRadius;			// der Radius der Bounding Sphere 
		GENERAL::TBox boundingBox;

		float stepHeight;				// unused here, wichtig fr collision mit bsp

	public:
		DECLARE_SERIALIZEABLE(CPhysicalLevelObject);

		inline CPhysicalLevelObject(CDomain *_domain) 
				: CLevelObject(_domain),
				  isInterpolating(false),
				  boundingBox()
			{ 
				SetClassKey(GET_KEY(CPhysicalLevelObject)); 
			};
		inline CPhysicalLevelObject(CDomain *_domain, int _odid, long _id,
									CLevel *_lvl,
									GENERAL::CVector3f _pos,
									GENERAL::CVector3f _rot,
									GENERAL::CVector3f _scale,
									float _mass,
									float _radius,
									GENERAL::TBox _boundingbox,
									GENERAL::CVector3f _velocity) 
									:
									CLevelObject(_domain,_odid,_id, _lvl),
									mass(mass),
									boundingRadius(_radius),
									boundingBox(_boundingbox),
									isInterpolating(false)
								{
									//SetLvl(_lvl);
									SetClassKey(GET_KEY(CPhysicalLevelObject));

									pos[ACT] = _pos;
									rotation[ACT] = _rot;
									velocity[ACT] = _velocity;
									scale = _scale;

									stepHeight = 15.0f;

									movementMode = MOVEMENTMODE_FLOATING;
									
									//debug
//									boundingRadius = 15.0f;


									collisionMode = COLLISIONMODE_SPHERE;

								};

		virtual ~CPhysicalLevelObject();

		inline float GetMass() { return mass; };
		inline float GetBoundingRadius() { return boundingRadius; };
		inline int GetMovementMode() { return movementMode; };
		inline int GetCollisionMode() { return collisionMode; };
		inline float GetStepHeight() { return stepHeight; };
		inline GENERAL::TBox &GetBoundingBox() { return boundingBox; };


		inline GENERAL::CVector3f GetVelocity() { return velocity[ACT]; };
	
		inline void SetMass(float _mass) { mass = _mass; };
		inline void SetBoundingRadius(float _radius) { boundingRadius = _radius; };
		inline void SetVelocity(GENERAL::CVector3f _vel) { velocity[ACT] = _vel; };
		inline void SetMovementMode(int _mode) { movementMode = _mode; };
		inline void SetCollisionMode(int _mode) { collisionMode = _mode; };

		
		// from CSerializeAble:
		virtual long GetSerializedSize();
		virtual long Serialize(GENERAL::CBuffer &buffer);
		virtual long DeSerialize(GENERAL::CBuffer &buffer);
		virtual void CopyPDUto(char *data);
		virtual void CopyPDUfrom(char *data);
		virtual int GetPDUSize();
		virtual void SetPDU();
		virtual float GetPDUTreshHold();
		virtual ECollisionAction Collided(CVector3f pos, CVector3f normal);

		virtual void Print(FILE *f,bool newline=true);
		//

//		virtual void Assign(CGameObject *o);
		virtual void TimeStep(GENERAL::TGameTick dt); 
		
		virtual void MoveTo(GENERAL::CVector3f &_pos);
		virtual void Forward(float amount);
		virtual void LookX(float angle);
		virtual void LookY(float angle);
		virtual void LookZ(float angle);

		//PY_GETSET_ACCESSOR(GetVelocity);
		//PY_GETSET_ACCESSOR(SetVelocity);

	};
}

#endif