#include "objects.h"

#include "../../general/base/morftools.h"
#include "../../application/base/application.h"
#include "domainlight.h"
#include "../../general/base/trace.h"
#include "../../general/base/config.h"
#include "../../frontend/base/renderable.h"
#include "../implementation/domaincommand.h"
#include "../base/objectman.h"
#include "chatchannel.h"

#include "../../game/implementation/gamefsm.h"


namespace DOMAIN
{
	//class CChatChannel;

	IMPLEMENT_SERIALIZEABLE_DOMAIN(CDomainObject);
	IMPLEMENT_SERIALIZEABLE_DOMAIN(CGameObject);
	IMPLEMENT_SERIALIZEABLE_DOMAIN(CGraphObject);
	IMPLEMENT_SERIALIZEABLE_DOMAIN(CWorldObject);

	CGameObject *CGameObject::selectedGameObject;

	CDomainObject::~CDomainObject()
	{}


	////////////////////////////////////////////////////////////////////////
	// CGameObject
	CGameObject::tActionList CGameObject::tActionList::operator & (CGameObject::tActionList &al)
	{
		CGameObject::tActionList res;

		int i = al.size();

		for (iterator it=begin(); it!=end(); it++)
		{
			if (std::find(al.begin(),al.end(),*it)!=al.end())
				res.push_back(*it);
		}

		return res;
	}

	CGameObject::tActionList CGameObject::tActionList::operator | (CGameObject::tActionList &al)
	{
		CGameObject::tActionList res(*this);

		for (iterator it=al.begin(); it!=al.end(); it++)
		{
			if (std::find(res.begin(),res.end(),*it)==res.end())
				res.push_back(*it);
		}

		return res;
	}


	CGameObject::CGameObject(CDomain *_domain) : CDomainObject(_domain), id(0), doDeathReckoning(true), canDelete(true)
	{
		SetClassKey(GET_KEY(CGameObject));
		loaded=true;
		odid=0;
		inContainerType = CONFIG::g_config.ObjectStatus["INWORLD"];

		timestamp[PDU]=0;
		timestamp[ACT]=APPLICATION::CApplication::GetApplication()->GetTicker().getTick();
		fsmState[ACT]=fsmState[PDU]=0;


	}

	CGameObject::CGameObject(CDomain *_domain, int _odid,long _id) : CDomainObject(_domain), id(_id), doDeathReckoning(true), canDelete(true)
	{
		SetClassKey(GET_KEY(CGameObject));
		loaded=true;
		timestamp[PDU]=0;
		timestamp[ACT]=APPLICATION::CApplication::GetApplication()->GetTicker().getTick();
		fsmState[ACT]=fsmState[PDU]=0;
		inContainerType = CONFIG::g_config.ObjectStatus["INWORLD"];

		odid=_odid; //(_od)?domain->GetObjMan()->GetDescriptorID(_od):0;
		sptr<GAME::CObjectDescriptor> temp = domain->GetObjMan()->GetDescriptor(odid);
//		ReadObjectDescriptor(domain->GetObjMan()->GetDescriptor(odid));

		
	}

	CGameObject::~CGameObject()
	{
	}

	void CGameObject::Done()
	{
	}

	long CGameObject::GetSerializedSize()
	{
		return CSerializeableObject::GetSerializedSize()+sizeof(id)+sizeof(who)+sizeof(GENERAL::TGameTick)+sizeof(odid);
	}

	long CGameObject::Serialize(GENERAL::CBuffer &buffer)
	{
		CSerializeableObject::Serialize(buffer);
		buffer.Write(&id,sizeof(id));
		buffer.Write(&timestamp[ACT],sizeof(timestamp[ACT]));
		buffer.Write(&who,sizeof(who));
		buffer.Write(&odid,sizeof(odid));
		if (odid!=0)
			int blork=1;
		return buffer.GetPos();
	}

	long CGameObject::DeSerialize(GENERAL::CBuffer &buffer)
	{
		CSerializeableObject::DeSerialize(buffer);
		id=*(long*)buffer.Read(sizeof(id));
		timestamp[ACT]=*(GENERAL::TGameTick*)buffer.Read(sizeof(timestamp[ACT]));
		who=*(long*)buffer.Read(sizeof(who));
		odid=*(long*)buffer.Read(sizeof(odid));
		if (odid!=0)
			int blork=1;
//		ReadObjectDescriptor(domain->GetObjMan()->GetDescriptor(odid));
		return buffer.GetPos();
	}

	void CGameObject::GetSerializeObjectsBefore(std::vector<sptr<CGameObject> > &_objs) 
	{
	}

	void CGameObject::Print(FILE *f,bool newline)
	{
		GENERAL::CSerializeableObject::Print(f,false);
		fprintf(f,"ID: %d",id);
		if (newline) fprintf(f,"\n");
	}

	bool CGameObject::Load()
	{
		ReadObjectDescriptor(domain->GetObjMan()->GetDescriptor(odid));
		loaded=true;
		return true;
	}

	void CGameObject::TimeStep(GENERAL::TGameTick dt)
	{
//		if (IsControled()) 
//		timestamp[ACT]+=dt;
		timestamp[ACT]=APPLICATION::CApplication::GetApplication()->GetTicker().getTick();
	}

/*	void CGameObject::Assign(CGameObject *o)
	{
		id = o->GetID();
	}*/

	void CGameObject::CopyPDUto(char *data)
	{
		TPDU *pdu = (TPDU*)data;
		pdu->timestamp = timestamp[PDU];
		pdu->fsmState = fsmState[PDU];
	}

	void CGameObject::CopyPDUfrom(char *data)
	{
		TPDU *pdu = (TPDU*)data;
		timestamp[PDU] = timestamp[ACT];
		fsmState[PDU] = pdu->fsmState;
//		timestamp[ACT] = 
//		timestamp[PDU] = pdu->timestamp;
	}

	void CGameObject::SetPDU()
	{
		timestamp[PDU]=timestamp[ACT];
		fsmState[PDU]=fsmState[ACT];
	}

	int CGameObject::GetPDUSize()
	{
		return sizeof(TPDU);
	}

	float CGameObject::GetPDUTreshHold()
	{
		return fsmState[PDU]!=fsmState[ACT]?1000.0f:0.0f;
	}

	void CGameObject::GetActionList(tActionList &_actions, int witch)
	{
		_actions = actions.get((witch==-1)?inContainerType:witch);
	}


	int CGameObject::ReadObjectDescriptor(sptr<GAME::CObjectDescriptor> _od)
	{
		GAME::CObjectDescriptor::TPropertyMap *pm;
		pm = _od->GetPropertyMap();

		GAME::CObjectDescriptor::TStatePropertyMap::iterator it;
		GAME::CObjectDescriptor::TPropertyMap::iterator pmit;
		GAME::CObjectDescriptor::TStatePropertyMap spm;

		char *act, *token;

		std::map<std::string, long>::iterator ci;

		for (ci = CONFIG::g_config.ObjectStatus.begin(); ci != CONFIG::g_config.ObjectStatus.end(); ci++)
		{
			long num = ci->second;
			pmit = pm->find(ci->first);
			if (pmit!=pm->end())
			{
				// eintrag im descriptor gefunden
				spm = pmit->second;
				it = spm.find("actions");
				if (it!=spm.end())
				{
					act = (char*)it->second.sVal.data();
					token = strtok(act, ",");
					while (token!=0)
					{
						long nr = CONFIG::g_config.ObjectActions[token];
						actions[ci->second].push_back(nr);
						token = strtok(NULL,",");
					}
				}

				spm = pmit->second;
				it = spm.find("popupactions");
				if (it!=spm.end())
				{
					act = (char*)it->second.sVal.data();
					token = strtok(act, ",");
					while (token!=0)
					{
						long nr = CONFIG::g_config.ObjectActions[token];
						popupactions.push_back(nr);
						token = strtok(NULL,",");
					}
				}
                
			}
		}
		return 0;
	}


	void CGameObject::InitiateAction(TDomainObjectAction *act, unsigned short size)
	{
		sptr<CDomainCommand> dc = sptr<CDomainCommand>(new CDomainCommand(domain));
		dc->SetCommand(DOMAINCMD_OBJECTACTION_REQUEST);
		dc->SetParamLen(size);
		dc->SetParam(act);
		dc->SetSendToClientList(NULL);
		APPLICATION::CApplication::GetApplication()->GetNetwork()->PushObject(dc,sendToClients);
	}

	void CGameObject::HandleDomainObjectActionRequest(sptr<CDomainCommand> &cmd)	
	{}
	
	void CGameObject::HandleDomainObjectActionReply(sptr<CDomainCommand> &cmd)	
	{}

	////////////////////////////////////////////////////////////////////////
	// CGraphObject

	CGraphObject::CGraphObject(CDomain *_domain) : CGameObject(_domain), feData()
	{
		SetClassKey(GET_KEY(CGraphObject));
		parent = 0;
		parentId = 0;
		ownedChatChannel = 0;
		fsm= new GAME::CGameFSM(this);
	}

	CGraphObject::CGraphObject(CDomain *_domain, int _odid,long _id) : CGameObject(_domain,_odid,_id), feData()
	{
		SetClassKey(GET_KEY(CGraphObject));
		parent = 0;
		parentId = 0;
		ownedChatChannel = 0;
		fsm= new GAME::CGameFSM(this);

	}

	void CGraphObject::Command(char *_cmd)
	{
		fsm->Command(_cmd);
	}

	CGraphObject::~CGraphObject()
	{
		childs.clear();
		lights.clear();
	}

	long CGraphObject::GetSerializedSize()
	{
		return 
			CGameObject::GetSerializedSize()+
			sizeof(long)+
			sizeof(long)+ // the parents ID, 0 if negative
			sizeof(long)+ // the id of the owned chatchannel
			sizeof(unsigned short int)+
			sizeof(unsigned short);
        
	}

	long CGraphObject::Serialize(GENERAL::CBuffer &buffer)
	{
		CGameObject::Serialize(buffer);
		buffer.Write(&feRepresentation,sizeof(long));
		buffer.Write(&feStatus,sizeof(unsigned short int));
		
		long parentid = 0;
		if (parent) parentid = parent->GetID();
		if (parentid < 0) parentid = 0;
		buffer.Write(&parentid, sizeof(long));
		buffer.Write(&fsmState[ACT], sizeof(unsigned short));
		buffer.Write(&ownedChatChannel, sizeof(ownedChatChannel));

		return buffer.GetPos();
	}

	long CGraphObject::DeSerialize(GENERAL::CBuffer &buffer)
	{
		CGameObject::DeSerialize(buffer);

		feRepresentation=*(long*)buffer.Read(sizeof(feRepresentation));
		feStatus=*(unsigned short int*)buffer.Read(sizeof(feStatus));
		parentId =*(long*)buffer.Read(sizeof(parentId));
		fsmState[ACT]=*(unsigned short*)buffer.Read(sizeof(fsmState[ACT]));
		ownedChatChannel=*(long*)buffer.Read(sizeof(ownedChatChannel));
		return buffer.GetPos();
	}

/*	void CGraphObject::Assign(CGameObject *o)
	{
		CGameObject::Assign(o);
		feRepresentation = ((CGraphObject*)o)->GetfeRepresentation();
		feStatus = ((CGraphObject*)o)->GetfeStatus();
	}*/

	void CGraphObject::GetSerializeObjectsBefore(std::vector<sptr<CGameObject> > &_objs) 
	{
		if (ownedChatChannel!=0) _objs.push_back(domain->GetObjMan()->GetPtr(ownedChatChannel));

	}
	void CGraphObject::Print(FILE *f,bool newline)
	{
		CGameObject::Print(f,false);
		fprintf(f,", feRep: %i, feStatus: %i",feRepresentation,feStatus);
		if (newline) fprintf(f,"\n");
	}

	long CGraphObject::CreateOwnedChatChannel()
	{

		long ccid = domain->GetObjMan()->CreateGlobalId();
		sptr<CChatChannel> cc = new CChatChannel(domain, 0, ccid, GetID());
		cc->SetControl();
//		cc->SetSendToClientList(&allUsers);
		domain->GetObjMan()->Add(cc);
		ownedChatChannel = ccid;
		AddChild((sptr<CGraphObject>) cc);

		return ccid;

	}

	int CGraphObject::ActionChatReply(CGraphObject *_caller,long _whoId,char *_msg)
	{
		return 0;
	}

	void CGraphObject::UpdateRenderables(int _flags)
	{
		tRenderableList::iterator it;
		for (it = updateRenderables.begin(); it!= updateRenderables.end(); it++)
		{
			(*it)->Update(_flags);
		}
	}

	void CGraphObject::SetParent(CGraphObject *_parent)
	{
		parent = _parent;
	}

	void CGraphObject::AddChild(sptr<CGraphObject> _child)
	{
		if (ISOFTYPE(_child,DOMAIN::CLight))
		{
			lights.push_back(_child);
			if (parent) parent->RegisterLight(_child);
		}
		else 
		{
			childs.push_back(_child);
		}
		_child->SetParent(this);

		UpdateRenderables(FRONTEND::UPDATEFLAGS_ALL);
	}

	void CGraphObject::RemoveChild(sptr<CGraphObject> _child)
	{
		if (ISOFTYPE(_child,DOMAIN::CLight))
		{
			tLightsType::iterator it;

			it = lights.begin();

			for (;it!=lights.end();it++)
			{
				if ( _child == *it)
					break;
			}

			if (it!=lights.end())
			{
				lights.erase(it);
				_child->SetParent(0);
				if (parent) parent->UnRegisterLight(_child);
			}
		}
		else
		{
			tChildsType::iterator it;
			it = std::find(childs.begin(), childs.end(), _child);

			if (it!=childs.end())
			{
				childs.erase(it);
				_child->SetParent(0);
			}
		}
		UpdateRenderables(FRONTEND::UPDATEFLAGS_ALL);
	}
	void CGraphObject::HandoverChild(sptr<CGraphObject> _target, sptr<CGraphObject> _child)
	{
		tChildsType::iterator it;
		it = std::find(childs.begin(), childs.end(), _child);

		if (it!=childs.end()) 
		{
			childs.erase(it);
			_target->AddChild(_child);
		}
	}

	void CGraphObject::RegisterCamera(sptr<CGraphObject> _camera) 
	{ 
		if (parent) parent->RegisterCamera(_camera);
	}

	void CGraphObject::RegisterLight(sptr<CLight> _light)
	{
		if (parent) parent->RegisterLight(_light);
	}
    void CGraphObject::UnRegisterCamera(sptr<CGraphObject> _camera)
	{
		if (parent) parent->UnRegisterCamera(_camera);
	}
	void CGraphObject::UnRegisterLight(sptr<CLight> _light)
	{
		if (parent) parent->UnRegisterLight(_light);
	}

	int CGraphObject::ReadObjectDescriptor(sptr<GAME::CObjectDescriptor> _od)
	{
		CGameObject::ReadObjectDescriptor(_od);

		GAME::CObjectDescriptor::TStatePropertyMap *pm = _od->GetPropertyMap("COMMON");
		if (!pm) return -1;

		GAME::CObjectDescriptor::TStatePropertyMap::iterator it;

		it = pm->find("icon");
		if (it!=pm->end()) icon = it->second.sVal;

		it = pm->find("caption");
		if (it!=pm->end()) caption = it->second.sVal;


		return 0;
	}

	int CGraphObject::ExecuteAction(int _actionId,  CGraphObject *_caller, long lParam, void *pParam) { return 0; }

	////////////////////////////////////////////////////////////////////////
	// CWorldObject

	long CWorldObject::GetSerializedSize()
	{
		return 
			CGraphObject::GetSerializedSize()+
//			sizeof(long)+
			GENERAL::GetSerializedSizeVector3f()*3;
	}

	long CWorldObject::Serialize(GENERAL::CBuffer &buffer)
	{
		CGraphObject::Serialize(buffer);
		GENERAL::SerializeVector3f(pos[ACT],buffer);
		GENERAL::SerializeVector3f(scale,buffer);
		GENERAL::SerializeVector3f(rotation[ACT],buffer);
		return buffer.GetPos();
	}

	long CWorldObject::DeSerialize(GENERAL::CBuffer &buffer)
	{
		CGraphObject::DeSerialize(buffer);
		GENERAL::DeSerializeVector3f(pos[ACT],buffer);
		GENERAL::DeSerializeVector3f(scale,buffer);
		GENERAL::DeSerializeVector3f(rotation[ACT],buffer);
		return buffer.GetPos();
	}

/*	void CWorldObject::Assign(CGameObject *o)
	{
		CGraphObject::Assign(o);
		pos = ((CWorldObject*)o)->pos;
		scale = ((CWorldObject*)o)->scale;
		rotation = ((CWorldObject*)o)->rotation;
	}*/

	void CWorldObject::CopyPDUto(char *data)
	{
		CGraphObject::CopyPDUto(data);
		TPDU *pdu = (TPDU*)data;
		pdu->pos = pos[PDU];
		pdu->rotation = rotation[PDU];
	}

	void CWorldObject::CopyPDUfrom(char *data)
	{
		CGraphObject::CopyPDUfrom(data);
		TPDU *pdu = (TPDU*)data;
		pos[PDU] = pdu->pos;
		rotation[PDU] = pdu->rotation;
	}

	void CWorldObject::SetPDU()
	{
		CGraphObject::SetPDU();
		pos[PDU]=pos[ACT];
		rotation[PDU]=rotation[ACT];
	}

	int CWorldObject::GetPDUSize()
	{
		return sizeof(TPDU);
	}

	float CWorldObject::GetPDUTreshHold()
	{
		return (pos[PDU]-pos[ACT]).abs2() / 25.0f + (rotation[PDU]-rotation[ACT]).abs2()*10.0f+CGraphObject::GetPDUTreshHold();
	}

	void CWorldObject::Print(FILE *f,bool newline)
	{
		CGraphObject::Print(f,false);
		fprintf(f,", pos: (%f,%f,%f), scale: (%f,%f,%f), rotation: (%f,%f,%f)",
			pos[ACT].x,pos[ACT].y,pos[ACT].z,
			scale.x,scale.y,scale.z,
			rotation[ACT].x,rotation[ACT].y,rotation[ACT].z);
		if (newline) fprintf(f,"\n");
	}

/*	PyObject *CWorldObject::PyGetPos(PyObject *self, PyObject *args)
	{ 
		PyObject *thisPtr = PyTuple_GetItem( args, 0);
		CWorldObject *obj = (CWorldObject*)PyCObject_AsVoidPtr(thisPtr); 
//		PyObject *thisPtr1 = PyTuple_GetItem( args, 0);
		return Vector3fToPython(obj->GetPos());
	} */

	PY_GET_ACCESSOR_IMPL(CGraphObject,GetParent,GameObjectToPython);
	PY_GET_ACCESSOR_IMPL(CWorldObject,GetPos,Vector3fToPython);
	PY_GET_ACCESSOR_IMPL(CWorldObject,GetRotation,Vector3fToPython);
//	PY_SET_ACCESSOR_IMPL(CWorldObject,SetPos,PythonVecToVector3f);


/*	PyObject *CWorldObject::PyGetPos(PyObject *self, PyObject *args)
	{
		PyObject *thisObj = PyTuple_GetItem( args, 0);
		CWorldObject *wo = (CWorldObject*)PyCObject_AsVoidPtr(thisObj);
		PyObject *result = Vector3fToPython(wo->GetPos());
		return result;
	}*/


}
