#ifndef GAMEFSM_H
#define GAMEFSM_H

#include <map>
#include <string>
#include <vector>

#include "../../general/base/parser.h"

namespace DOMAIN
{

	class CGraphObject;
}


namespace GAME
{

	#define STATE_ENABLED  0
	#define STATE_DISABLED 1




	class CGameFSM
	{
	public:

		struct TScriptEvent
		{
			std::string module;
			std::string func;
		};

		struct TState
		{
			char *stateName;
			int	enabled;

			std::string disabledBy;

			std::map<std::string,std::string> params;
			std::map<int, TState* > nextStates;

			inline void Disable(std::string _msg) { enabled = STATE_DISABLED; disabledBy = _msg; };
			inline void Enable() { enabled = STATE_ENABLED; disabledBy = ""; };
			inline bool IsEnabled() { if (enabled == STATE_ENABLED) return true; else return false; }; 

			typedef std::map<std::string,TScriptEvent> TEventMap;

			TEventMap EventMap;

			inline bool HasEventScript(std::string _event)
			{
				std::map<std::string,TScriptEvent>::iterator it = EventMap.find(_event);
				if (it == EventMap.end()) return false; else return true;
				return false;
			};


		};

		typedef std::map<std::string,std::string> TConstantMap;
		typedef std::map<std::string,TState* > TStateMap;
		typedef std::map<std::string,int> TCommandMap;

		
		struct TStateSet
		{
			char *stateSetName;
			TStateSet *parent;
			std::vector <TStateSet* > childs;
			TStateMap states;

			TState *currentState;

			inline TState *GetNextState(int _cmd)
			{
				std::map<int, TState* >::iterator it = currentState->nextStates.find(_cmd);
				return it==currentState->nextStates.end()?NULL:it->second;
			}

			
			inline TState *GetState(char *_stateName)
			{
				TStateMap::iterator it = states.find(_stateName);
				return it==states.end()?NULL:it->second;
			}

			inline bool IsEnabled(char *_stateName)
			{
				TState *st =GetState(_stateName);
				if (st==0) return false;
                return st->IsEnabled();

			}

			inline std::vector<std::string> GetStates()
			{
				TStateMap::iterator it;
				std::vector<std::string> res;
				for (it = states.begin(); it != states.end(); it++)
				{
					res.push_back(it->first);
				}
				return res;

			}

			inline char *GetCurrentState() { return currentState->stateName; };


		};

		typedef std::map<std::string,TStateSet* > TStateSetMap;

		

	protected:
		//TState *state;

		TConstantMap constants;
		//TStateMap stateNames;
		TStateSetMap stateSets;
		TCommandMap commandNames;

		DOMAIN::CGraphObject *owner;






		inline int GetCommand(char *_cmdName)
		{
			TCommandMap::iterator it = commandNames.find(_cmdName);
			return it==commandNames.end()?0:it->second;
		}
		int ReadSet(GENERAL::CParser::TNode *fsmNode, TStateSet *set, char *_filename, GENERAL::CParser &parser);

		std::vector<std::string> GetStateSets();


		

		char *name;
	public:
		CGameFSM(DOMAIN::CGraphObject *_owner);
		virtual ~CGameFSM();

				inline TStateSet *GetStateSet(char *_setName)
			{
				TStateSetMap::iterator it = stateSets.find(_setName);
				return it==stateSets.end()?NULL:it->second;
			}

		inline TState *GetState(char *_setName, char *_stateName)
		{
			TStateSetMap::iterator it = stateSets.find(_setName);
			if (it->second!=NULL)
			{
				return it->second->GetState(_stateName);
			}
		}

		inline bool IsStateEnabled(char *_setName, char *_stateName)
		{
			TStateSetMap::iterator it = stateSets.find(_setName);
			if (it->second!=NULL)
			{
				TState *st = it->second->GetState(_stateName);
				if (st==0) return false;
				return st->IsEnabled();
			}
			return false;
		}

		inline char *GetCurrentState(char *_setName)
		{
			TStateSetMap::iterator it = stateSets.find(_setName);
			if (it->second!=NULL)
			{
				return it->second->GetCurrentState();
			}
			return false;
		}

		inline std::vector<std::string> GetCurrentStates()
		{
			std::vector<std::string> res;

			TStateSetMap::iterator it;
			for (it = stateSets.begin(); it != stateSets.end(); it++)
			{
				std::string line;
				line = it->first+" ("+it->second->GetCurrentState()+")"; 
				res.push_back(line);


			}


			return res;
		}


		virtual bool Init(char *_filename);

		virtual void Command(char *_cmd);



	};
}

#endif