#ifndef GAME_OBJECTDESCRIPTOR_H
#define GAME_OBJECTDESCRIPTOR_H

#include <string>
#include <map>

#include "../general/base/smartpointer.h"

namespace GAME
{
	struct TObjectProperty
	{
		int iVal;
		float fVal; 
		std::string sVal;

		inline TObjectProperty() : iVal(0), fVal(0.0f), sVal() {};

		inline TObjectProperty(const TObjectProperty& _op) : iVal(_op.iVal), fVal(_op.fVal), sVal(_op.sVal) {};

		inline TObjectProperty(char *_st) : sVal(_st)
		{
			char *eptr;

            iVal = strtol(_st, &eptr, 0);
			if ((eptr==_st) || (*eptr!='\0')) iVal = 0;

			fVal = (float)strtod(_st,&eptr);
			if ((eptr==_st) || (*eptr!='\0')) fVal = 0.0f;
		}
	};

	class CObjectDescriptor
	{
	public:

		typedef std::map < std::string, TObjectProperty > TStatePropertyMap;
		typedef std::map < std::string, TStatePropertyMap > TPropertyMap;


	protected:

		TPropertyMap properties;

		virtual bool LoadFromFile(char *_name, char *_filename);
        
	public:

		inline CObjectDescriptor() {};
		virtual ~CObjectDescriptor();

		virtual bool Init(char *_name, char *_filename);

		inline TStatePropertyMap *GetPropertyMap(char *_section) 
		{
			TPropertyMap::iterator it = properties.find(_section);
			if (it==properties.end()) return 0;
			return &(it)->second;
		}

		inline TPropertyMap *GetPropertyMap() 
		{
			return &properties;
		}

	};
/*
	class Objects
	{
	protected:
		std::map<std::string,sptr<CObjectDescriptor> > omap;

	public:
		sptr<CObjectDescriptor>	GetDescriptor(char *_name);
		sptr<CObjectDescriptor> GetDescriptor(int _id);

	};
	*/
}
#endif