// MorfMath.cpp: Implementierung der Klasse CPlane.
//
//////////////////////////////////////////////////////////////////////

#include "morfmath.h"
#include "morfwindows.h"

#include <math.h>

namespace GENERAL
{

	double Sin(double _alpha)
	{
		return sin(_alpha);
	}

	double Cos(double _alpha)
	{
		return cos(_alpha);
	}

	double Sqrt(double _alpha)
	{
		return sqrt(_alpha);
	}




	TMult_V4xM44f   Mult_V4xM44f   = Mult_V4xM44f_SSE;
	TTranspose_M44f Transpose_M44f = Transpose_M44f_SSE;

	void CQuaternion::CreateMatrix(float *pMatrix)
	{
		// Make sure the matrix has allocated memory to store the rotation data
		if(!pMatrix) return;
		
		// Fill in the rows of the 4x4 matrix, according to the quaternion to matrix equations

		// First row
		pMatrix[ 0] = 1.0f - 2.0f * ( y * y + z * z );  
		pMatrix[ 1] = 2.0f * ( x * y - w * z );  
		pMatrix[ 2] = 2.0f * ( x * z + w * y );  
		pMatrix[ 3] = 0.0f;  

		// Second row
		pMatrix[ 4] = 2.0f * ( x * y + w * z );  
		pMatrix[ 5] = 1.0f - 2.0f * ( x * x + z * z );  
		pMatrix[ 6] = 2.0f * ( y * z - w * x );  
		pMatrix[ 7] = 0.0f;  

		// Third row
		pMatrix[ 8] = 2.0f * ( x * z - w * y );  
		pMatrix[ 9] = 2.0f * ( y * z + w * x );  
		pMatrix[10] = 1.0f - 2.0f * ( x * x + y * y );  
		pMatrix[11] = 0.0f;  

		// Fourth row
		pMatrix[12] = 0;  
		pMatrix[13] = 0;  
		pMatrix[14] = 0;  
		pMatrix[15] = 1.0f;

		// Now pMatrix[] is a 4x4 homogeneous matrix that can be applied to an OpenGL Matrix
	}


	///////////////////////////////// CREATE FROM MATRIX \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
	/////
	/////	This creates a quaternion from a 3x3 or a 4x4 matrix, depending on rowColumnCount
	/////
	///////////////////////////////// CREATE FROM MATRIX \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*

	void CQuaternion::CreateFromMatrix(float *pTheMatrix, int rowColumnCount)
	{
		// Make sure the matrix has valid memory and it's not expected that we allocate it.
		// Also, we do a check to make sure the matrix is a 3x3 or a 4x4 (must be 3 or 4).
		if(!pTheMatrix || ((rowColumnCount != 3) && (rowColumnCount != 4))) return;

		// This function is used to take in a 3x3 or 4x4 matrix and convert the matrix
		// to a quaternion.  If rowColumnCount is a 3, then we need to convert the 3x3
		// matrix passed in to a 4x4 matrix, otherwise we just leave the matrix how it is.
		// Since we want to apply a matrix to an OpenGL matrix, we need it to be 4x4.

		// Point the matrix pointer to the matrix passed in, assuming it's a 4x4 matrix
		float *pMatrix = pTheMatrix;

		// Create a 4x4 matrix to convert a 3x3 matrix to a 4x4 matrix (If rowColumnCount == 3)
		float m4x4[16] = {0};

		// If the matrix is a 3x3 matrix (which it is for Quake3), then convert it to a 4x4
		if(rowColumnCount == 3)
		{
			// Set the 9 top left indices of the 4x4 matrix to the 9 indices in the 3x3 matrix.
			// It would be a good idea to actually draw this out so you can visualize it.
			m4x4[0]  = pTheMatrix[0];	m4x4[1]  = pTheMatrix[1];	m4x4[2]  = pTheMatrix[2];
			m4x4[4]  = pTheMatrix[3];	m4x4[5]  = pTheMatrix[4];	m4x4[6]  = pTheMatrix[5];
			m4x4[8]  = pTheMatrix[6];	m4x4[9]  = pTheMatrix[7];	m4x4[10] = pTheMatrix[8];

			// Since the bottom and far right indices are zero, set the bottom right corner to 1.
			// This is so that it follows the standard diagonal line of 1's in the identity matrix.
			m4x4[15] = 1;

			// Set the matrix pointer to the first index in the newly converted matrix
			pMatrix = &m4x4[0];
		}

		// The next step, once we made sure we are dealing with a 4x4 matrix, is to check the
		// diagonal of the matrix.  This means that we add up all of the indices that comprise
		// the standard 1's in the identity matrix.  If you draw out the identity matrix of a
		// 4x4 matrix, you will see that they 1's form a diagonal line.  Notice we just assume
		// that the last index (15) is 1 because it is not effected in the 3x3 rotation matrix.

		// Find the diagonal of the matrix by adding up it's diagonal indices.
		// This is also known as the "trace", but I will call the variable diagonal.
		float diagonal = pMatrix[0] + pMatrix[5] + pMatrix[10] + 1;
		float scale = 0.0f;

		// Below we check if the diagonal is greater than zero.  To avoid accidents with
		// floating point numbers, we substitute 0 with 0.00000001.  If the diagonal is
		// great than zero, we can perform an "instant" calculation, otherwise we will need
		// to identify which diagonal element has the greatest value.  Note, that it appears
		// that %99 of the time, the diagonal IS greater than 0 so the rest is rarely used.

		// If the diagonal is greater than zero
		if(diagonal > 0.00000001)
		{
			scale = (float)(sqrt(diagonal )*2);

			x = ( pMatrix[9] - pMatrix[6] ) / scale;
			y = ( pMatrix[2] - pMatrix[8] ) / scale;
			z = ( pMatrix[4] - pMatrix[1] ) / scale;
			w = 0.25f * scale;
		}
		else 
		{
			// If the first element of the diagonal is the greatest value
			if ( pMatrix[0] > pMatrix[5] && pMatrix[0] > pMatrix[10] )  
			{	
				// Find the scale according to the first element, and double that value
				scale  = (float)sqrt( 1.0f + pMatrix[0] - pMatrix[5] - pMatrix[10] ) * 2.0f;

				// Calculate the x, y, x and w of the quaternion through the respective equation
				x = 0.25f * scale;
				y = (pMatrix[4] + pMatrix[1] ) / scale;
				z = (pMatrix[2] + pMatrix[8] ) / scale;
				w = (pMatrix[9] - pMatrix[6] ) / scale;	
			} 
			// Else if the second element of the diagonal is the greatest value
			else if ( pMatrix[5] > pMatrix[10] ) 
			{
				// Find the scale according to the second element, and double that value
				scale  = (float)sqrt( 1.0f + pMatrix[5] - pMatrix[0] - pMatrix[10] ) * 2.0f;
				
				// Calculate the x, y, x and w of the quaternion through the respective equation
				x = (pMatrix[4] + pMatrix[1] ) / scale;
				y = 0.25f * scale;
				z = (pMatrix[9] + pMatrix[6] ) / scale;
				w = (pMatrix[2] - pMatrix[8] ) / scale;
			} 
			// Else the third element of the diagonal is the greatest value
			else 
			{	
				// Find the scale according to the third element, and double that value
				scale  = (float)sqrt( 1.0f + pMatrix[10] - pMatrix[0] - pMatrix[5] ) * 2.0f;

				// Calculate the x, y, x and w of the quaternion through the respective equation
				x = (pMatrix[2] + pMatrix[8] ) / scale;
				y = (pMatrix[9] + pMatrix[6] ) / scale;
				z = 0.25f * scale;
				w = (pMatrix[4] - pMatrix[1] ) / scale;
			}
		}
	}


	/////////////////////////////////////// SLERP \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
	/////
	/////	Returns a spherical linear interpolated quaternion between q1 and q2, with respect to t
	/////
	/////////////////////////////////////// SLERP \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*

	CQuaternion CQuaternion::Slerp(CQuaternion &q1, CQuaternion &q2, float t)
	{
		// Create a local quaternion to store the interpolated quaternion
		CQuaternion qInterpolated;

		// This function is the milk and honey of our quaternion code, the rest of
		// the functions are an appendage to what is done here.  Everyone understands
		// the terms, "matrix to quaternion", "quaternion to matrix", "create quaternion matrix",
		// "quaternion multiplication", etc.. but "SLERP" is the stumbling block, even a little 
		// bit after hearing what it stands for, "Spherical Linear Interpolation".  What that
		// means is that we have 2 quaternions (or rotations) and we want to interpolate between 
		// them.  The reason what it's called "spherical" is that quaternions deal with a sphere.  
		// Linear interpolation just deals with 2 points primarily, where when dealing with angles
		// and rotations, we need to use sin() and cos() for interpolation.  If we wanted to use
		// quaternions for camera rotations, which have much more instant and jerky changes in 
		// rotations, we would use Spherical-Cubic Interpolation.  The equation for SLERP is this:
		//
		// q = (((b.a)^-1)^t)a
		//
		// Go here for an a detailed explanation and proofs of this equation:
		//
		// http://www.magic-software.com/Documentation/quat.pdf
		//
		// Now, Let's code it

		// Here we do a check to make sure the 2 quaternions aren't the same, return q1 if they are
		if(q1.x == q2.x && q1.y == q2.y && q1.z == q2.z && q1.w == q2.w) 
			return q1;

		// Following the (b.a) part of the equation, we do a dot product between q1 and q2.
		// We can do a dot product because the same math applied for a 3D vector as a 4D vector.
		float result = (q1.x * q2.x) + (q1.y * q2.y) + (q1.z * q2.z) + (q1.w * q2.w);

		// If the dot product is less than 0, the angle is greater than 90 degrees
		if(result < 0.0f)
		{
			// Negate the second quaternion and the result of the dot product
			q2 = CQuaternion(-q2.x, -q2.y, -q2.z, -q2.w);
			result = -result;
		}

		// Set the first and second scale for the interpolation
		float scale0 = 1 - t, scale1 = t;

		// Next, we want to actually calculate the spherical interpolation.  Since this
		// calculation is quite computationally expensive, we want to only perform it
		// if the angle between the 2 quaternions is large enough to warrant it.  If the
		// angle is fairly small, we can actually just do a simpler linear interpolation
		// of the 2 quaternions, and skip all the complex math.  We create a "delta" value
		// of 0.1 to say that if the cosine of the angle (result of the dot product) between
		// the 2 quaternions is smaller than 0.1, then we do NOT want to perform the full on 
		// interpolation using.  This is because you won't really notice the difference.

		// Check if the angle between the 2 quaternions was big enough to warrant such calculations
		if(1 - result > 0.1f)
		{
			// Get the angle between the 2 quaternions, and then store the sin() of that angle
			float theta = (float)acos(result);
			float sinTheta = (float)sin(theta);

			// Calculate the scale for q1 and q2, according to the angle and it's sine value
			scale0 = (float)sin( ( 1 - t ) * theta) / sinTheta;
			scale1 = (float)sin( ( t * theta) ) / sinTheta;
		}	

		// Calculate the x, y, z and w values for the quaternion by using a special
		// form of linear interpolation for quaternions.
		qInterpolated.x = (scale0 * q1.x) + (scale1 * q2.x);
		qInterpolated.y = (scale0 * q1.y) + (scale1 * q2.y);
		qInterpolated.z = (scale0 * q1.z) + (scale1 * q2.z);
		qInterpolated.w = (scale0 * q1.w) + (scale1 * q2.w);

		// Return the interpolated quaternion
		return qInterpolated;
	}

	//////////// *** NEW *** ////////// *** NEW *** ///////////// *** NEW *** ////////////////////




	//////////////////////////////////////////////////////////////////////
	// Konstruktion/Destruktion
	//////////////////////////////////////////////////////////////////////

	SCALAR Distance2SegSeg(CVector3f B0,CVector3f M0,CVector3f B1,CVector3f M1)
	{
	// refering http://www.magic-software.com/Documentation/DistanceLine3Line3.pdf

		SCALAR a = M0.ScalarMultiply(M0);
		SCALAR b = -M0.ScalarMultiply(M1);
		SCALAR c = M1.ScalarMultiply(M1);
		SCALAR d = M0.ScalarMultiply(B0-B1);
		SCALAR e = -M1.ScalarMultiply(B0-B1);
		SCALAR f = (B0-B1).ScalarMultiply(B0-B1);

		SCALAR det = a*c-b*b;
		SCALAR s = b*e - c*d;
		SCALAR t = b*d - a*e;

		SCALAR tmp;

		if (det > EPSILON)
		{
			// non parallel
			if (s>(SCALAR)0)
			{
				if ( s<=det )
				{
					if ( t>=(SCALAR)0 )
					{
						if (t<=det)
						{
							// regn 0
							tmp = (SCALAR)1.0/det;
							s*=tmp;
							t*=tmp;
						} else {
							// regn 3
							t=(SCALAR)1.0;
							tmp = b+d;
							if ( tmp>(SCALAR)0 ) s=(SCALAR)0.0; 
							else if (-tmp > a) s=(SCALAR)1.0;
							else s = -tmp/a;
						}
					} else {
						// regn 7
						t=(SCALAR)0.0;
						if ( d>(SCALAR)0 ) s=(SCALAR)0.0; 
						else if (-d > a) s=(SCALAR)1.0;
						else s = -d/a;
					}
				} else {
					if ( t>=(SCALAR)0 )
					{
						if (t<=det)
						{
							// regn 1
							s=(SCALAR)1.0;
							tmp = b+e;
							if ( tmp>(SCALAR)0 ) t=(SCALAR)0.0; 
							else if (-tmp > c) t=(SCALAR)1.0;
							else t = -tmp/c;
						} else {
							// regn 2
							tmp = b+d;
							if ( -tmp < a ) 
							{
								t = 1;
								if ( tmp > (SCALAR)0 ) s = (SCALAR)0;
								else s = -tmp/a;
							}
							else 
							{
								s = (SCALAR)1.0;
								tmp=b+e;
								if ( tmp >= (SCALAR)0.0 ) t = (SCALAR)0.0;
								else if ( -tmp >= c ) t = (SCALAR)1.0;
								else t = -e/c;

							}
						}
					} else {
						// regn 8
						//tmp = b+d;
						if ( -d < a ) 
						{
							t = (SCALAR)0.0;
							if ( d > (SCALAR)0.0 ) s = (SCALAR)0.0;
							else s = -d/a;
						}
						else 
						{
							s = (SCALAR)1.0;
							tmp=b+e;
							if ( tmp >= (SCALAR)0.0 ) t = (SCALAR)0.0;
							else if ( -tmp >= c ) t = (SCALAR)1.0;
							else t = -e/c;
						}
					}
				}
			} else {
				if ( t>=(SCALAR)0 )
				{
					if (t<=det)
					{
						// regn 5
						s=(SCALAR)0.0;
						if ( d>(SCALAR)0 ) t=(SCALAR)0.0; 
						else if (-d > c) t=(SCALAR)1.0;
						else t = -d/c;
					} else {
						// regn 4
						tmp = b+d;
						if ( tmp < (SCALAR)0.0 ) 
						{
							t = (SCALAR)1;
							if ( tmp > a ) s = (SCALAR)1.0;
							else s = -tmp/a;
						}
						else 
						{
							s = (SCALAR)0.0;
							if ( e >= (SCALAR)0.0 ) t = (SCALAR)0.0;
							else if ( -e >= c ) t = (SCALAR)1.0;
							else t = -e/c;
						}
					}
				} else {
					// regn 6
					if ( d < (SCALAR)0.0 ) //???
					{
						t = (SCALAR)0.0;
						if ( -d > a ) s = (SCALAR)1.0;
						else s = -d/a;
					}
					else 
					{
						s = (SCALAR)0.0;
						if ( e >= (SCALAR)0.0 ) t = (SCALAR)0.0;
						else if ( -e >= c ) t = (SCALAR)1.0;
						else t = -e/c;
					}
				}
			}
		} else {
			// parallel segments
			if ( b > (SCALAR)0 )
			{
				if ( d >= (SCALAR)0 )
				{
					s=(SCALAR)0; t=(SCALAR)0;
				} else if (-d <= a)
				{
					s=-d/a;
					t=(SCALAR)0;
				} else {
					s = 1;
					if (b>(SCALAR)0)
					{
						tmp = a+d;
						if ( -tmp >= b ) t = 1; else t = -tmp/b;
					} else {
						if (d >= -b) t=1; else t= -d/b;
					}
				}
			} else {
				if ( -d >= a )
				{
					s=1; t=(SCALAR)0;
				} else if ( d<=(SCALAR)0 )
				{
					s=-d/a; t=(SCALAR)0;
				} else {
					s = 1;
					if (b>(SCALAR)0)
					{
						SCALAR tmp = a+d;
						if ( -tmp >= b ) t = 1; else t = -tmp/b;
					} else {
						if (d >= -b) t=1; else t= -d/b;
					}
				}
			}

		}

		return ((B0+M0*t)-(B0+M0*s)).abs2();

	}

	void _fastcall Mult_V4xM44f_SSE(float *matrix4, float *vector4, float *res)
	{
		_asm {
	// Load Matrix
			mov EAX,matrix4
			movups XMM0, [EAX]
			movups XMM1, [EAX+16]
			movups XMM2, [EAX+32]
			movups XMM3, [EAX+48]

	// Load Vector
			mov EAX,vector4
			movups XMM4, [EAX]

	// Multiply Cols * Vector
			mulps XMM0,XMM4
			mulps XMM1,XMM4
			mulps XMM2,XMM4
			mulps XMM3,XMM4


	// transpose Matrox
			movups XMM4,XMM0
			shufps XMM4,XMM1,0x44  ; xmm4 = tmp0
			shufps XMM0,XMM1,0xEE  ; xmm0 = tmp2
			movups XMM5,XMM2
			shufps XMM5,XMM3,0x44  ; xmm5 = tmp1
			shufps XMM2,XMM3,0xEE  ; xmm2 = tmp3
			movups XMM1,XMM4
			shufps XMM4,XMM5,0x88   ; row0
			shufps XMM1,XMM5,0xDD   ; row1
			movups XMM3,XMM0
			shufps XMM0,XMM2,0x88   ; row2
			shufps XMM3,XMM2,0xDD   ; row3

	// Add results
			addps XMM4,XMM1
			addps XMM4,XMM0
			addps XMM4,XMM3

	// store res 
			mov EAX,res
			movups [EAX],xmm4
		}
	}

	void _fastcall Transpose_M44f_SSE(float *matrix4, float *res)
	{
		_asm {
	// Load Matrix
			mov EAX,matrix4
			movups XMM0, [EAX]
			movups XMM1, [EAX+16]
			movups XMM2, [EAX+32]
			movups XMM3, [EAX+48]

	// transpose Matrox
			movups XMM4,XMM0
			shufps XMM4,XMM1,0x44  ; xmm4 = tmp0
			shufps XMM0,XMM1,0xEE  ; xmm0 = tmp2
			movups XMM5,XMM2
			shufps XMM5,XMM3,0x44  ; xmm5 = tmp1
			shufps XMM2,XMM3,0xEE  ; xmm2 = tmp3
			movups XMM1,XMM4
			shufps XMM4,XMM5,0x88   ; row0
			shufps XMM1,XMM5,0xDD   ; row1
			movups XMM3,XMM0
			shufps XMM0,XMM2,0x88   ; row2
			shufps XMM3,XMM2,0xDD   ; row3

			mov EAX,res
			movups [EAX   ],XMM4
			movups [EAX+16],XMM1
			movups [EAX+32],XMM0
			movups [EAX+48],XMM3
		}
	}

	void _fastcall Mult_V4xM44f_C(float *matrix4, float *vector4, float *res)
	{
		res[0]=vector4[0]*matrix4[ 0]+vector4[1]*matrix4[ 1]+vector4[2]*matrix4[ 2]+vector4[3]*matrix4[ 3];
		res[1]=vector4[0]*matrix4[ 4]+vector4[1]*matrix4[ 5]+vector4[2]*matrix4[ 6]+vector4[3]*matrix4[ 7];
		res[2]=vector4[0]*matrix4[ 8]+vector4[1]*matrix4[ 9]+vector4[2]*matrix4[10]+vector4[3]*matrix4[11];
		res[3]=vector4[0]*matrix4[12]+vector4[1]*matrix4[13]+vector4[2]*matrix4[14]+vector4[3]*matrix4[15];
	}

	inline void xchg(float &x,float &y)
	{
		float t = x;
		x=y;
		y=t;
	}

	void _fastcall Transpose_M44f_C(float *matrix4, float *res)
	{
		if (res!=matrix4) memcpy(res,matrix4,16*sizeof(float));
		xchg(res[1],res[4]);
		xchg(res[2],res[8]);
		xchg(res[3],res[12]);

		xchg(res[6],res[9]);
		xchg(res[7],res[13]);

		xchg(res[11],res[14]);
	}

}