#ifndef MORFTYPES_H
#define MORFTYPES_H

namespace GENERAL
{
	typedef unsigned char UBYTE;

	template <class Key, class Type,class Traits = std::less<Key>, class Allocator=std::allocator<std::pair <const Key, Type> > > 
		class morfmap : public std::map<Key,Type,Traits,Allocator>
		{
		public:
			Type defReturnVal;
		public:
			inline setdefault(Type &_defaultVal)
			{
				defReturnVal = _defaultVal;
			}

			inline Type &get(Key k)
			{
				iterator it = find(k);
				return (it!=end())?it->second:defReturnVal;
			}
		};


	template <typename A> union TMorfPoint
	{
		inline TMorfPoint() {x=y=(A)0;};
		inline TMorfPoint(A _x, A _y) {x=_x; y=_y;};
		inline TMorfPoint(const TMorfPoint<A> &mp) {x=mp.x; y=mp.y;};

		inline TMorfPoint<A> & operator += (const TMorfPoint<A> &p)
		{ x+=p.x; y+=p.y; return *this; }
		inline TMorfPoint<A> & operator -= (const TMorfPoint<A> &p)
		{ x-=p.x; y-=p.y; return *this; }
		inline TMorfPoint<A> & operator = (const TMorfPoint<A> &p)
		{ x=p.x; y=p.y; return *this; }
		inline TMorfPoint<A> operator + (const TMorfPoint<A> &p)
		{ 
			TMorfPoint<A> r;
			r.x=x+p.x;
			r.y=y+p.y;
			return r; 
		}
		inline TMorfPoint<A> operator * (const A s)
		{ 
			TMorfPoint<A> r;
			r.x=x*s;
			r.y=y*s;
			return r; 
		}

		struct 
		{
			A x,y;
		};
		A array[2];
	};

/*	union TMorfPointf
	{
		inline TMorfPointf() {x=y=0;};
		inline TMorfPointf(float _x, float _y) {x=_x; y=_y;};
		inline TMorfPointf(TMorfPointf &mp) {x=mp.x; y=mp.y;};

		struct 
		{
			float x,y;
		};
		float array[2];
	};*/

/*	union TMorfRect
	{
		inline TMorfRect() {x=y=w=h=0;};
		inline TMorfRect(int _x, int _y,int _w, int _h) {x=_x; y=_y; w=_w; h=_h;};
		inline TMorfRect(TMorfPoint &pos, TMorfPoint &size) {x=pos.x; y=pos.y; w=size.x; h=size.y;};
		inline TMorfRect(TMorfRect &mr) {x=mr.x; y=mr.y; w=mr.w; h=mr.h;};

		inline bool operator == (TMorfPoint &p)
		{
			return 
				((p.x>=x) && (p.x<(x+w)) && 
				 (p.y>=y) && (p.y<(y+h)));
		}
		inline bool operator != (TMorfPoint &p)
		{
			return !(*this==p);
		}

		struct 
		{
			int x,y,w,h;
		};
		struct 
		{
			TMorfPoint pos,size;
		};
		int array[4];
		
	};*/

	template<typename A> union TMorfRect
	{
		inline TMorfRect() {x=y=w=h=(A)0;};
		inline TMorfRect(A _x, A _y,A _w, A _h) {x=_x; y=_y; w=_w; h=_h;};
		inline TMorfRect(TMorfPoint<A> &pos, TMorfPoint<A> &size) {x=pos.x; y=pos.y; w=size.x; h=size.y;};
		inline TMorfRect(const TMorfRect<A> &mr) {x=mr.x; y=mr.y; w=mr.w; h=mr.h;};

		inline bool operator == (TMorfPoint<A> &p)
		{
			return 
				((p.x>=x) && (p.x<(x+w)) && 
				 (p.y>=y) && (p.y<(y+h)));
		}
		inline bool operator != (TMorfPoint<A> &p)
		{
			return !(*this==p);
		}

		struct 
		{
			A x,y,w,h;
		};
		struct 
		{
			TMorfPoint<A> pos,size;
		};
		A array[4];
		
	};

	typedef TMorfRect<int> TMorfRecti;
	typedef TMorfRect<SCALAR> TMorfRectf;
	typedef TMorfPoint<int> TMorfPointi;
	typedef TMorfPoint<SCALAR> TMorfPointf;

/*	union TMorfRect
	{
		inline TMorfRect() {x=y=w=h=0;};
		inline TMorfRect(int _x, int _y,int _w, int _h) {x=_x; y=_y; w=_w; h=_h;};
		inline TMorfRect(TMorfPoint &pos, TMorfPoint &size) {x=pos.x; y=pos.y; w=size.x; h=size.y;};
		inline TMorfRect(TMorfRect &mr) {x=mr.x; y=mr.y; w=mr.w; h=mr.h;};

		inline bool operator == (TMorfPoint &p)
		{
			return 
				((p.x>=x) && (p.x<(x+w)) && 
				 (p.y>=y) && (p.y<(y+h)));
		}
		inline bool operator != (TMorfPoint &p)
		{
			return !(*this==p);
		}

		struct 
		{
			int x,y,w,h;
		};
		struct 
		{
			TMorfPoint pos,size;
		};
		int array[4];
		
	};*/

	typedef union
	{
		struct {
			UBYTE r,g,b;
		};
		UBYTE array[3];
	} TColor3ub;	
	
	union TColor4ub
	{

		inline TColor4ub(const TColor4ub &col)
		{
			memcpy(array,col.array,4*sizeof(UBYTE));
		}

		inline TColor4ub()
		{
			memset(array,0,4*sizeof(UBYTE));
		}

		inline TColor4ub(UBYTE _r, UBYTE _g,UBYTE _b,UBYTE _a)
		{
			r=_r;
			g=_g;
			b=_b;
			a=_a;
		}

		inline TColor4ub operator + (const TColor4ub & c)
		{
			TColor4ub r;
			r.array[0]=array[0]+c.array[0];
			r.array[1]=array[1]+c.array[1];
			r.array[2]=array[2]+c.array[2];
			r.array[3]=array[3]+c.array[3];
			return r;
		}
		inline TColor4ub operator * (const SCALAR s)
		{
			TColor4ub r;
			r.array[0]=(UBYTE)(((SCALAR)array[0])*s);
			r.array[1]=(UBYTE)(((SCALAR)array[1])*s);
			r.array[2]=(UBYTE)(((SCALAR)array[2])*s);
			r.array[3]=(UBYTE)(((SCALAR)array[3])*s);
			return r;
		}


		struct {
			UBYTE r,g,b,a;
		};
		UBYTE array[4];
		//GLuint rgba;
	};

	struct TBox
	{
		CVector3f minV;
		CVector3f maxV;

		inline TBox(TBox &_box) : minV(_box.minV), maxV(_box.maxV) {};

		inline TBox(CVector3f &_minV, CVector3f &_maxV)  : minV(_minV), maxV(_maxV) {};

		inline TBox() : minV(0.0f,0.0f,0.0f), maxV(0.0f,0.0f,0.0f) {};

	};
}

#endif