#ifndef SMARTPOINTER_H
#define SMARTPOINTER_H

#include "trace.h"

namespace GENERAL
{
	struct sptrinfo
	{
		inline sptrinfo(int _c, bool _d=false) : count(_c), deleted(_d){}; 
		int count;
		bool deleted;
	};

	extern int newcount;

	template <class T> class wptr;

	template <class T> class sptr
	{
	public:
		T *val;
		sptrinfo *info;

		inline sptr<T> &Assign(const sptr<T> &_assign)
		{
			(_assign.info->count)++;
			
			(info->count)--;
			if (!(info->count))
			{
				if (!(info->deleted)&&(val)) delete val;
				delete info;
				newcount--;
			}

			info = _assign.info;
			val = _assign.val;

			return *this;
		}

	public:
		typedef T value_type;
		typedef T *ptr_type;

		inline sptr() : val(0),info(new sptrinfo(1)) {newcount++;}
		inline sptr(T *_val) : val(_val), info(new sptrinfo(1)) {newcount++;}
		inline sptr(T *_val, sptrinfo *_info) : val(_val), info(_info) { (info->count)++; }
		inline sptr(const sptr<T> &h) : info(h.info),val(h.val) { (info->count)++; }
		inline ~sptr()
		{
			(info->count)--;

			if (info->count==0)
			{
				if (!(info->deleted)&&(val)) delete val;
				delete info;
				newcount--;
			}
		}

		inline sptr<T> &operator = (const sptr<T> &assign) {return Assign(assign);}
		inline T &operator * () { return *val; };
		inline T *operator -> () { return val; };
		inline T *operator & () { return val; };
		inline bool operator == (T *v) { return val == v; };
		inline bool operator == (const sptr<T> &v) { return val == v.val; };
		inline bool operator ! () { return val==(T*)0;};
		inline bool operator != (T *v) { return val != v; };
		inline bool operator != (const sptr<T> &v) { return val != v.val; };

		inline int GetRefCount()  { return info->count; };
		inline void ForceDelete()
		{
			if (val) delete val;
			info->deleted=true;
		}


		// cast operator
		template <class Y> operator sptr<Y> () 
		{
			return sptr<Y>((sptr<Y>::ptr_type)val,info);
		};
	};

	template <class T> class wptr
	{
	public:
		T *val;
		sptrinfo *info;

		inline wptr<T> &Assign(const sptr<T> &_assign)
		{
			info = _assign.info;
			val = _assign.val;

			return *this;
		}

		inline wptr<T> &Assign(const wptr<T> &_assign)
		{
			info = _assign.info;
			val = _assign.val;

			return *this;
		}

	public:
		typedef T value_type;
		typedef T *ptr_type;

		inline wptr() : val(NULL), info(NULL) {}
		inline wptr(T *_val, sptrinfo *_info) : val(_val), info(_info) {}
		inline wptr(const wptr<T> &h) : info(h.info),val(h.val) {}
		inline wptr(const sptr<T> &h) : info(h.info),val(h.val) {}

		inline wptr<T> &operator = (const sptr<T> &assign) {return Assign(assign);}
		inline wptr<T> &operator = (const wptr<T> &assign) {return Assign(assign);}
		inline T &operator * () { return *val; };
		inline T *operator -> () { return val; };
		inline T *operator & () { return val; };
		inline bool operator == (T *v) { return val == v; };
		inline bool operator == (const sptr<T> &v) { return val == v.val; };
		inline bool operator ! () { return val==(T*)0;};
		inline bool operator != (T *v) { return val != v; };
		inline bool operator != (const sptr<T> &v) { return val != v.val; };

		inline int GetRefCount()  { return info->count; };

		// cast operator
		template <class Y> operator sptr<Y> () 
		{
			return sptr<Y>((sptr<Y>::ptr_type)val,info);
		};
		template <class Y> operator wptr<Y> () 
		{
			return wptr<Y>((wptr<Y>::ptr_type)val,info);
		};
	};

//	template <class T, class P> class spt2 : public sptr<T>
//	{};
	/*
	{
	protected:
/*		inline sptr<T> &Assign(const sptr<P> &_assign)
		{
			sptr<P>::Assign(assign);
		}* /

	public:
		inline spt2() : sptr<P>() {}
		inline spt2(T *_val) : sptr<P>(_val) {}
		inline spt2(const spt2<T,P> &h) : sptr<P>(h) {}

		inline T &operator * () { return (T)*val; };
		inline T *operator -> () { return (T*)val; };
		inline T *operator & () { return (T*)val; };
		inline bool operator == (T *v) { return (T*)val == v; };
		inline bool operator == (spt2<T,P> &v) { return val == v.val; };
		inline bool operator ! () { return val==0;};
		inline bool operator != (T *v) { return (T*)val != v; };
		inline bool operator != (spt2<T,P> &v) { return val != v.val; };
		inline spt2<T,P> &operator = (const sptr<P> &assign)
		{	
			Assign(assign);
			return *this;
		}
	};*/


//	typedef sptr<void> HVoid;

}

#endif