#ifndef DOMAIN_LEVEL_OBJECT_H
#define DOMAIN_LEVEL_OBJECT_H

#include <morfbase.h>

namespace DOMAIN
{

	typedef enum {	MOVEMENTMODE_FLOATING  =0,
					MOVEMENTMODE_GRAVITY   =1 } EMovementModes; 

	typedef enum {	COLLISIONMODE_NONE = 0,
					COLLISIONMODE_SPHERE,
					COLLISIONMODE_BOX,
					COLLISIONMODE_POINT } ECollisionModes; 


	class CLevel : public CWorldObject
	{
	public:
		DECLARE_SERIALIZEABLE(CLevel);
		inline CLevel() : CWorldObject() {};
		inline CLevel(long _id) : CWorldObject(_id) {};
		virtual ~CLevel();
	};

	class CLevelObject : public CWorldObject
	{
	protected:

		CLevel *lvl;
		long lvlId;

	public:
		DECLARE_SERIALIZEABLE(CLevelObject);

		inline CLevelObject() : CWorldObject() { SetClassKey(GET_KEY(CLevelObject)); };
		inline CLevelObject(long _id, CLevel* _lvl) : CWorldObject(_id) , lvl(_lvl) { SetClassKey(GET_KEY(CLevelObject)); lvlId = _lvl->GetID(); };
		inline CLevelObject(long _id, int _lvlId) : CWorldObject(_id) , lvlId(_lvlId) { SetClassKey(GET_KEY(CLevelObject)); lvl = 0; };
		virtual ~CLevelObject();

		
		void SetLvl(CLevel *_lvl);
		inline CLevel *GetLvl() { return lvl; };

		inline void SetLvlId(long _lvl) { lvlId = _lvl; };
		inline long GetLvlId() { return lvlId; };


	//	inline CLevel *GetLvl() { return lvl; };

		virtual long GetSerializedSize();
		virtual long Serialize(GENERAL::CBuffer &buffer);
		virtual long DeSerialize(GENERAL::CBuffer &buffer);

//		virtual void Assign(CGameObject *o);

	};

	class CPhysicalLevelObject : public CLevelObject
	{
	public:
		struct TPDU : public CWorldObject::TPDU {
			GENERAL::CVector3f velocity;
		};
	private:
//		TPDU pdu;

	protected:
        
		GENERAL::CVector3f velocity[2];	// Geschwindigkeit des Objekts
		GENERAL::CVector3f anglevelocity;
		

		int movementMode;
		int collisionMode;

		float mass;						// Masse des Objekts
		float boundingRadius;			// der Radius der Bounding Sphere 
		GENERAL::TBox boundingBox;

		float stepHeight;				// unused here, wichtig fr collision mit bsp

	public:
		DECLARE_SERIALIZEABLE(CPhysicalLevelObject);

		inline CPhysicalLevelObject() 
				: CLevelObject()
			{ SetClassKey(GET_KEY(CPhysicalLevelObject)); };
		inline CPhysicalLevelObject(long _id,
									CLevel *_lvl,
									GENERAL::CVector3f _pos,
									GENERAL::CVector3f _rot,
									GENERAL::CVector3f _scale,
									float _mass,
									float _radius,
									GENERAL::TBox _boundingbox,
									GENERAL::CVector3f _velocity) 
									:
									CLevelObject(_id, _lvl),
									mass(mass),
									boundingRadius(_radius),
									boundingBox(_boundingbox)
								{
									//SetLvl(_lvl);
									SetClassKey(GET_KEY(CPhysicalLevelObject));

									pos[ACT] = _pos;
									rotation[ACT] = _rot;
									velocity[ACT] = _velocity;
									scale = _scale;

									stepHeight = 15.0f;

									movementMode = MOVEMENTMODE_FLOATING;
									
									//debug
									boundingRadius = 15.0f;


									collisionMode = COLLISIONMODE_SPHERE;

								};

		virtual ~CPhysicalLevelObject();

		inline float GetMass() { return mass; };
		inline float GetBoundingRadius() { return boundingRadius; };
		inline int GetMovementMode() { return movementMode; };
		inline int GetCollisionMode() { return collisionMode; };
		inline float GetStepHeight() { return stepHeight; };
		inline GENERAL::TBox &GetBoundingBox() { return boundingBox; };


		inline GENERAL::CVector3f GetVelocity() { return velocity[ACT]; };
	
		inline void SetMass(float _mass) { mass = _mass; };
		inline void SetBoundingRadius(float _radius) { boundingRadius = _radius; };
		inline void SetVelocity(GENERAL::CVector3f _vel) { velocity[ACT] = _vel; };
		inline void SetMovementMode(int _mode) { movementMode = _mode; };
		inline void SetCollisionMode(int _mode) { collisionMode = _mode; };



		
		// from CSerializeAble:
		virtual long GetSerializedSize();
		virtual long Serialize(GENERAL::CBuffer &buffer);
		virtual long DeSerialize(GENERAL::CBuffer &buffer);
		virtual void CopyPDUto(char *data);
		virtual void CopyPDUfrom(char *data);
		virtual int GetPDUSize();
		virtual void SetPDU();

		virtual void Print(FILE *f,bool newline=true);
		//

//		virtual void Assign(CGameObject *o);
		virtual void TimeStep(GENERAL::TGameTick dt); 
		
		virtual void MoveTo(GENERAL::CVector3f &_pos);
		virtual void Forward(float amount);
		virtual void LookX(float angle);
		virtual void LookY(float angle);
		virtual void LookZ(float angle);

	};
}

#endif