#include "md3object.h"
#include "characterobject.h"

namespace DOMAIN
{
	CMD3Object::tLink::~tLink() 
	{
	}


	IMPLEMENT_SERIALIZEABLE_DOMAIN(CMD3Object);
	IMPLEMENT_SERIALIZEABLE_DOMAIN(CWeapon);
	IMPLEMENT_SERIALIZEABLE_DOMAIN(CShot);
		
	CMD3Object::CMD3Object(CDomain *_domain)
		: CPhysicalLevelObject(_domain)
	{
		SetClassKey(GET_KEY(CMD3Object));

		AddTag(1);
		AddTag(2);
		AddTag(3);
		AddTag(4);
		AddTag(5);
	}

	CMD3Object::CMD3Object(
			CDomain *_domain,
			int _odid,
			long _id,
			CLevel *_lvl,
			GENERAL::CVector3f _pos,
			GENERAL::CVector3f _rot,
			GENERAL::CVector3f _scale,
			float _mass,
			float _radius,
			GENERAL::TBox &_boundingBox,
			GENERAL::CVector3f _velocity)
		: CPhysicalLevelObject(_domain,_odid,_id, _lvl, _pos, _rot, _scale, _mass, _radius, _boundingBox, _velocity)
	{
		SetClassKey(GET_KEY(CMD3Object));

		AddTag(1);
		AddTag(2);
		AddTag(3);
		AddTag(4);
		AddTag(5);

		targetLink = 0;
		targetTag = 0;
		worldLink = 0;
	}



	CMD3Object::~CMD3Object()
	{
		links.clear();
	}

	bool CMD3Object::Load()
	{
//		collisionMode = COLLISIONMODE_NONE;
//		boundingRadius = 10;
		return CPhysicalLevelObject::Load();
	}

	void CMD3Object::Done()
	{
		CPhysicalLevelObject::Done();
		//links.clear();
	}

	long CMD3Object::GetSerializedSize()
	{
		long linkDataSize = 
			sizeof (unsigned short int) + // num of Tags
			(sizeof(long)+sizeof(unsigned short int)) * (long)links.size(); // tagheader (tagType, num of linked objects)
		tLinkMap::iterator it;
		for (it = links.begin(); it != links.end(); it++)
		{
			linkDataSize+=sizeof(long)*((long)it->second.size());
		}

		return CPhysicalLevelObject::GetSerializedSize()+linkDataSize;
	}

	long CMD3Object::Serialize(GENERAL::CBuffer &buffer)
	{
		CPhysicalLevelObject::Serialize(buffer);

		tLinkMap::iterator it;

		unsigned short int numTags = (unsigned short int)links.size();
		buffer.Write(&numTags,sizeof(unsigned short int));
		for (it = links.begin(); it != links.end(); it++)
		{
			long tagID = it->first;
			buffer.Write(&tagID,sizeof(long));
			unsigned short int numLinked = (unsigned short int)it->second.size();
			buffer.Write(&numLinked,sizeof(unsigned short int));

			std::vector<tLink>::iterator lit;
			for (lit=it->second.begin(); lit!=it->second.end(); lit++)
			{
				buffer.Write(&lit->id,sizeof(lit->id));
			}
		}

		return buffer.GetPos();
	}

	long CMD3Object::DeSerialize(GENERAL::CBuffer &buffer)
	{
		CPhysicalLevelObject::DeSerialize(buffer);

		links.clear();

		tLinkMap::iterator it;

		unsigned short int numTags = *(unsigned short int*)buffer.Read(sizeof(unsigned short int));
		while (numTags--)
		{
			long tagID = *(long*)buffer.Read(sizeof(long));
			AddTag(tagID);

			unsigned short int numLinked = *(unsigned short int*)buffer.Read(sizeof(unsigned short int));

			while (numLinked--)
			{
				long linkID = *(long*)buffer.Read(sizeof(long));

				if (linkID<0)
					AddLink(tagID,linkID);
				else 
				{	
//					AddLink((sptr<CMD3Object>)domain->GetObjMan()->GetPtr(linkID));
				}
			}
		}

		return buffer.GetPos();
	}

	void CMD3Object::GetSerializeObjectsBefore(std::vector<sptr<CGameObject> > &_objs)
	{
		CPhysicalLevelObject::GetSerializeObjectsBefore(_objs);

		tLinkMap::iterator lit;

		for (lit = links.begin(); lit!=links.end(); lit++)
		{
			std::vector<tLink>::iterator it;
			for (it = lit->second.begin(); it!= lit->second.end(); it++)
			{
				if (it->object!=NULL)
					_objs.push_back(it->object);
			}
		}
	}

	int CMD3Object::ReadObjectDescriptor(sptr<GAME::CObjectDescriptor> _od)
	{
		CPhysicalLevelObject::ReadObjectDescriptor(_od);

		//GAME::CObjectDescriptor::TPropertyMap::iterator it;

		GAME::CObjectDescriptor::TStatePropertyMap *sp;
		GAME::CObjectDescriptor::TStatePropertyMap::iterator it;

		sp = _od->GetPropertyMap("EQUIPPED");
		if (sp)
		{
			it = sp->find("link");
			if (it!=sp->end()) targetLink = it->second.iVal;
	
			it = sp->find("tag");
			if (it!=sp->end()) targetTag = it->second.iVal;
		} 

		sp = _od->GetPropertyMap("INWORLD");
		if (sp)
		{
			it = sp->find("link");
			if (it!=sp->end()) worldLink = it->second.iVal;
		}

		return 0;
	}
/*	void CMD3Object::CopyPDUto(char *data)
	{
	}*/

	void CMD3Object::SetPDU()
	{
		if (fsmState[ACT]!=fsmState[PDU])
			feCommands.push(fsmState[ACT]);
		CPhysicalLevelObject::SetPDU();

	}

/*	void CMD3Object::CopyPDUfrom(char *data)
	{
	}

	int CMD3Object::GetPDUSize()
	{
	}

	float CMD3Object::GetPDUTreshHold()
	{
	}*/

	void CMD3Object::TimeStep(GENERAL::TGameTick dt)
	{
		if (doDeathReckoning)
		{
			if (fsmState[ACT]!=fsmState[PDU])
			{
				feCommands.push(fsmState[PDU]);
				fsmState[ACT]=fsmState[PDU];
			}
		}

		if (GetID()==6000)
		{
			int blu = 0;
		//	LookY(0.01);

		}
		else
		{
			//if (GetID()>0)
			{
				LookY(0.01);
			}
		}


		CPhysicalLevelObject::TimeStep(dt);
	}

//	void CMD3Object::Print(FILE *f,bool newline=true);
//	void CMD3Object::Assign(CGameObject *o);
		
	int CMD3Object::AddLink(int _tag, int _id)
	{
		sptr<FRONTEND::CRenderAble> ra(NULL);
		if (APPLICATION::CApplication::GetApplication()->GetFE())
			ra=APPLICATION::CApplication::GetApplication()->GetFE()->CreateObject(this,_id,_tag);

		links[_tag].clear();
		links[_tag].push_back(tLink(_id,ra));
		if (_tag==1)
			SetfeData(ra);

//		if (APPLICATION::CApplication::GetApplication()->GetFE())
//			APPLICATION::CApplication::GetApplication()->GetFE()->CreateObject(this,0);
		/// @todo create frontendobject here (if necessary)
		return 0;
	}

	void CMD3Object::AddChild(sptr<CGraphObject> _child)
	{
		CPhysicalLevelObject::AddChild(_child);

		if (ISOFTYPE(_child,CMD3Object))
		{
			AddLink((sptr<CMD3Object>)_child);
		}

	}

	void CMD3Object::RemoveChild(sptr<CGraphObject> _child)
	{
		CPhysicalLevelObject::RemoveChild(_child);

		if (ISOFTYPE(_child,CMD3Object))
			RemoveLink((sptr<CMD3Object>)_child);
	}

	int CMD3Object::AddLink(sptr<DOMAIN::CMD3Object> &_obj)
	{	
		return 0;
		if (_obj==NULL) return -1;

		sptr<FRONTEND::CRenderAble> ra(NULL);
		if (APPLICATION::CApplication::GetApplication()->GetFE())
		{
			ra=APPLICATION::CApplication::GetApplication()->GetFE()->CreateObject(&_obj,_obj->GetTargetLink(),_obj->GetTargetTag());
			_obj->SetfeData(ra);
		}

		links[_obj->GetTargetTag()].push_back(tLink(_obj->GetTargetLink(),ra,_obj));
		if (_obj->GetTargetTag()==1)
			SetfeData(ra);

		_obj->SetContainerType(CONFIG::g_config.ObjectStatus["EQUIPPED"]);



//		if (APPLICATION::CApplication::GetApplication()->GetFE())
//			APPLICATION::CApplication::GetApplication()->GetFE()->CreateObject(this,0);
		/// @todo create frontendobject here (if necessary)
		return 0;
	}

	int CMD3Object::RemoveLink(sptr<DOMAIN::CMD3Object> &_obj)
	{
		tLinkMap::iterator lit;

		for (lit = links.begin(); lit!=links.end(); lit++)
		{
			std::vector<tLink>::iterator it;
			for (it = lit->second.begin(); it!= lit->second.end(); it++)
			{
				if (_obj == it->object)
				{
					lit->second.erase(it);
					return 0;
				}
			}
		}
		return -1;
	}

	int CMD3Object::RemoveLink(int _tag, int _id)
	{
		return 0;
	}

	int CMD3Object::AddTag(int _id)
	{
		std::vector<tLink> nothinglinked;

		links[_id]=nothinglinked;
		return 0;
	}

	int CMD3Object::RemoveTag(int _id)
	{
		tLinkMap::iterator it = links.find(_id);
		if (it==links.end())
		{
			GENERAL::dprintf("[ WARN ] RemoveTag: Tag nor Found %d!\n",_id);
			return -1;
		}

		links.erase(it);
		return 0;
	}

	bool CMD3Object::HasTag(int _id)
	{
		tLinkMap::iterator it = links.find(_id);
		return (it!=links.end());
	}

	bool CMD3Object::HasTagRecursive(int _id)
	{
		return false;
	}

	bool CMD3Object::HasLink(int _id)
	{
		return false;
	}

	bool CMD3Object::HasLinkRecursive(int _id)
	{
		return false;
	}

	void CMD3Object::GetActionList(CGameObject::tActionList &actions)
	{
		CPhysicalLevelObject::GetActionList(actions);
	//	actions.push_back(ACTION_WALK);
	//	actions.push_back(ACTION_RUN);
	//	actions.push_back(ACTION_STOP);
	//	actions.push_back(ACTION_STATUS);
	}

	void CMD3Object::ActionUse(sptr<CWorldObject> &target)
	{
	}

	int CWeapon::ReadObjectDescriptor(sptr<GAME::CObjectDescriptor> _od)
	{
		GAME::CObjectDescriptor::TStatePropertyMap *sp;
		GAME::CObjectDescriptor::TStatePropertyMap::iterator it;

		sp = _od->GetPropertyMap("WEAPON");
		if (sp)
		{
			it = sp->find("projectileod");
			if (it!=sp->end()) projectileodid = domain->GetObjMan()->GetDescriptorID((char*)it->second.sVal.data());
	
			it = sp->find("shootspeed");
			if (it!=sp->end()) shootSpeed = it->second.iVal;
		} 
		return CMD3Object::ReadObjectDescriptor(_od);
	}

	void CWeapon::ActionUse(sptr<CWorldObject> &target)
	{
		if (ISOFTYPE(parent,CCharacterObject))
		{
			if ( ((sptr<CCharacterObject>)target)->GetHealth()<=0)
			{
				return;

			}


			CCharacterObject* co = (CCharacterObject*)parent;
			CVector3f dir=(target->GetPos() - co->GetPos());
			dir.Normal();

			if (dir.ScalarMultiply(co->GetRotation())<0.9f)
			{
				co->IncNextUse(100);
			}
			else 
			{
				CVector3f srot;
				srot.x  = (((SCALAR)(rand()&0xff))-128.0f) / 50.0f;
				srot.y  = (((SCALAR)(rand()&0xff))-128.0f) / 50.0f;
				srot.z  = (((SCALAR)(rand()&0xff))-128.0f) / 50.0f;

				sptr<DOMAIN::CShot> shot(
					new DOMAIN::CShot(
							domain,
							target,
							projectileodid,
							0,
							co->GetLvl(),
							co->GetPos(),
							srot,
							CVector3f(1.0f,1.0f,1.0f),
							0.0f,
							0.0f,
							TBox(),
							dir));
				domain->GetObjMan()->Add(shot);
				co->GetLvl()->AddChild(shot);
				shot->SetLvl(co->GetLvl());
				shot->Load();
				shot->SetControl();
				shot->GetLvl()->LevelObjectMoved(&shot);	
				co->feCommands.push(5);
				co->IncNextUse(shootSpeed);
			}
		}
	}


	CShot::CShot(CDomain *_domain) : CMD3Object(_domain),lifetime(100000) 
	{
		SetClassKey(GET_KEY(CShot));
	};
	CShot::CShot(
			CDomain *_domain,
			sptr<CWorldObject> &_target,
			int _odid,
			long _id,
			CLevel *_lvl,
			GENERAL::CVector3f _pos,
			GENERAL::CVector3f _rot,
			GENERAL::CVector3f _scale,
			float _mass,
			float _radius,
			GENERAL::TBox &_boundingBox,
			GENERAL::CVector3f _velocity)
			: CMD3Object(_domain,_odid,_id,_lvl,_pos,_rot,_scale,_mass,_radius,_boundingBox,_velocity),lifetime(100000),target(_target) 
	{
		SetClassKey(GET_KEY(CShot));
	}


	int CShot::ReadObjectDescriptor(sptr<GAME::CObjectDescriptor> _od)
	{
		GAME::CObjectDescriptor::TStatePropertyMap *sp;
		GAME::CObjectDescriptor::TStatePropertyMap::iterator it;

		sp = _od->GetPropertyMap("COMMON");
		if (sp)
		{
			it = sp->find("flyspeed");
			if (it!=sp->end()) flySpeed = it->second.fVal;
	
			it = sp->find("damage");
			if (it!=sp->end()) damage = it->second.iVal;

			it = sp->find("shotlink");
			if (it!=sp->end()) shotLink = it->second.iVal;

			it = sp->find("impactlink");
			if (it!=sp->end()) impactLink = it->second.iVal;

		} 
		return CMD3Object::ReadObjectDescriptor(_od);
	}

	bool CShot::Load()
	{
		bool res = CMD3Object::Load();
		SetMovementMode(MOVEMENTMODE_FLOATING);
		SetCollisionMode(COLLISIONMODE_SPHERE);
		AddLink(1,shotLink); // rootnode 

		SetRotation(GetVelocity());

		CVector3f displace(0.0f,20.0f,0.0f);
		displace+=GetVelocity()*10.0f;
		displace+=GetPos();
		MoveTo(GetLvl()->Trace(this,GetPos(), displace));

		SetVelocity(GetVelocity()*flySpeed);

		return res;
	}


	ECollisionAction CShot::Collided(CVector3f pos, CVector3f normal)
	{
		normal = normal;
		AddLink(1,impactLink);
		SetRotation(normal);
		SetVelocity(CVector3f(0.0f,0.0f,0.0f));
		lifetime=500;
		return COLLISIONACTION_STOP;
	}
	
	void CShot::TimeStep(GENERAL::TGameTick dt)
	{
		CMD3Object::TimeStep(dt);

		float fdt = (float)dt;
		float vel = GetVelocity().abs2();
		fdt*=vel;

//		CVector3f &rot = GetRotation();
//		rot.x+=fdt*0.01f;
//		rot.y+=fdt*0.02f;
//		rot.z+=fdt*0.04f;

		CVector3f pos = GetPos();
		pos.y-=20;



	


		if (lifetime!=100000)
		{
			lifetime-=dt;
			if (lifetime<=0)
			{
				sptr<CShot> self = (sptr<CShot>)domain->GetObjMan()->GetPtr(GetID());
				GetLvl()->RemoveChild(self);
				domain->GetObjMan()->Release(self);
			}
		} 
		else
		{
			if (((pos-target->GetPos()).abs2())<(10.0f*10.0f))
			{
				CVector3f vel = GetVelocity();
				vel.Normal();
				Collided(pos,vel);

				if (ISOFTYPE(target,DOMAIN::CCharacterObject))
				{
					((DOMAIN::CCharacterObject*)&target)->ActionHit(damage);
				}
			}
		}
	}
}