#ifndef DOMAIN_MD3OBJECT
#define DOMAIN_MD3OBJECT

#include <morfbase.h>

namespace DOMAIN
{
	class CMD3Object : public CPhysicalLevelObject
	{
	public:

		struct tLink
		{
			inline tLink(int _id, sptr<FRONTEND::CRenderAble> &_link) : id(_id), object(0), link(_link) {};
			inline tLink(int _id, sptr<FRONTEND::CRenderAble> &_link, sptr<CMD3Object> &_object) : id(_id), object(_object), link(_link) {};
			~tLink();

			int id;
			sptr<FRONTEND::CRenderAble> link;
			sptr<CMD3Object> object;
		};

		typedef std::map<unsigned short int, std::vector<tLink> > tLinkMap;

	protected:
		tLinkMap links;
		int targetTag;
		int targetLink;
		int worldLink;

	public:
		std::queue<long> feCommands;

		DECLARE_SERIALIZEABLE(CMD3Object);
		
		CMD3Object(CDomain *_domain);
		CMD3Object(
			CDomain *_domain,
			int _odid,
			long _id,
			CLevel *_lvl,
			GENERAL::CVector3f _pos,
			GENERAL::CVector3f _rot,
			GENERAL::CVector3f _scale,
			float _mass,
			float _radius,
			GENERAL::TBox &_boundingBox,
			GENERAL::CVector3f _velocity);
		virtual ~CMD3Object();

		virtual bool Load();
		virtual void Done();

		virtual long GetSerializedSize();
		virtual long Serialize(GENERAL::CBuffer &buffer);
		virtual long DeSerialize(GENERAL::CBuffer &buffer);
		virtual void GetSerializeObjectsBefore(std::vector<sptr<CGameObject> > &_objs);

		virtual void GetActionList(CGameObject::tActionList &actions);
		virtual int ReadObjectDescriptor(sptr<GAME::CObjectDescriptor> _od);



		virtual void SetPDU();
/*		virtual void CopyPDUto(char *data);
		virtual void CopyPDUfrom(char *data);
		virtual int GetPDUSize();
		virtual float GetPDUTreshHold();*/

		virtual void TimeStep(GENERAL::TGameTick dt);

//		virtual void Print(FILE *f,bool newline=true);
//		virtual void Assign(CGameObject *o);

		virtual void AddChild(sptr<CGraphObject> _child);
		virtual void RemoveChild(sptr<CGraphObject> _child);
		int AddLink(int _tag, int _id);		//  wenn link ein CMD3Object --> dessen feStatus auf FESTATUS_TAGGED setzen
		virtual int AddLink(sptr<DOMAIN::CMD3Object> &_obj);		//  wenn link ein CMD3Object --> dessen feStatus auf FESTATUS_TAGGED setzen
		virtual int RemoveLink(int _tag, int _id);  //  --"-- FESTATUS_NORMAL
		virtual int RemoveLink(sptr<DOMAIN::CMD3Object> &_obj);		//  wenn link ein CMD3Object --> dessen feStatus auf FESTATUS_TAGGED setzen

		int AddTag(int _id);
		int RemoveTag(int _id);

		bool HasTag(int _id);
		bool HasTagRecursive(int _id);
		bool HasLink(int _id);
		bool HasLinkRecursive(int _id);

		inline tLinkMap &GetLinks() { return links; };
		inline int GetTargetTag() { return targetTag; };
		inline int GetTargetLink() { return targetLink; };
		inline int GetWorldLink() { return worldLink; };

		virtual void ActionUse(sptr<CWorldObject> &target);

		//void TagsChanged();		// send a notification to the server
		//void LinksChanged();	// send a notification to the server
	};

	class CWeapon : public CMD3Object
	{
	protected:
		int shootSpeed;
		int projectileodid;
	public:
		DECLARE_SERIALIZEABLE(CWeapon);
		inline CWeapon(CDomain *_domain) : CMD3Object(_domain) 
		{
			SetClassKey(GET_KEY(CWeapon));
		};
		inline CWeapon(
			CDomain *_domain,
			int _odid,
			long _id,
			CLevel *_lvl,
			GENERAL::CVector3f _pos,
			GENERAL::CVector3f _rot,
			GENERAL::CVector3f _scale,
			float _mass,
			float _radius,
			GENERAL::TBox &_boundingBox,
			GENERAL::CVector3f _velocity) : CMD3Object(_domain, _odid, _id, _lvl, _pos, _rot, _scale, _mass, _radius, _boundingBox, _velocity) 
		{
			SetClassKey(GET_KEY(CWeapon));
		};

		virtual int ReadObjectDescriptor(sptr<GAME::CObjectDescriptor> _od);
		virtual void ActionUse(sptr<CWorldObject> &target);
	};

	class CShot : public CMD3Object
	{
	protected:
		TGameTick lifetime;
		sptr<CWorldObject> target;

		SCALAR flySpeed;
		long   shotLink;
		long   impactLink;
		long   damage;
	public:
		DECLARE_SERIALIZEABLE(CShot);
		CShot(CDomain *_domain);
		CShot(
			CDomain *_domain,
			sptr<CWorldObject> &_target,
			int _odid,
			long _id,
			CLevel *_lvl,
			GENERAL::CVector3f _pos,
			GENERAL::CVector3f _rot,
			GENERAL::CVector3f _scale,
			float _mass,
			float _radius,
			GENERAL::TBox &_boundingBox,
			GENERAL::CVector3f _velocity);
		
		virtual bool Load();

		virtual int ReadObjectDescriptor(sptr<GAME::CObjectDescriptor> _od);
		virtual ECollisionAction Collided(CVector3f pos, CVector3f normal);
		virtual void TimeStep(GENERAL::TGameTick dt);

	};
}

#endif