#include "md3.h"

#include <morfbase.h>

namespace GENERAL
{
	int g_MD3TorsoStateAnimation[4] = { 11,11,0,1 };
	int g_MD3LegsStateAnimation[4] = { 15,7,0,1 };

	///////////////////////////////// CLOAD MD3 \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
	/////
	/////	This constructor initializes the md3 structures
	/////
	///////////////////////////////// CLOAD MD3 \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*

	CLoadMD3::CLoadMD3()
	{
		memset(&m_Header, 0, sizeof(tMD3Header));

		

		m_pSkins=NULL;
		m_pTexCoords=NULL;
		m_pTriangles=NULL;
		m_pFrames=NULL;
	}


	///////////////////////////////// IMPORT MD3 \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
	/////
	/////	This is called by the client to open the .Md3 file, read it, then clean up
	/////
	///////////////////////////////// IMPORT MD3 \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*

	bool CLoadMD3::ImportMD3(tMD3Model *pModel, char *strFileName)
	{
		char strMessage[255] = {0};

		m_FilePointer = fopen(strFileName, "rb");

		if(!m_FilePointer) 
		{
//			sprintf(strMessage, "Unable to find the file: %s!", strFileName);
//			MessageBox(NULL, strMessage, "Error", MB_OK);
			dprintf("[FAILED] LoadMD3: Unable to find the file: %s!\n", strFileName);
			return false;
		}
		
		fread(&m_Header, 1, sizeof(tMD3Header), m_FilePointer);

		char *ID = m_Header.fileID;

		if((ID[0] != 'I' || ID[1] != 'D' || ID[2] != 'P' || ID[3] != '3') || m_Header.version != 15)
		{
			dprintf("[FAILED] LoadMD3: Invalid file format (Version not 15): %s!\n", strFileName);
			return false;
		}
		
		ReadMD3Data(pModel);

		pModel->currentAnim =0;


		CleanUp();

		return true;
	}


	///////////////////////////////// READ MD3 DATA \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
	/////
	/////	This function reads in all of the model's data, except the animation frames
	/////
	///////////////////////////////// READ MD3 DATA \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*

	void CLoadMD3::ReadMD3Data(tMD3Model *pModel)
	{
		int i = 0;

		m_pFrames = new tMD3Frame [m_Header.numFrames];
		fread(m_pFrames, sizeof(tMD3Frame), m_Header.numFrames, m_FilePointer);

		pModel->numOfFrames = m_Header.numFrames;
		pModel->pFrames = m_pFrames;
		
		pModel->pTags = new tMD3Tag [m_Header.numFrames * m_Header.numTags];
		fread(pModel->pTags, sizeof(tMD3Tag), m_Header.numFrames * m_Header.numTags, m_FilePointer);


/*		for (int n=0; n<m_Header.numFrames * m_Header.numTags; n++)
		{
			GENERAL::dprintf("%s %f,%f,%f\n", pModel->pTags[n].strName, pModel->pTags[n].vPosition.x, pModel->pTags[n].vPosition.y,pModel->pTags[n].vPosition.z);
		}*/


		pModel->numOfTags = m_Header.numTags;
		
		pModel->pLinks = (tMD3Model **) malloc(sizeof(tMD3Model) * m_Header.numTags);
		
		for (i = 0; i < m_Header.numTags; i++)
			pModel->pLinks[i] = NULL;

		long meshOffset = ftell(m_FilePointer);

		tMD3MeshInfo meshHeader;

		for (i = 0; i < m_Header.numMeshes; i++)
		{
			fseek(m_FilePointer, meshOffset, SEEK_SET);
			fread(&meshHeader, sizeof(tMD3MeshInfo), 1, m_FilePointer);

			m_pSkins     = new tMD3Skin [meshHeader.numSkins];
			m_pTexCoords = new tMD3TexCoord [meshHeader.numVertices];
			m_pTriangles = new tMD3Face [meshHeader.numTriangles];
			m_pVertices  = new tMD3Triangle [meshHeader.numVertices * meshHeader.numMeshFrames];

			fread(m_pSkins, sizeof(tMD3Skin), meshHeader.numSkins, m_FilePointer);
			
			fseek(m_FilePointer, meshOffset + meshHeader.triStart, SEEK_SET);
			fread(m_pTriangles, sizeof(tMD3Face), meshHeader.numTriangles, m_FilePointer);

			fseek(m_FilePointer, meshOffset + meshHeader.uvStart, SEEK_SET);
			fread(m_pTexCoords, sizeof(tMD3TexCoord), meshHeader.numVertices, m_FilePointer);

			fseek(m_FilePointer, meshOffset + meshHeader.vertexStart, SEEK_SET);
			fread(m_pVertices, sizeof(tMD3Triangle), meshHeader.numMeshFrames * meshHeader.numVertices, m_FilePointer);

			ConvertDataStructures(pModel, meshHeader);

			delete [] m_pSkins;    
			delete [] m_pTexCoords;
			delete [] m_pTriangles;
			delete [] m_pVertices;   

			meshOffset += meshHeader.meshSize;
		}
	}


	///////////////////////////////// CONVERT DATA STRUCTURES \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
	/////
	/////	This function converts the .md3 structures to our own model and object structures
	/////
	///////////////////////////////// CONVERT DATA STRUCTURES \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*

	void CLoadMD3::ConvertDataStructures(tMD3Model *pModel, tMD3MeshInfo meshHeader)
	{
		int i = 0;

		pModel->numOfObjects++;
			
		tMD3Object currentMesh = {0};

		strcpy(currentMesh.strName, meshHeader.strName);

		currentMesh.numOfVerts   = meshHeader.numVertices;
		currentMesh.numTexVertex = meshHeader.numVertices;
		currentMesh.numOfFaces   = meshHeader.numTriangles;

		currentMesh.pVerts    = new CVector3 [currentMesh.numOfVerts * meshHeader.numMeshFrames];
		currentMesh.pNormals  = new CVector3f [currentMesh.numOfVerts * meshHeader.numMeshFrames];
		currentMesh.pTexVerts = new CVector2 [currentMesh.numOfVerts];
		currentMesh.pFaces    = new tMD3FaceTex [currentMesh.numOfFaces];

		for (i=0; i < currentMesh.numOfVerts * meshHeader.numMeshFrames; i++)
		{
			currentMesh.pVerts[i].x =  m_pVertices[i].vertex[0] / 64.0f;
			currentMesh.pVerts[i].y =  m_pVertices[i].vertex[1] / 64.0f;
			currentMesh.pVerts[i].z =  m_pVertices[i].vertex[2] / 64.0f;
			
			float lat = ((float)m_pVertices[i].normal[0]) * (2*PI) / 255;
			float lng = ((float)m_pVertices[i].normal[1]) * (2*PI) / 255;
			currentMesh.pNormals[i].x = cosf(lat)*sinf(lng);
			currentMesh.pNormals[i].y = sinf(lat)*sinf(lng);
			currentMesh.pNormals[i].z = cosf(lng);
		}

		for (i=0; i < currentMesh.numTexVertex; i++)
		{
			currentMesh.pTexVerts[i].x =  m_pTexCoords[i].textureCoord[0];
			currentMesh.pTexVerts[i].y =  m_pTexCoords[i].textureCoord[1];
		}

		for(i=0; i < currentMesh.numOfFaces; i++)
		{
			currentMesh.pFaces[i].vertIndex[0] = m_pTriangles[i].vertexIndices[0];
			currentMesh.pFaces[i].vertIndex[1] = m_pTriangles[i].vertexIndices[1];
			currentMesh.pFaces[i].vertIndex[2] = m_pTriangles[i].vertexIndices[2];

			currentMesh.pFaces[i].coordIndex[0] = m_pTriangles[i].vertexIndices[0];
			currentMesh.pFaces[i].coordIndex[1] = m_pTriangles[i].vertexIndices[1];
			currentMesh.pFaces[i].coordIndex[2] = m_pTriangles[i].vertexIndices[2];
		}
	
		pModel->pObject.push_back(currentMesh);
	}


	///////////////////////////////// LOAD SKIN \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
	/////
	/////	This loads the texture information for the model from the *.skin file
	/////
	///////////////////////////////// LOAD SKIN \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*

	bool CLoadMD3::LoadSkin(tMD3Model *pModel, char *strSkin)
	{
		if(!pModel || !strSkin) return false;

		std::ifstream fin(strSkin);

		if(fin.fail())
		{
			dprintf("[FAILED] LoadMD3: Unable to load skin %s!\n", strSkin);
			return false;
		}

		std::string strLine = "";
		int textureNameStart = 0;

		while(getline(fin, strLine))
		{
			for(int i = 0; i < pModel->numOfObjects; i++)
			{
				
				char *strr = pModel->pObject[i].strName;

				if( IsInString(strLine, pModel->pObject[i].strName) )			
				{			
					for(int j = (int)strLine.length() - 1; j > 0; j--)
					{
						if(strLine[j] == '/')
						{
							textureNameStart = j + 1;
							break;
						}	
					}
				

					tMD3MaterialInfo texture;

					strcpy(texture.strFile, &strLine[textureNameStart]);
					
					texture.uTile = texture.uTile = 1;

					pModel->pObject[i].materialId = pModel->numOfMaterials;
					pModel->pObject[i].hasTexture = true;

					pModel->numOfMaterials++;

					pModel->pMaterials.push_back(texture);
				}
			}
		}
		fin.close();
		return true;
	}

	///////////////////////////////// LOAD SHADER \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
	/////
	/////	This loads the basic shader texture info associated with the weapon model
	/////
	///////////////////////////////// LOAD SHADER \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
	
	bool CLoadMD3::LoadShader(tMD3Model *pModel, char *strShader)
	{
		if(!pModel || !strShader) return false;

		std::ifstream fin(strShader);

		if(fin.fail())
		{
			dprintf("[FAILED] LoadMD3: Unable to load shader  %s!\n", strShader);
			return false;
		}

		std::string strLine = "";
		int currentIndex = 0;
		
		while(getline(fin, strLine))
		{
			tMD3MaterialInfo texture;
			strcpy(texture.strFile, strLine.c_str());
				
			texture.uTile = texture.uTile = 1;

			pModel->pObject[currentIndex].materialId = pModel->numOfMaterials;
			pModel->pObject[currentIndex].hasTexture = true;

			pModel->numOfMaterials++;

			pModel->pMaterials.push_back(texture);

			currentIndex++;
		}

		fin.close();
		return true;
	}

	///////////////////////////////// CLEAN UP \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
	/////
	/////	This function cleans up our allocated memory and closes the file
	/////
	///////////////////////////////// CLEAN UP \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*
	
	void CLoadMD3::CleanUp()
	{
		fclose(m_FilePointer);						
	}

	

}