#ifndef FILEFORMATS_MD3MODEL
#define FILEFORMATS_MD3MODEL

#include <morfbase.h>

namespace MD3MODEL
{
	typedef enum 
	{
		BOTH_DEATH1 = 0,		
		BOTH_DEAD1,				
		BOTH_DEATH2,			
		BOTH_DEAD2,				
		BOTH_DEATH3,			
		BOTH_DEAD3,				

		TORSO_GESTURE,			
		TORSO_ATTACK,			
		TORSO_ATTACK2,			
		TORSO_DROP,				
		TORSO_RAISE,			
		TORSO_STAND,			
		TORSO_STAND2,			

		LEGS_WALKCR,			
		LEGS_WALK,				
		LEGS_RUN,				
		LEGS_BACK,				
		LEGS_SWIM,				
		LEGS_JUMP,				
		LEGS_LAND,				
		LEGS_JUMPB,				
		LEGS_LANDB,				
		LEGS_IDLE,				
		LEGS_IDLECR,			
		LEGS_TURN,				

		MAX_ANIMATIONS			
	} eMD3Animations;
	
	typedef enum 
	{
		MD3STATE_IDLE = 0,
		MD3STATE_WALK,
		MD3STATE_DIE,
		MD3STATE_DEAD,
		
		MD3_MAX_STATES			
	} eMD3States;

	typedef enum 
	{
		MD3COMMAND_NOC = -1,
		MD3COMMAND_FINISHED,
		MD3COMMAND_STOP,
		MD3COMMAND_WALK,
		MD3COMMAND_DIE,
		MD3COMMAND_RESURRECT
				
	} eMD3StateCommands;

/*
	class GENERAL::CVector3f 
	{
	public:
		float x, y, z;
	};

	class GENERAL::CVector2f 
	{
	public:
		float x, y;
	};*/

	struct tMD3Vertex
	{
		signed short vertex[3];			
		unsigned char normal[2];			
	};

	struct tMD3Triangle
	{
		int vertexIndices[3];				
	};

	struct tMD3TexCoord
	{
		GENERAL::CVector2f textureCoord;
	};

	struct tMD3Shader 
	{
		char strName[64];
		long index;
	};

	struct tMD3MaterialInfo
	{
		char strName[255];
		char strFile[255];

		unsigned char  color[3];	

		int textureId;

		float uTile, vTile;
		float uOffset, vOffset;
	};

	struct tMD3Mesh
	{
		int numOfVerts;
		int numOfTriangles;
		int numShaders;
		int materialId;

		char strName[255];

		tMD3Shader *pShaders;
		GENERAL::CVector3f  *pVerts;		
		GENERAL::CVector3f  *pNormals;	
		GENERAL::CVector2f  *pTexCoords;	

		tMD3Triangle *pTriangles;
	};

	struct tMD3AnimationInfo
	{
		//char strName[255];
		int id;
		int startFrame;
		int numFrames;
		int loopingFrames;
		int framesPerSecond;

	};

	struct tMD3Frame
	{
		float	mins[3];					
		float	maxs[3];					
		float	origin[3];				
		float	boundingRadius;						
		char	name[16];				
	};

	struct tMD3TagLoader
	{
		char		strName[64];			
		GENERAL::CVector3f	vPosition;				
		GENERAL::CMatrix3f	rotation;			
	};

	struct tMD3Tag
	{
		int			tagid;
		GENERAL::CVector3f	vPosition;				
		GENERAL::CMatrix3f	rotation;			
	};
	
	struct tMD3Header
		// siehe http://linux.ucla.edu/~phaethon/q3/formats/md3format.html
	{ 
		char	fileID[4];					
		int		version;		// version			
		char	strFile[68];	// name			
		int		numFrames;					
		int		numTags;					
		int		numMeshes;		// num_surfaces			
		int		numMaxSkins;	// num_skins			
		int		ofsFrames;		// ofs_frames			
		int		ofsTags;		// ofs_tags			
		int		ofsMeshes;		// ofs_surfaces				
		int		fileSize;		// ofs_eof			
	};

	struct tMD3MeshInfo
	{
		char	meshID[4];					
		char	strName[64];
		long	flags;
		long	numMeshFrames;				
		long	numShader;					
		long    numVertices;				
		long	numTriangles;				
		long	ofsTriangles;					
		long	ofsShader;					
		long    ofsTexCoord;					
		long	ofsVertices;				
		long	meshSize;					
	};

	struct tMD3Link
	{
		char fsmName[64];

		long numOfMeshes;
		long numOfMaterials;
		long numOfAnimations;
		long numOfFrames;

		std::vector<tMD3MaterialInfo> pMaterials;
		std::vector<tMD3Mesh> pMeshes;
		std::vector<tMD3AnimationInfo> pAnimations; 
		


		tMD3Frame *pFrames; 

		// moved to FRONTEND::CMD3Link 
		//int currentAnim;					
		//int currentFrame;					
		//int nextFrame;						
		//float t;							
		//float lastTime;		

		// moved to MD3MODEL::TMD3Modeldata
		
		int numOfTags;						
		struct tMD3Tag *pTags;			
		
		//tMD3Model **pLinks;	

		// frontend class: FRONTEND::CMD3LinkedModel

		//void Render();
	};



	struct tMD3DomainData
	{
		float boundingRadius;
		GENERAL::CVector3f bbMin;
		GENERAL::CVector3f bbMax;

		int numOfTags;						
		struct tMD3Tag *pTags;
	};

	/*
	class CLoadMD3
	{
	public:

		CLoadMD3();								

		bool ImportMD3(tMD3Model *pModel, char *strFileName);
		bool LoadSkin(tMD3Model *pModel, char* strSkin);
		bool LoadShader(tMD3Model *pModel, char* strShader);

	private:

		void ReadMD3Data(tMD3Model *pModel);
		void ConvertDataStructures(tMD3Model *pModel, tMD3MeshInfo meshHeader);
		void CleanUp();
		
		FILE *m_FilePointer;

		tMD3Header				m_Header;			

		tMD3Skin				*m_pSkins;			
		tMD3TexCoord			*m_pTexCoords;		
		tMD3Face				*m_pTriangles;		
		tMD3Triangle			*m_pVertices;		
		tMD3Frame				*m_pFrames;			
	}; */

	class CLoadMD3Link
	{
	public:

		CLoadMD3Link();								

		//bool LoadSkin(tMD3Model *pModel, char* strSkin);
		//bool LoadShader(tMD3Model *pModel, char* strShader);

		bool ReadMD3LinkData(tMD3Link *pLink, char *strFileName);

	private:

		
		
		void ConvertDataStructures(tMD3Link *pLink, tMD3MeshInfo meshHeader);
		void LoadStateMachineFile(char *filename);

		
		
	//	FILE *m_FilePointer;

		tMD3Header				m_Header;			

		tMD3Shader				*m_pShaders;			
		tMD3TexCoord			*m_pTexCoords;		
		tMD3Triangle			*m_pTriangles;		
		tMD3Vertex				*m_pVertices;		
		tMD3Frame				*m_pFrames;			
	};

}
#endif