#include "quakebspfile.h"

namespace GENERAL
{
//#define LOAD_LUMP(lump,restyp)
	template <eBSPLumps lump, typename resTyp, int additionalSpace>
		inline resTyp *Load(FILE *file, tBSPLump *lumps, int &size)
		{
			resTyp *res;
			size = lumps[lump].length/sizeof(resTyp);
			res = (resTyp*) malloc( (sizeof(resTyp)+additionalSpace) * size);
			fseek(file,lumps[lump].offset,SEEK_SET);
			fread(res,sizeof(resTyp),size,file);
			size+=additionalSpace;
			return res;
		}

	CQuakeBSPFile::CQuakeBSPFile()
	{
	}

	bool CQuakeBSPFile::Open(char *_fileName)
	{
		file = GENERAL::morfOpenFile(_fileName,"rb");

		if (file!=NULL)
		{
			fread(&header, 1, sizeof(GENERAL::tBSPHeader), file);
			fread(&lumps, GENERAL::kBSPMaxLumps, sizeof(GENERAL::tBSPLump), file);

			return true;
		}

		return false;
	}

	char *CQuakeBSPFile::GetEntities(int &size)
	{
		char *res=Load<kBSPEntities,char,1>(file,lumps,size);
		res[size-1]='\0';
		return res;
	}

	tBSPTexture	*CQuakeBSPFile::GetTextures(int &size)
	{
		return Load<kBSPTextures,tBSPTexture,0>(file,lumps,size);
	}

	tBSPPlane *CQuakeBSPFile::GetPlanes(int &size)
	{
		tBSPPlane *res = Load<kBSPPlanes,tBSPPlane,0>(file,lumps,size);

		for(int i = 0; i < size; i++)
		{
			float temp = res[i].vNormal.y;
			res[i].vNormal.y = res[i].vNormal.z;
			res[i].vNormal.z = -temp;
		}

		return res;
	}

	tBSPNode *CQuakeBSPFile::GetNodes(int &size)
	{
		return Load<kBSPNodes,tBSPNode,0>(file,lumps,size);
	}

	tBSPLeaf *CQuakeBSPFile::GetLeafs(int &size)
	{
		tBSPLeaf *res = Load<kBSPLeafs,tBSPLeaf,0>(file,lumps,size);
		for(int i = 0; i < size; i++)
		{
			int temp = res[i].min.y;
			res[i].min.y = res[i].min.z;
			res[i].min.z = -temp;

			temp = res[i].max.y;
			res[i].max.y = res[i].max.z;
			res[i].max.z = -temp;
		}	
		return res;
	}

	int *CQuakeBSPFile::GetLeafFaces(int &size)
	{
		return Load<kBSPLeafFaces,int,0>(file,lumps,size);
	}

	int *CQuakeBSPFile::GetLeafBrushes(int &size)
	{
		return Load<kBSPLeafBrushes,int,0>(file,lumps,size);
	}

	tBSPBrush *CQuakeBSPFile::GetBrushes(int &size)
	{
		return Load<kBSPBrushes,tBSPBrush,0>(file,lumps,size);
	}

	tBSPBrushSide *CQuakeBSPFile::GetBrushSides(int &size)
	{
		return Load<kBSPBrushSides,tBSPBrushSide,0>(file,lumps,size);
	}

	tBSPVertex *CQuakeBSPFile::GetVertices(int &size)
	{
		tBSPVertex *res = Load<kBSPVertices,tBSPVertex,0>(file,lumps,size);
		for(int i = 0; i < size; i++)
		{
			float temp = res[i].vPosition.y;
			res[i].vPosition.y = res[i].vPosition.z;
			res[i].vPosition.z = -temp;

			temp = res[i].vNormal.y;
			res[i].vNormal.y = res[i].vNormal.z;
			res[i].vNormal.z = -temp;
		}	

		return res;
	}

	int *CQuakeBSPFile::GetMeshIndices(int &size)
	{
		return Load<kBSPMeshIndices,int,0>(file,lumps,size);
	}

	tBSPShader *CQuakeBSPFile::GetShaders(int &size)
	{
		return Load<kBSPShaders,tBSPShader,0>(file,lumps,size);
	}

	tBSPFace *CQuakeBSPFile::GetFaces(int &size)
	{
		return Load<kBSPFaces,tBSPFace,0>(file,lumps,size);
	}

	tBSPLightmap *CQuakeBSPFile::GetLightMaps(int &size)
	{
		return Load<kBSPLightmaps,tBSPLightmap,0>(file,lumps,size);
	}

	void CQuakeBSPFile::GetVisData(tBSPVisData	&visData)
	{
		fseek(file, lumps[kBSPVisData].offset, SEEK_SET);

		if(lumps[kBSPVisData].length>0) 
		{
			fread(&(visData.numOfClusters),	 1, sizeof(int), file);
			fread(&(visData.bytesPerCluster), 1, sizeof(int), file);

			int size = visData.numOfClusters * visData.bytesPerCluster;
			visData.pBitsets = (UBYTE*)malloc(size*sizeof(UBYTE));
	
			fread(visData.pBitsets, 1, size*sizeof(UBYTE), file);
		}
		else
		{
			visData.numOfClusters=0;
			visData.bytesPerCluster=0;
			visData.pBitsets = NULL;
		}
	}

    void CQuakeBSPFile::GetEntities(TEntities &_entities)
	{
		int len;
		char *ents = GetEntities(len);
		char *wordpos1, *wordpos2;

		GENERAL::dprintf("%s",ents);

		bool iskey = true;
		char *curkey, *curclassname;
	
		curkey = 0;

		TEntity newentries;

		char *token = strtok(ents, "}");
		while (token!=NULL)
		{

			newentries.clear();
			curclassname = 0;
			wordpos2 = token-1;
			while (wordpos2!=0)
			{
				wordpos1 = strstr(wordpos2+1,"\"");
				if (wordpos1) wordpos2 = strstr(wordpos1+1,"\""); else break;
				if (wordpos2)
				{
					// found word
					if (wordpos2>(wordpos1+1)) 
					{
						*wordpos2='\0';
						if (iskey)
						{
							curkey = wordpos1+1;
							iskey = false;
						}
						else
						{
							if (curkey)
							{
								newentries[curkey]=wordpos1+1;
								iskey = true;
								if (stricmp(curkey,"classname")==0) 
								{
									curclassname = wordpos1+1;
								}
							}
							else
							{
								dprintf("should not be reached!\n");
							}
						}
					}
				}
			} // end of block
			if (!curclassname)
			{
				GENERAL::dprintf("[ WARN ] CQuakeBSPFile::GetEntities: syntax Error, missing classname\n");
			}
			else
			{
				GENERAL::dprintf("[ INFO ] CQuakeBSPFile::GetEntities: creating entity class %s\n", curclassname);
				_entities.insert(TEntities::value_type(curclassname,newentries));
			}
			token = strtok(NULL, "}");
		}

		free(ents);
        
	}

	CQuakeBSPFile::~CQuakeBSPFile()
	{
		fclose(file);
	}
}