#include "tcpnetwork.h"
#include <trace.h>

namespace NETWORK
{
	CTCPNetwork::~CTCPNetwork()
	{
		Done();
	}

	void CTCPNetwork::Init(bool listen)
	{
		WSADATA wsaData;
		if (WSAStartup(MAKEWORD(2,0),&wsaData))
			NETTHROW("Could not initialize WSA");

		if ((hNetworkLock = CreateMutex(NULL,FALSE,NULL))==NULL)
		{
			NETTHROW("Cannot create global synchronization object");
		}

		if ((hOutgoingSemaphore = CreateSemaphore(NULL,0,1000,NULL))==NULL)
		{
			NETTHROW("Cannot create Sender Semaphore");
		}

		hSenderThread = (HANDLE)_beginthread((void (*)(void*))staticSenderThread,0,this);
		if (hSenderThread==(HANDLE)-1L)
		{
			NETTHROW("Cannot create sender thread");
		}

		if (listen) 
		{
			serverSocket = new CTCPServerSocket(this);
			serverSocket->Listen(
				(char*)CONFIG::g_config.ConfString["ServerListenAddress"].data(),
				CONFIG::g_config.ConfInt["ServerListenPort"],
				CONFIG::g_config.ConfInt["ServerListenBacklog"] );
		}
	}

	void CTCPNetwork::Done()
	{
		if (serverSocket)
			delete serverSocket;

		bTerminateSender = true;
		
		if (ReleaseSemaphore(hOutgoingSemaphore,1,NULL)==0)
		{
			NETTHROW("Could not release Semaphore");
		}
		if (WaitForSingleObject(hSenderThread,2000)==WAIT_TIMEOUT)
		{
			NETTHROW("Listen Thread did not finish within 2 seconds -> releasing Resources anyway");
		}

		
		Lock();
			HANDLE *handleList = (HANDLE*)malloc(sizeof(HANDLE)*connectedTo.size());
			int i=0;
			TClientInfoList::iterator it;
			for (it=connectedTo.begin(),i=0; it!=connectedTo.end();it++,i++)
			{
				handleList[i]=((CTCPClientInfo*)&(*it))->GetSocket()->GetListenThread();
				((CTCPClientInfo*)&(*it))->Shutdown();
			}
		Unlock();
		if (WaitForMultipleObjects(i,handleList,true,10000)==WAIT_TIMEOUT)
		{
			NETTHROW("Client Listen Thread did not finish within 10 seconds -> releasing Resources anyway");
		}
		free(handleList);

		connectedTo.clear();

		WSACleanup();
	}

	void CTCPNetwork::Connect()
	{
		NETWORK::CTCPClientSocket *cs = new CTCPClientSocket(this);
		cs->ConnectTo(
			(char*)CONFIG::g_config.ConfString["ServerAddress"].data(),
			CONFIG::g_config.ConfInt["ServerListenPort"]);
		Lock();
			connectedTo.push_back(cs->GetClientInfo());
		Unlock();

		return;
	}

	void CTCPNetwork::PushObject(sptr<GENERAL::CSerializeableObject> o, PClientInfoList destination)
	{
		if (destination==NULL) return;
		Lock();
			destination->GetRef();
			outgoingObject.push_back(CTCPNetwork::TOutgoingObject(o,destination));

			o->SetLastPush(APPLICATION::CApplication::GetApplication()->GetTicker().getTick());
#ifdef _DEBUG
//			o->Print(stdout);
//			fprintf(stdout,"outgoing Queues size = %d\n",outgoingObject.size());
//			fflush(stdout);
#endif
			LONG qs;

			if (ReleaseSemaphore(hOutgoingSemaphore,1,&qs)==0)
			{
				Unlock();
				NETTHROW(GENERAL::GetLastErrorMessage());
			}
//			fprintf(stdout,"Push qs: %d\n",qs);
		Unlock();
	}

	void CTCPNetwork::Lock(DWORD time)
	{
		if (WaitForSingleObject(hNetworkLock,time)!=WAIT_OBJECT_0)
		{
			NETTHROW("cannot obtain lock to global lock");
		}
	}

	void CTCPNetwork::Unlock()
	{
		if (ReleaseMutex(hNetworkLock)==0)
			NETTHROW("cannot release lock to global lock");
			
	}

	void CTCPNetwork::staticSenderThread(CTCPNetwork *self)
	{
		self->SenderThread();
	}

	void CTCPNetwork::SenderThread()
	{
		while (true)
		{
			if ((WaitForSingleObject(hOutgoingSemaphore,INFINITE))!=WAIT_FAILED)
			{
//				LONG queueSize;
//				ReleaseSemaphore(hOutgoingSemaphore,1,&queueSize);
//				WaitForSingleObject(hOutgoingSemaphore,INFINITE);
//				fprintf(stdout,"Sender qs: %d\n",queueSize);
				if (bTerminateSender) 
					_endthread();

				TClientInfoList::iterator it;
				TClientInfoList cinfo;

				// Synchronized part
				GENERAL::CBuffer *mb=NULL;

				CTCPNetwork::TOutgoingObject *o=NULL;
				Lock();

				while (!outgoingObject.empty())
				{
					try
					{
						o = &outgoingObject.front();
						mb = GENERAL::Serialize(o->so);

						PClientInfoList cil = o->destination;
						for (it = cil->begin(); it != cil->end(); it++)
						{	
							WORD size = (WORD)mb->GetPos();
							try {
								((CTCPClientInfo*)&(*it))->GetSocket()->DoSend((char*)&size,sizeof(size),0);
								((CTCPClientInfo*)&(*it))->GetSocket()->DoSend((char*)mb->GetData(),size,0);
							}
							catch (exception &)
							{}
						}
//1						o->so->ReleaseRef();
//1						cil->ReleaseRef();

						outgoingObject.pop_front();
						o=NULL;

						delete mb;
						mb=NULL;
					} 
					catch (exception &)
					{
						if (mb) delete mb;
//1						if (o) o->so->ReleaseRef();
						domain->pushNetworkEvent(sptr<CNetworkEvent>(new CNetworkEvent(CNetworkEvent::TEventTyp::CRITICALERROR,GENERAL::sptr<CClientInfo>(NULL))));
						Unlock();
						_endthread();
					}
					
					DWORD res = WaitForSingleObject(hOutgoingSemaphore,0);
					if (res!=WAIT_OBJECT_0) break;
				}
				Unlock();

			}			
		}
	}

} // namespace NETWORK
