#ifndef TCPSOCKET_H
#define TCPSOCKET_H

#pragma comment (lib,"ws2_32")

#include <morfbase.h>
#include <windows.h>
#include <winsock.h>

namespace NETWORK
{
	class CTCPClientSocket;
	class CTCPNetwork;

/**********************************************************************************/
/* CTCPClientInfo                                                                 */
/**********************************************************************************/
	class CTCPClientInfo : public CClientInfo
	{
	protected:
		CTCPClientSocket *clientsocket;
	public:
		inline CTCPClientInfo() : CClientInfo() {};
		virtual ~CTCPClientInfo();

		inline void SetSocket(CTCPClientSocket *cs) {clientsocket = cs;};
		inline CTCPClientSocket *GetSocket() {return clientsocket;};
		virtual void Shutdown();
		char *GetClientInfoString();
	}; // class CTCPClientInfo

/**********************************************************************************/
/* CTCPSocket                                                                     */
/**********************************************************************************/
	class CTCPSocket
	{
	protected:
		/// Socket file descroptor
		SOCKET socketfd;

		/// Structure containing address data
		sockaddr_in	addr;

		/// pointer to the corresponding Network;
		CTCPNetwork *network;

	public:
		inline CTCPSocket(CTCPNetwork *_network) : socketfd(0),network(_network) {};
		virtual ~CTCPSocket();

		inline sockaddr *GetpAddr() { return (struct sockaddr*)&addr; };
		inline SOCKET *GetpSocketfd() { return &socketfd; };
		inline SOCKET GetSocketfd() { return socketfd; };
		inline void SetSocketfd(SOCKET _socketfd) { socketfd=_socketfd; };

		//* Shuts down the Socket, this will lead to the termination of the corresponding thread
		void Shutdown();

		/** Receives data from a Socket, this function garanties the reception of len
		*	bytes, or an exception will occure
		*	\param data buffer where the data will be stored
		*	\param len length of the data to be read
		*	\param flags will be passed to recv (see socket dokumentation)
		*/
		int DoRecv(char *data,int len,int flags);

		/** Send data to Socket, this function garanties that len
		*	bytes will bee sent or an exception will occure
		*	\param data buffer where the data is be stored
		*	\param len length of the data to be sent
		*	\param flags will be passed to send (see socket dokumentation)
		*/
		int DoSend(char *data,int len,int flags);
	};

/**********************************************************************************/
/* CTCPClientSocket                                                               */
/**********************************************************************************/
	class CTCPClientSocket : public CTCPSocket
	{
	private:
		static void staticListenThread(CTCPClientSocket *self);

	protected:
		GENERAL::sptr<CTCPClientInfo> clientInfo;
		PClientInfoList clientInfoList;
		HANDLE listenThread;

		virtual void ListenThread();
	public:
		inline CTCPClientSocket(CTCPNetwork *_network) : CTCPSocket(_network), listenThread((HANDLE)-1L) 
		{
			clientInfoList = new TClientInfoList();
			clientInfo=new CTCPClientInfo();
			clientInfo->SetSocket(this); 
			clientInfoList->push_back(GENERAL::sptr<CClientInfo>(clientInfo));
		};
		virtual ~CTCPClientSocket();

		virtual void ConnectTo(char *ip, int port);

		inline HANDLE GetListenThread() { return listenThread; };
		inline sptr<CTCPClientInfo> &GetClientInfo() { return clientInfo; };
		inline TClientInfoList &GetClientInfoList() { return *clientInfoList; };

		virtual void StartListenThread();

	}; // class CTCPClientSocket

/**********************************************************************************/
/* CTCPServerSocket                                                               */
/**********************************************************************************/
	class CTCPServerSocket : public CTCPSocket
	{
	private:
		static void staticListenThread(CTCPServerSocket *self);

	protected:
		HANDLE listenThread;

		virtual void ListenThread();
	public:
		inline CTCPServerSocket(CTCPNetwork *_network) : CTCPSocket(_network), listenThread((HANDLE)-1L) {};
		virtual ~CTCPServerSocket();

		virtual void Listen(char *ip, int port, int backlog);
		virtual void StartListenThread();
	}; // class CTCPServerSocket

} // namespace NETWORK

#endif