// FontTool.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include <windows.h>

struct THeader
{
	char id[3];
	long firstchar;
	long charcount;
	long format;  
			// 0 ... Bitmap
			// 1 ... 2 Bit Grayscale
			// 2 ... 4 Bit Grayscale
			// 3 ... 8 Bit Grayscale
	long alignment;
	long height;
	long ascent;
	long descent;
	long charofs;
};

struct TGlyphData
{
	GLYPHMETRICS metrics;
	int datasize;
	char *data;
};

void PrintGlyphData(TGlyphData &glyph, UINT format)
{
	int x,y;
	int offset=-1;
	char c;

	printf("(%d,%d) (%d,%d) (%d,%d)\n",glyph.metrics.gmBlackBoxX,glyph.metrics.gmBlackBoxY,glyph.metrics.gmptGlyphOrigin.x,glyph.metrics.gmptGlyphOrigin.y,glyph.metrics.gmCellIncX,glyph.metrics.gmCellIncY);

	if (glyph.datasize==0)
		return;
	for (y=0; y<(int)glyph.metrics.gmBlackBoxY;y++)
	{
		printf("#");
		for(x=0; x<(int)glyph.metrics.gmBlackBoxX; x++)
		{
			if (format==GGO_BITMAP)
			{
				if (x%8==0)
				{
					offset++;
					c=glyph.data[offset];
				}

				printf(c&0x80?"1":" ");
				c<<=1;
			} else if (format==GGO_GRAY8_BITMAP)
			{
				offset++;
				c=glyph.data[offset];
				if (c==0)
				{
					printf("  ");
				} else {
					printf("%2X",(unsigned int)(c&0xFF));
				}
			}
		}
		while (offset%4!=3)
		{
			offset++;
		}
		printf("#\n");
	}

	fflush(stdout);
}

void GetGlyph(HDC hdc, int c,TGlyphData &glyph,UINT format)
{
	MAT2 m;
	m.eM11.fract=0;	m.eM11.value=1;
	m.eM22.fract=0;	m.eM22.value=1;
	m.eM12.fract=0;	m.eM12.value=0;
	m.eM21.fract=0;	m.eM21.value=0;

	char *data;

	glyph.datasize = GetGlyphOutline(hdc,c,format,&glyph.metrics,0,NULL,&m);
	if (glyph.datasize!=-1)
	{
		data = (char*)malloc(glyph.datasize);
		glyph.datasize = GetGlyphOutline(hdc,c,format,&glyph.metrics,glyph.datasize,data,&m);
	}

	int linesize=glyph.datasize/glyph.metrics.gmBlackBoxY;
	glyph.data=(char*)malloc(glyph.datasize);
	int ofs=0;
	for (int y=0; y<(int)glyph.metrics.gmBlackBoxY;y++)
	{
		memcpy(glyph.data+y*linesize,data+(glyph.metrics.gmBlackBoxY-(y+1))*linesize,linesize);
		ofs=y*linesize;
		for (int x=0; x<(int)glyph.metrics.gmBlackBoxX;x++)
		{
			if (format==GGO_GRAY8_BITMAP)
			{
				if (glyph.data[ofs]!=0)
					glyph.data[ofs]=(glyph.data[ofs]*4-1)&0xFF;
				ofs++;
			}
		}
	}
	glyph.metrics.gmptGlyphOrigin.x=-glyph.metrics.gmptGlyphOrigin.x;
	glyph.metrics.gmptGlyphOrigin.y=glyph.metrics.gmBlackBoxY-glyph.metrics.gmptGlyphOrigin.y;
	free(data);
}

int _tmain(int argc, _TCHAR* argv[])
{
	HFONT hFont = CreateFont( 
		-30,							
		0,								
		0,								
		0,								
		FW_REGULAR,						
		FALSE,							
		FALSE,							
		FALSE,							
		ANSI_CHARSET,					
		OUT_TT_PRECIS,					
		CLIP_DEFAULT_PRECIS,			
		ANTIALIASED_QUALITY,			
		FF_DONTCARE|DEFAULT_PITCH,
		"1st Sortie");

	TEXTMETRIC metrics;
	HDC hdc=GetDC(NULL);
	HANDLE old=SelectObject(hdc,hFont);
	GetTextMetrics(hdc,&metrics);

	TGlyphData data[256-32];

	FILE *f=fopen("1st Sortie30.fnt","wb");
	
	THeader header;
	header.alignment=4;
	header.charcount=256-32;
	header.firstchar=32;
	header.format=0;
	header.height=metrics.tmHeight;
	header.ascent=metrics.tmAscent;
	header.descent=metrics.tmDescent;
	header.id[0]='F';
	header.id[1]='N';
	header.id[2]='T';
	header.charofs=sizeof(header);
	fwrite(&header,sizeof(header),1,f);

	UINT format;
	switch (header.format)
	{
	case 0: format = GGO_BITMAP; break;
	case 1: format = GGO_GRAY2_BITMAP; break;
	case 2: format = GGO_GRAY4_BITMAP; break;
	case 3: format = GGO_GRAY8_BITMAP; break;
	}

	for (int i=0;i<header.charcount;i++)
	{
		GetGlyph(hdc,header.firstchar+i,data[i],format);
		printf("%c\n",i+header.firstchar);
		PrintGlyphData(data[i],format);
//		fgetc(stdin);
		fwrite(&data[i].metrics,sizeof(data[i].metrics),1,f);
		fwrite(&data[i].datasize,sizeof(data[i].datasize),1,f);
		fwrite(data[i].data,1,data[i].datasize,f);
	//	PrintGlyphData(data[0]);
	}

	SelectObject(hdc,old);

	fclose(f);
	fgetc(stdin);

	return 0;
}

